//
// Tests on additions and various multiplications of FlOpCounters.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke.
//


// Tests on well-formed additions.
assertFlOpCount(flopctr(3) + 5, 8, %eps, [1,0,0,2,1,0]);
assertFlOpCount([3,6,-9] + flopctr([9,0,1]), [12,6,-8], %eps, [3,0,0,6,3,0]);
assertFlOpCount(flopctr([5;-3;1;-10]) + flopctr([1;-4;-8;0]), [6;-7;-7;-10], %eps, [4,0,0,8,4,0]);
assertFlOpCount(flopctr([1,3,5 ; -9,10,0]) + [2,-8,0; 3,4,2], [3,-5,5 ; -6,14,2], %eps, [6,0,0,12,6,0]);

// Tests on well-formed multiplications.
assertFlOpCount(flopctr(9) * flopctr(3), 27, %eps, [0,1,0,2,1,0]);
assertFlOpCount([3;0;-1] * flopctr(2), [6;0;-2], %eps, [0,3,0,6,3,0]);
assertFlOpCount(flopctr(5) * [6,0,-2,3], [30,0,-10,15], %eps, [0,4,0,8,4,0]);
assertFlOpCount(flopctr([9,-1,5]) * [8;2;0], 70, %eps, [2,3,0,6,1,0]);
assertFlOpCount([1;5] * flopctr([9,-2]), [9,-2 ; 45,-10], %eps, [0,4,0,8,4,0]);
assertFlOpCount([2,3,-5] * flopctr([1,0 ; 9,-3 ; -6,2]), [59,-19], %eps, [4,6,0,12,2,0]);
assertFlOpCount(flopctr([2,5 ; 0,-2 ; -9,1]) * [3;6], [36;-12;-21], %eps, [3,6,0,12,3,0]);
assertFlOpCount(flopctr([3,4,5 ; 8,-9,0]) * flopctr([1,0 ; 3,-1; 2,9]), [25,41 ; -19,9], %eps, [8,12,0,24,4,0]);

// Tests on well-formed element-wise multiplications.
assertFlOpCount(flopctr(3) .* 5, 15, %eps, [0,1,0,2,1,0]);
assertFlOpCount([3,6,-9] .* flopctr([9,0,1]), [27,0,-9], %eps, [0,3,0,6,3,0]);
assertFlOpCount(flopctr([5;-3;1;-10]) .* flopctr([1;-4;-8;0]), [5;12;-8;0], %eps, [0,4,0,8,4,0]);
assertFlOpCount(flopctr([1,3,5 ; -9,10,0]) .* [2,-8,0; 3,4,2], [2,-24,0 ; -27,40,0], %eps, [0,6,0,12,6,0]);

// Tests on auto-reformed additions.
assertFlOpCount(3 + flopctr([-9,2,0]), [-6,5,3], %eps, [3,4,0,14,7,0]);
assertFlOpCount(flopctr([-9;2;0]) + 3, [-6;5;3], %eps, [3,6,0,18,9,0]);
assertFlOpCount(3 + flopctr([2,0,9 ; -2,1,5 ; 10,-2,-7]), [5,3,12 ; 1,4,8 ; 13,1,-4], %eps, [9,12,0,42,21,0]);

// Tests on auto-reformed element-wise multiplications.
assertFlOpCount(3 .* flopctr([-9,2,0]), [-27,6,0], %eps, [0,7,0,14,7,0]);
assertFlOpCount(flopctr([-9;2;0]) .* 3, [-27;6;0], %eps, [0,9,0,18,9,0]);
assertFlOpCount(3 .* flopctr([2,0,9 ; -2,1,5 ; 10,-2,-7]), [6,0,27 ; -6,3,15 ; 30,-6,-21], %eps, [0,21,0,42,21,0]);

// Tests on auto-reformed multiplications.
assertFlOpCount(flopctr([4,-9,0]) * 2, [8,-18,0], %eps, [0,7,0,14,7,0]);
assertFlOpCount(flopctr(2) * [4;-9;0], [8;-18;0], %eps, [0,9,0,18,9,0]);
assertFlOpCount((-2) * flopctr([1,0 ; 9,-3 ; -6,2]), [-2,0 ; -18,6 ; 12,-4], %eps, [0,15,0,30,15,0]);
assertFlOpCount([1,0 ; 9,-3 ; -6,2] * flopctr(-2), [-2,0 ; -18,6 ; 12,-4], %eps, [0,15,0,30,15,0]);

printf("    Adds and mults : all unit tests passed.\n");


// End of script.
