//
// Tests on exponentials and monomials.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Tests on exponentials.
assertFlOpCount(%e ^ flopctr(3.2), 24.5325302, 1e-7, [0,0,0,2,1,1]);
assertFlOpCount(5 ^ flopctr(3.2), 172.4662077, 1e-7, [0,0,0,2,1,2]);
assertFlOpCount([4,%e,6] ^ flopctr(2.5), [32,12.182494,88.181631], 1e-6, [0,4,0,14,7,5]);
assertFlOpCount([7,2 ; 3,1] .^ flopctr(3), [343,8 ; 27,1], %eps, [0,6,0,20,10,8]);
assertFlOpCount(exp(flopctr([-1.4 ; 5 ; 2.2])), [0.2465966963 ; 148.4131591 ; 9.025013499], 1e-6, [0,6,0,18,9,3]);

// Tests on monomials.
assertFlOpCount(flopctr(3) ^ 2, 9, %eps, [0,1,0,2,1,0]);
assertFlOpCount(flopctr(3) ^ 4, 81, %eps, [0,2,0,4,2,0]);
assertFlOpCount(flopctr(3) ^ 3, 27, %eps, [0,2,0,4,2,0]);
assertFlOpCount(flopctr(3) ^ 7, 2187, %eps, [0,4,0,8,4,0]);
assertFlOpCount(flopctr(4) ^ 3.5, 128, %eps, [0,3,0,8,4,1]);

// For element-wise monomials, the procedure uses insertions, for which exact FlOp counts can only be tracked with difficulty.
// Hence, FlOp counts will not be taken in account, only the results.
//
assert(and(FlOpCtr_GetValue(flopctr(3) .^ 2) == 9));
assert(and(FlOpCtr_GetValue(flopctr(3) .^ 4) == 81));
assert(and(FlOpCtr_GetValue(flopctr(3) .^ 3) == 27));
assert(and(FlOpCtr_GetValue(flopctr(3) .^ 7) == 2187));
assert(and(FlOpCtr_GetValue(flopctr(4) .^ 3.5) == 128));
assert(and(abs(FlOpCtr_GetValue(flopctr([2,3,4]) .^ 5.2) - [36.75834736,302.7126183,1351.176101]) < 1e-6));
assert(and(abs(FlOpCtr_GetValue(flopctr(3) .^ [2;5.5;7.2]) - [9;420.8883462;2724.413565]) < 1e-6));
assert(and(abs(FlOpCtr_GetValue(flopctr([2,4,0 ; 1.2,2.3,0.6]) .^ [8.3,3,10 ; 4.3,6,5.3]) - ...
	[315.1729698,64,0 ; 2.190177727,148.0358890,0.066711641]) < 1e-7));
assert(and(abs(FlOpCtr_GetValue(sqrt(flopctr([4,16 ; 5,100]))) - [2,4 ; sqrt(5),10]) < %eps));


FLOPCTR_Reset();

printf("    Exps and monomials : all unit tests passed.\n");


// End of script.
