//
// Tests on FlOpCounter matrix manipulations (extraction, concatenation and insertion).
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Tests on extraction.
f = flopctr(5);
assertFlOpCount(f(1), 5, %eps, [0,0,0,1,1,0]);
assertFlOpCount(f($), 5, %eps, [0,0,0,1,1,0]);
assertFlOpCount(f(:), 5, %eps, [0,0,0,1,1,0]);
assertFlOpCount(f(1,1), 5, %eps, [0,0,0,1,1,0]);
assertFlOpCount(f($,:), 5, %eps, [0,0,0,1,1,0]);

f = flopctr([2;5;9;1;0;-2]);
assertFlOpCount(f(2), 5, %eps, [0,0,0,1,1,0]);
assertFlOpCount(f([3,5]), [9;0], %eps, [0,0,0,2,2,0]);
assertFlOpCount(f(4:$), [1;0;-2], %eps, [0,0,0,3,3,0]);
assertFlOpCount(f(:), [2;5;9;1;0;-2], %eps, [0,0,0,6,6,0]);
assertFlOpCount(f(4,1), 1, %eps, [0,0,0,1,1,0]);
assertFlOpCount(f([$,5,2], $), [-2;0;5], %eps, [0,0,0,3,3,0]);
assertFlOpCount(f(:,:), [2;5;9;1;0;-2], %eps, [0,0,0,6,6,0]);

f = flopctr([2,5,9,1,0,-2]);
assertFlOpCount(f(2), 5, %eps, [0,0,0,1,1,0]);
assertFlOpCount(f([3,5]), [9,0], %eps, [0,0,0,2,2,0]);
assertFlOpCount(f(4:$), [1,0,-2], %eps, [0,0,0,3,3,0]);
assertFlOpCount(f(:), [2;5;9;1;0;-2], %eps, [0,0,0,6,6,0]);
assertFlOpCount(f(1,4), 1, %eps, [0,0,0,1,1,0]);
assertFlOpCount(f($, [$,5,2]), [-2,0,5], %eps, [0,0,0,3,3,0]);
assertFlOpCount(f(:,:), [2,5,9,1,0,-2], %eps, [0,0,0,6,6,0]);

f = flopctr([0,1,2,3,4 ; 5,6,7,8,9 ; 10,11,12,13,14]);
assertFlOpCount(f(6), 11, %eps, [0,0,0,1,1,0]);
assertFlOpCount(f([9,2,$,4]), [12;5;14;1], %eps, [0,0,0,4,4,0]);
assertFlOpCount(f(:), [0;5;10;1;6;11;2;7;12;3;8;13;4;9;14], %eps, [0,0,0,15,15,0]);
assertFlOpCount(f(2,4), 8, %eps, [0,0,0,1,1,0]);
assertFlOpCount(f([2,3], [$,2,3]), [9,6,7 ; 14,11,12], %eps, [0,0,0,6,6,0]);
assertFlOpCount(f(:, [2,4]), [1,3 ; 6,8 ; 11,13], %eps, [0,0,0,6,6,0]);
assertFlOpCount(f($, :), [10,11,12,13,14], %eps, [0,0,0,5,5,0]);
assertFlOpCount(f(:, :), [0,1,2,3,4 ; 5,6,7,8,9 ; 10,11,12,13,14], %eps, [0,0,0,15,15,0]);


// Tests on horizontal concatenation.
assertFlOpCount([flopctr(5), flopctr(0)], [5,0], %eps, [0,0,0,2,2,0]);
assertFlOpCount([flopctr([1,2,3]), flopctr([4,5])], [1,2,3,4,5], %eps, [0,0,0,5,5,0]);
assertFlOpCount([flopctr([1;2;3]), flopctr([4;5;6])], [1,4 ; 2,5 ; 3,6], %eps, [0,0,0,6,6,0]);
assertFlOpCount([flopctr([1,2 ; 3,4]), flopctr([4,5,6 ; 7,8,9])], [1,2,4,5,6 ; 3,4,7,8,9], %eps, [0,0,0,10,10,0]);

// Tests on vertical concatenation.
assertFlOpCount([flopctr(6) ; flopctr(3)], [6;3], %eps, [0,0,0,2,2,0]);
assertFlOpCount([flopctr([1;2;3]) ; flopctr([4;5])], [1;2;3;4;5], %eps, [0,0,0,5,5,0]);
assertFlOpCount([flopctr([1,2,3]) ; flopctr([4,5,6])], [1,2,3 ; 4,5,6], %eps, [0,0,0,6,6,0]);
assertFlOpCount([flopctr([1,2 ; 3,4]) ; flopctr([5,6 ; 7,8 ; 9,10])], [1,2 ; 3,4 ; 5,6 ; 7,8 ; 9,10], %eps, ...
		[0,0,0,10,10,0]);

// Tests on insertion.  Since insertion is based on a complex composition of additions, products, element-wise products and
// concatenations, we won't care about FlOp counts here, we'll only make sure insertion works.
f = flopctr(6);
f(1) = 3;
assertFlOpCount(f, 3, %eps, [0,0,0,1,1,0]);
//assert(and(FlOpCtr_GetValue(f) == 3));
f($) = 8;
assertFlOpCount(f, 8, %eps, [0,0,0,1,1,0]);
//assert(and(FlOpCtr_GetValue(f) == 8));
f(4) = 12;
assertFlOpCount(f, [8;0;0;12], %eps, [0,0,0,1,1,0]);
//assert(and(FlOpCtr_GetValue(f) == [8;0;0;12]));
f(3,1) = 5;
assertFlOpCount(f, [8;0;5;12], %eps, [0,0,0,1,1,0]);
//assert(and(FlOpCtr_GetValue(f) == [8;0;5;12]));
f(7,1) = 7;
assertFlOpCount(f, [8;0;5;12;0;0;7], %eps, [0,0,0,1,1,0]);
//assert(and(FlOpCtr_GetValue(f) == [8;0;5;12;0;0;7]));
f(4,3) = 10;
assertFlOpCount(f, ...
	[ ...
		 8,0, 0 ; ...
		 0,0, 0 ; ...
		 5,0, 0 ; ...
		12,0,10 ; ...
		 0,0, 0 ; ...
		 0,0, 0 ; ...
		 7,0, 0   ...
	], %eps, [0,0,0,1,1,0]);
//assert(and(FlOpCtr_GetValue(f) == [8,0,0 ; 0,0,0 ; 5,0,0 ; 12,0,10 ; 0,0,0 ; 0,0,0 ; 7,0,0]));
f(8,4) = 100;
assertFlOpCount(f, ...
	[ ...
		 8, 0, 0,   0 ; ...
		 0, 0, 0,   0 ; ...
		 5, 0, 0,   0 ; ...
		12, 0,10,   0 ; ...
		 0, 0, 0,   0 ; ...
		 0, 0, 0,   0 ; ...
		 7, 0, 0,   0 ; ...
         0, 0, 0, 100   ...
	], %eps, [0,0,0,1,1,0]);
//assert(and(FlOpCtr_GetValue(f) == [8,0,0,0 ; 0,0,0,0 ; 5,0,0,0 ; 12,0,10,0 ; 0,0,0,0 ; 0,0,0,0 ; 7,0,0,0 ; 0,0,0,100]));
f($,[2,3]) = [9,8];
assertFlOpCount(f, ...
	[ ...
		 8, 0, 0,   0 ; ...
		 0, 0, 0,   0 ; ...
		 5, 0, 0,   0 ; ...
		12, 0,10,   0 ; ...
		 0, 0, 0,   0 ; ...
		 0, 0, 0,   0 ; ...
		 7, 0, 0,   0 ; ...
         0, 9, 8, 100   ...
	], %eps, [0,0,0,2,2,0]);
//assert(and(FlOpCtr_GetValue(f) == [8,0,0,0 ; 0,0,0,0 ; 5,0,0,0 ; 12,0,10,0 ; 0,0,0,0 ; 0,0,0,0 ; 7,0,0,0 ; 0,9,8,100]));
f(:,3) = [2;3;4;5;6;7;8;9];
assertFlOpCount(f, ...
	[ ...
		 8, 0, 2,   0 ; ...
		 0, 0, 3,   0 ; ...
		 5, 0, 4,   0 ; ...
		12, 0, 5,   0 ; ...
		 0, 0, 6,   0 ; ...
		 0, 0, 7,   0 ; ...
		 7, 0, 8,   0 ; ...
         0, 9, 9, 100   ...
	], %eps, [0,0,0,8,8,0]);
//assert(and(FlOpCtr_GetValue(f) == [8,0,2,0 ; 0,0,3,0 ; 5,0,4,0 ; 12,0,5,0 ; 0,0,6,0 ; 0,0,7,0 ; 7,0,8,0 ; 0,9,9,100]));
f(9,:) = [10,11,12,13];
assertFlOpCount(f, ...
	[ ...
		 8, 0, 2,   0 ; ...
		 0, 0, 3,   0 ; ...
		 5, 0, 4,   0 ; ...
		12, 0, 5,   0 ; ...
		 0, 0, 6,   0 ; ...
		 0, 0, 7,   0 ; ...
		 7, 0, 8,   0 ; ...
         0, 9, 9, 100 ; ...
		10,11,12,  13   ...
	], %eps, [0,0,0,4,4,0]);
//assert(and(FlOpCtr_GetValue(f) == [8,0,2,0 ; 0,0,3,0 ; 5,0,4,0 ; 12,0,5,0 ; 0,0,6,0 ; 0,0,7,0 ; 7,0,8,0 ; 0,9,9,100 ; 10,11,12,13]));


clear f;
FLOPCTR_Reset();

printf("    Manipulations : all unit tests passed.\n");


// End of script.
