//
// Tools for linear system solving.
//
// Author : Benoit Hamelin
// 2004 Universit de Sherbrooke
//


function [x]=lowtrisolve(A, b)
	// Solves the linear system Ax=b, where A is supposed invertible and lower
	// triangular.
	sA = size(A)
	x = b(1) / A(1,1)
	for i = 2:sA(1) do
		x = [x ; (b(i) - A(i,1:(i-1)) * x) / A(i,i)]
	end  // for
endfunction


function [x]=uptrisolve(A, b)
	// Solves the linear system Ax=b, where A is supposed invertible and upper
	// triangular.
	sA = size(A)
	x = b($) / A($,$)
	for i = (sA(1) - 1):-1:1 do
		x = [(b(i) - A(i, (i+1):$) * x) / A(i,i) ; x]
	end  // for
endfunction


function [x]=diagsolve(A, b)
	// Solves the linear system Ax=b, where A is supposed invertible and
	// diagonal.
	x = b ./ diag(A)
endfunction


// End of module.
