//
// Tests : tools to work of uni-dimensional and multi-dimensional indices.
//
// Author : Benoit Hamelin
// 2002 Universite de Sherbrooke
//


// combineIndexDim2Multi()
assert(combineIndexDim2Multi(1) == [1]);
assert(combineIndexDim2Multi(5) == [1;2;3;4;5]);
assert(combineIndexDim2Multi([1,5]) == [1,1;1,2;1,3;1,4;1,5]);
assert(combineIndexDim2Multi([5,1]) == [1,1;2,1;3,1;4,1;5,1]);
assert(combineIndexDim2Multi([3,3]) == [1,1;2,1;3,1;1,2;2,2;3,2;1,3;2,3;3,3]);
assert(combineIndexDim2Multi([3,4,2]) == ...
	[1,1,1;...
	 2,1,1;...
	 3,1,1;...
	 1,2,1;...
	 2,2,1;...
	 3,2,1;...
	 1,3,1;...
	 2,3,1;...
	 3,3,1;...
	 1,4,1;...
	 2,4,1;...
	 3,4,1;...
	 1,1,2;...
	 2,1,2;...
	 3,1,2;...
	 1,2,2;...
	 2,2,2;...
	 3,2,2;...
	 1,3,2;...
	 2,3,2;...
	 3,3,2;...
	 1,4,2;...
	 2,4,2;...
	 3,4,2]);
assert(combineIndexDim2Multi([2,1,2,1,2,1,2]) == ...
	[1,1,1,1,1,1,1;...
	 2,1,1,1,1,1,1;...
	 1,1,2,1,1,1,1;...
	 2,1,2,1,1,1,1;...
	 1,1,1,1,2,1,1;...
	 2,1,1,1,2,1,1;...
	 1,1,2,1,2,1,1;...
	 2,1,2,1,2,1,1;...
	 1,1,1,1,1,1,2;...
	 2,1,1,1,1,1,2;...
	 1,1,2,1,1,1,2;...
	 2,1,2,1,1,1,2;...
	 1,1,1,1,2,1,2;...
	 2,1,1,1,2,1,2;...
	 1,1,2,1,2,1,2;...
	 2,1,2,1,2,1,2]);
assertError('combineIndexDim2Multi(-2)');
assertError('combineIndexDim2Multi(0)');
assertError('combineIndexDim2Multi([0,1,-2])');
assertError('combineIndexDim2Multi([1,-1])');
assertError('combineIndexDim2Multi([3,0,2])');

// combineIndexVec2Multi()
assert(combineIndexVec2Multi(list(1)) == [1]);
assert(combineIndexVec2Multi(list(1, 2, 5)) == [1,2,5]);
assert(combineIndexVec2Multi(list(4:2:8)) == [4;6;8]);
assert(combineIndexVec2Multi(list(1, 1:4)) == [1,1;1,2;1,3;1,4]);
assert(combineIndexVec2Multi(list(4:-1:2, 3)) == [4,3;3,3;2,3]);
assert(combineIndexVec2Multi(list(1:3, 2:5, 6:-1:5)) == ...
	[...
		1,2,6;...
		2,2,6;...
		3,2,6;...
		1,3,6;...
		2,3,6;...
		3,3,6;...
		1,4,6;...
		2,4,6;...
		3,4,6;...
		1,5,6;...
		2,5,6;...
		3,5,6;...
		1,2,5;...
		2,2,5;...
		3,2,5;...
		1,3,5;...
		2,3,5;...
		3,3,5;...
		1,4,5;...
		2,4,5;...
		3,4,5;...
		1,5,5;...
		2,5,5;...
		3,5,5 ...
	]);
assert(combineIndexVec2Multi(list(1:2, 1, 1:3, 1, 1:2, 1, 1:2)) == ...
	[...
		1,1,1,1,1,1,1;...
		2,1,1,1,1,1,1;...
		1,1,2,1,1,1,1;...
		2,1,2,1,1,1,1;...
		1,1,3,1,1,1,1;...
		2,1,3,1,1,1,1;...
		1,1,1,1,2,1,1;...
		2,1,1,1,2,1,1;...
		1,1,2,1,2,1,1;...
		2,1,2,1,2,1,1;...
		1,1,3,1,2,1,1;...
		2,1,3,1,2,1,1;...
		1,1,1,1,1,1,2;...
		2,1,1,1,1,1,2;...
		1,1,2,1,1,1,2;...
		2,1,2,1,1,1,2;...
		1,1,3,1,1,1,2;...
		2,1,3,1,1,1,2;...
		1,1,1,1,2,1,2;...
		2,1,1,1,2,1,2;...
		1,1,2,1,2,1,2;...
		2,1,2,1,2,1,2;...
		1,1,3,1,2,1,2;...
		2,1,3,1,2,1,2;...
	]);
assertError('combineIndexVec2Multi(list(1:2, []))');
assertError('combineIndexVec2Multi(list([-2, -1], 1:9))');
assertError('combineIndexVec2Multi(list(4:5, 0:1, 9:12))');

// indexUni2Multi()
assert(indexUni2Multi(1, 1) == [1]);
assert(indexUni2Multi(1, 5) == [1]);
assert(indexUni2Multi(3, 5) == [3]);
assert(indexUni2Multi(5, 5) == [5]);
assert(indexUni2Multi(1, [3,3]) == [1,1]);
assert(indexUni2Multi(9, [3,3]) == [3,3]);
assert(indexUni2Multi(5, [3,3]) == [2,2]);
assert(indexUni2Multi(1, [4,4,4,4]) == [1,1,1,1]);
assert(indexUni2Multi(256, [4,4,4,4]) == [4,4,4,4]);
assert(indexUni2Multi(119, [4,4,4,4]) == [3,2,4,2]);
assertError('indexUni2Multi(0, 1)');
assertError('indexUni2Multi(-4, 1)');
//assertError('indexUni2Multi(2, 1)');
assertError('indexUni2Multi(0, [3,3])');
assertError('indexUni2Multi(-1, [3,3])');
//assertError('indexUni2Multi(10, [3,3])');

// indexMulti2Uni()
assert(indexMulti2Uni(1, 1) == [1]);
assert(indexMulti2Uni([1;2;3;4;5], 5) == [1;2;3;4;5]);
assert(indexMulti2Uni(...
	[1,1;1,2;1,3;1,4;...
	 2,1;2,2;2,3;2,4;...
	 3,1;3,2;3,3;3,4], [3,4]) == (1:12)');
assert(indexMulti2Uni(...
	[1,1,1,1;...
	 1,2,1,1;...
	 1,2,3,4;...
	 2,2,1,5;...
	 2,3,4,5], [2,3,4,5]) == [1;3;87;100;120]);
assert(indexMulti2Uni([], [2,3]) == []);
assert(indexMulti2Uni([], []) == []);
assertError("indexMulti2Uni([1,2,3;1,1,1], [2,3])");
assertError("indexMulti2Uni([1,2,3;1,1,1], [])");
// TODO - Figure out the "stack problem" that happens because of the three next tests.
// benoith 19jun2002 - It might be a Windows Scilab problem.  Check on Linux (or Sun, if can be made to work).
//assertError("indexMulti2Uni([1,2,3;1,0,1], [2,3,4])", "indexMulti2Uni() - EC - Multi index too small accepted.")
//assertError("indexMulti2Uni([1,2,-2;1,1,1], [2,3,4])", "indexMulti2Uni() - EC - Multi index too small accepted.")
//assertError("indexMulti2Uni([3,2,3;1,1,1], [2,3,4])", "indexMulti2Uni() - EC - Multi index out of bounds accepted.")

// concretizeIndexVector()
[r,o] = concretizeIndexVector(3, 5);
assert((r == 3) & ~o);
[r,o] = concretizeIndexVector(1, 4);
assert((r == 1) & ~o);
[r,o] = concretizeIndexVector(1:5, 5);
assert((r == (1:5)) & ~o);
[r,o] = concretizeIndexVector((1:5)', 5);
assert((r == [1;2;3;4;5]) & ~o);
[r,o] = concretizeIndexVector(2:$, 9);
assert((r == (2:9)) & ~o);
[r,o] = concretizeIndexVector($, 10);
assert((r == 10) & ~o);
[r,o] = concretizeIndexVector(:, 1);
assert((r == [1]) & ~o);
[r,o] = concretizeIndexVector(:, 10);
assert((r == (1:10)) & ~o);
[r,o] = concretizeIndexVector($:10, 8);
assert((r == [8,9,10]) & o);
[r,o] = concretizeIndexVector(3:$:12, 3);
assert((r == [3,6,9,12]) & o);
[r,o] = concretizeIndexVector(3:-1:$, 1);
assert((r == [3,2,1]) & o);
[r,o] = concretizeIndexVector(1:6, 5);
assert((r == (1:6)) & o);
assertError("concretizeIndexVector(1:$, [1,2])");
assertError("concretizeIndexVector(1:$, [1;2])");
assertError("concretizeIndexVector(3:$, 0)");
assertError("concretizeIndexVector(:, 0)");
assertError("concretizeIndexVector(:, -2)");
assertError("concretizeIndexVector(1:$, [])");
assertError("concretizeIndexVector([], 5)");
assertError("concretizeIndexVector(3:$, 1)");
assertError("concretizeIndexVector(0:6, 10)");

printf("ToolsIndex : all unit tests passed.\n")


// End of module.

