//
// Functions for the evaluator.
//
// Author : Benoit Hamelin
// 2004 Universit de Sherbrooke
//


function [this]=makeEvaluator(newIVarValue)
	global SCIAD_EVALUATOR_FIELDS
	this = mlist(SCIAD_EVALUATOR_FIELDS, ...
			list(), ...
			Evaluator_VisitIVar, ...
			Evaluator_VisitConst, ...
			Evaluator_VisitAdd, ...
			Evaluator_VisitAddInv, ...
			Evaluator_VisitT, ...
			Evaluator_VisitMult, ...
			Evaluator_VisitElemMult, ...
			Evaluator_VisitInv, ...
			Evaluator_VisitElemInv, ...
			Evaluator_VisitExp, ...
			Evaluator_VisitElemExp, ...
			Evaluator_VisitLog, ...
			Evaluator_VisitMonome, ...
			Evaluator_VisitElemMonome, ...
			Evaluator_VisitSin, ...
			Evaluator_VisitCos, ...
			Evaluator_VisitAtan, ...
			Evaluator_VisitAbs, ...
			Evaluator_VisitStub, ...
			newIVarValue ...
			)
endfunction


function [index]=Evaluator_VisitIVar(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	sivar = size(node)
	node.value = SCIAD_VISITOR($).newIVarValue(node.index)
	assert(and(sivar == size(node)))
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitConst(node, index)
	// Nothing to do : constants are... constant!
	global SCIAD_VISITOR SCIAD_EXPR
endfunction


function [index]=Evaluator_VisitAdd(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = SCIAD_EXPR($).nodes(predec(1)).value + ...
			SCIAD_EXPR($).nodes(predec(2)).value
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitAddInv(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = -SCIAD_EXPR($).nodes(predec).value
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitT(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = SCIAD_EXPR($).nodes(predec).value'
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitMult(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = SCIAD_EXPR($).nodes(predec(1)).value * ...
			SCIAD_EXPR($).nodes(predec(2)).value
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitElemMult(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = SCIAD_EXPR($).nodes(predec(1)).value .* ...
			SCIAD_EXPR($).nodes(predec(2)).value
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitInv(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = inv(SCIAD_EXPR($).nodes(predec).value)
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitElemInv(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = ones(SCIAD_EXPR($).nodes(predec).value) ./ ...
			SCIAD_EXPR($).nodes(predec).value
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitExp(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = node.base ^ SCIAD_EXPR($).nodes(predec).value
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitElemExp(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = node.base .^ SCIAD_EXPR($).nodes(predec).value
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitLog(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = log(SCIAD_EXPR($).nodes(predec).value)
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitMonome(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	[v,lp2,li,lp,re,rf] = elevatePower(...
	 		SCIAD_EXPR($).nodes(predec).value, ...
	 		node.exponent)
	node.value = v
	node.listPowers2 = lp2
	node.listIndex = li
	node.listProds = lp
	node.rootExpo = re
	node.rootFactor = rf
	node.listPowers2
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitElemMonome(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	[v,lp2,lm,lp,re,rf] = elevatePowerElem(...
	 		SCIAD_EXPR($).nodes(predec).value, ...
			node.exponent)
	node.value = v
	node.listPowers2 = lp2
	node.listMasks = lm
	node.listProds = lp
	node.rootExpo = re
	node.rootFactor = rf
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitSin(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = sin(SCIAD_EXPR($).nodes(predec).value)
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitCos(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = cos(SCIAD_EXPR($).nodes(predec).value)
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitAtan(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = atan(SCIAD_EXPR($).nodes(predec).value)
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitAbs(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	predec = SCIAD_VISITOR($).listPredec(index)
	node.value = abs(SCIAD_EXPR($).nodes(predec).value)
	SCIAD_EXPR($).nodes(index) = node
endfunction


function [index]=Evaluator_VisitStub(node, index)
	global SCIAD_VISITOR SCIAD_EXPR
	error("Not supposed to encounter stubs in expressions we re-evaluate...")
endfunction


// End of module.

