// ExprNodes.sci
//
// Definitions regarding nodes that compose an expression graph.
//
// Author : Benoit Hamelin
// 2002 Universite de Sherbrooke
//


// Fields common to all nodes :
//
// type  - Specific node type.
// display  - Quick string representation for the node.
// dims - Dimensions (size) of the represented sub-expression.
// Visit - Dispatcher for expression graph traversal.
//
global SCIAD_EXPRNODE_FIELDS SCIAD_EXPRNODE_BASETYPE;
SCIAD_EXPRNODE_FIELDS = ["ADNODE", "display", "value", "isConstant", "Visit"];


// Independant variable node.
global SCIAD_NODEIVAR_FIELDS;
SCIAD_NODEIVAR_FIELDS = [SCIAD_EXPRNODE_FIELDS, "index"];

// Constant node.
global SCIAD_NODECONST_FIELDS;
SCIAD_NODECONST_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Addition node.
global SCIAD_NODEADD_FIELDS;
SCIAD_NODEADD_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Additive-inverse node.
global SCIAD_NODEADDINV_FIELDS;
SCIAD_NODEADDINV_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Transposition node.
global SCIAD_NODET_FIELDS;
SCIAD_NODET_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Regular product node.
global SCIAD_NODEMULT_FIELDS;
SCIAD_NODEMULT_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Element-wise product node.
global SCIAD_NODEELEMMULT_FIELDS;
SCIAD_NODEELEMMULT_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Regular inverse node.
global SCIAD_NODEINV_FIELDS;
SCIAD_NODEINV_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Element-wise inverse node.
global SCIAD_NODEELEMINV_FIELDS;
SCIAD_NODEELEMINV_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Regular exponentiatial node (constant base, variable scalar exponent).
global SCIAD_NODEEXP_FIELDS;
SCIAD_NODEEXP_FIELDS = [SCIAD_EXPRNODE_FIELDS, "base"];

// Element-wise exponential node (constant base, variable same-dimensional exponent).
global SCIAD_NODEELEMEXP_FIELDS;
SCIAD_NODEELEMEXP_FIELDS = [SCIAD_EXPRNODE_FIELDS, "base"];

// Natural logarithm node.
global SCIAD_NODELOG_FIELDS;
SCIAD_NODELOG_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Regular monomial node (variable base, constant scalar exponent fractional).
global SCIAD_NODEMONOME_FIELDS;
SCIAD_NODEMONOME_FIELDS = [SCIAD_EXPRNODE_FIELDS, ...
		"exponent", ...
		"listPowers2", ...
		"listIndex", ...
		"listProds", ...
		"rootExpo", ...
		"rootFactor"];

// Element-wise monomial node (variable base, constant same-dimensional exponent fractional).
global SCIAD_NODEELEMMONODE_FIELDS;
SCIAD_NODEELEMMONODE_FIELDS = [SCIAD_EXPRNODE_FIELDS, ...
		"exponent", ...
		"listPowers2", ...
		"listMasks", ...
		"listProds", ...
		"rootExpo", ...
		"rootFactor"];

// Element-wise sine node.
global SCIAD_NODESIN_FIELDS;
SCIAD_NODESIN_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Element-wise cosine node.
global SCIAD_NODECOS_FIELDS;
SCIAD_NODECOS_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Element-wise circular arctangent node.
global SCIAD_NODEATAN_FIELDS;
SCIAD_NODEATAN_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Absolute value node.
global SCIAD_NODEABS_FIELDS;
SCIAD_NODEABS_FIELDS = SCIAD_EXPRNODE_FIELDS;

// Sub-expression stub node.
global SCIAD_NODESTUB_FIELDS;
SCIAD_NODESTUB_FIELDS = [SCIAD_EXPRNODE_FIELDS, "subexpr", "index"];


getf("ExprNode.sci");


// End of module.

