//
// Operators and overloaded functions that relate to trigonometrics (circular and hyperbolic, matricial and element-wise) to build
// expressions in regular Scilab syntax.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


function [r]=%ADEXPR_sin(expr)
	// Sine of an expression hypermatrix.
	r = extendExpr(expr, makeNodeSin(sin(Expr_GetEvaluation(expr))))
endfunction


function [r]=%ADEXPR_cos(expr)
	// Cosine of an expression hypermatrix.
	r = extendExpr(expr, makeNodeCos(cos(Expr_GetEvaluation(expr))))
endfunction


function [r]=%ADEXPR_atan(varargin)
	// Arc-tangent of an expression hypermatrix.
	// NOTE : this doesn't handle the second form (bi-parametric) of the atan Scilab primitive.

	[lhs,rhs] = argn()
	if (rhs <> 1) then
		error("Forms of atan with more than one argument are not handled by Sciad.");
	end  // if
	expr = varargin(1)

	r = extendExpr(expr, makeNodeAtan(atan(Expr_GetEvaluation(expr))))
endfunction


// End of module.

