//
// Module that runs the code necessary to set up the Sciad library.
//
// Author : Benoit Hamelin
// 2002 Universite de Sherbrooke
//


// Must augment the stack size to perform real work with this stuff - for now.
ss = stacksize();
if (ss(1) < 3e7) then
	stacksize(3e7);
end  // if
ss = gstacksize();
if (ss(1) < 3000) then
	gstacksize(3000);
end  // if
clear ss;

here = getcwd();
chdir("../Tools");
exec("Tools.sce");
chdir(here);
clear here;

exec("Visitor.sce");

exec("ExprNode.sce");
exec("Expr.sce");

getf("Utils.sci");

exec("Evaluator.sce");
exec("FiniteDiffCalc.sce");
exec("Derivator.sce");
exec("TaylorDev.sce");

getf("Interface.sci");

getf("UnitTests.sci");
global SCIAD_UNIT_TESTS;
if (SCIAD_UNIT_TESTS) then

	chdir("tests");
	timer();
	exec("LoadUNCprobs.sce");

	exec("Test_ExprNode.sce");

	exec("Test_Expr_Base.sce");
	exec("Test_Expr_VarConst.sce");
	exec("Test_Expr_Addition.sce");
	exec("Test_Expr_Multiplication.sce");
	exec("Test_Expr_Concatenation.sce");
	exec("Test_Expr_AddScalar.sce");
	exec("Test_Expr_Sum.sce");
	exec("Test_Expr_ElemMult.sce");
	exec("Test_Expr_Insertion.sce");
	exec("Test_Expr_ExtractionMult.sce");
	exec("Test_Expr_Subtraction.sce");
	exec("Test_Expr_Division.sce");
	exec("Test_Expr_Polynome.sce");
	exec("Test_Expr_Exponential.sce");
	exec("Test_Expr_Logarithm.sce");
	exec("Test_Expr_AbsoluteValue.sce");
	exec("Test_Expr_CircTrigo.sce");
	exec("Test_Expr_SubExprExtraction.sce");
	//exec("Test_Expr_ReplaceLeaf.sce");  // ReplaceLeaf doesn't work for the moment.
	//exec("Test_Expr_UNC.sce");

	exec("Test_Utils.sce");

	exec("Test_Evaluator.sce");
	exec("Test_FiniteDiffCalc.sce");

	exec("Test_DerivNum_Base.sce");
	exec("Test_DerivNum_RegProduct.sce");
	exec("Test_DerivNum_ElemProduct.sce");
	exec("Test_DerivNum_Construction.sce");
	exec("Test_DerivNum_InvMonome.sce");
	exec("Test_DerivNum_ExpoLog.sce");
	exec("Test_DerivNum_Abs.sce");
	exec("Test_DerivNum_CircTrigo.sce");

	exec("Test_DerivExpr_Base.sce");
	exec("Test_DerivExpr_ElemProduct.sce");
	exec("Test_DerivExpr_InvMonome.sce");
	exec("Test_DerivExpr_ExpoLog.sce");
	exec("Test_DerivExpr_Abs.sce");
	exec("Test_DerivExpr_CircTrigo.sce");

	exec("Test_TaylorDev_Base.sce");
	exec("Test_TaylorDev_RegProduct.sce");
	exec("Test_TaylorDev_ElemProduct.sce");
	exec("Test_TaylorDev_Inversion.sce");
	exec("Test_TaylorDev_ExpLog.sce");
	exec("Test_TaylorDev_Monome.sce");
	exec("Test_TaylorDev_Trigo.sce");

	exec("Test_Interface.sce");

	printf("\nSciad : all unit tests passed!\n");
	printf("Timer : %.1f\n\n", timer());
	chdir("..");

end  // if

printf("Sciad library loaded successfully.\n");


// End of module.

