//
// Functions for the stub substituter.
//
// Author : Benoit Hamelin
// 2004 Universit de Sherbrooke
//


function [stubSubst]=makeStubSubst(origExpr)
	stubSubst = tlist(SCIAD_STUBSUBST_FIELDS, ...
			StubSubst_VisitIVar, ...
			StubSubst_VisitConst, ...
			StubSubst_VisitAdd, ...
			StubSubst_VisitAddInv, ...
			StubSubst_VisitT, ...
			StubSubst_VisitMult, ...
			StubSubst_VisitElemMult, ...
			StubSubst_VisitInv, ...
			StubSubst_VisitElemInv, ...
			StubSubst_VisitExp, ...
			StubSubst_VisitElemExp, ...
			StubSubst_VisitLog, ...
			StubSubst_VisitMonome, ...
			StubSubst_VisitElemMonome, ...
			StubSubst_VisitSin, ...
			StubSubst_VisitCos, ...
			StubSubst_VisitAtan, ...
			StubSubst_VisitAbs, ...
			StubSubst_VisitStub, ...
			origExpr, ...
			list() ...
			)
endfunction


function [derivExpr]=StubSubst_Process(this, derivExpr)
	// Processes the stub substitution job on the given derivative expression.
	this.dict = list()
	nbNodes = Expr_GetNodeCount(derivExpr)
	for i = 1:nbNodes do
		this.dict(i) = 0
	end  // for
	[derivExpr,this] = Expr_VisitForward(derivExpr, this)
endfunction


// For all nodes other than stubs, absolutely nothing has to be done.

function [index]=StubSubst_VisitIVar(node, index)
endfunction

function [index]=StubSubst_VisitConst(node, index)
endfunction

function [index]=StubSubst_VisitAdd(node, index)
endfunction

function [index]=StubSubst_VisitAddInv(node, index)
endfunction

function [index]=StubSubst_VisitT(node, index)
endfunction

function [index]=StubSubst_VisitMult(node, index)
endfunction

function [index]=StubSubst_VisitElemMult(node, index)
endfunction

function [index]=StubSubst_VisitInv(node, index)
endfunction

function [index]=StubSubst_VisitElemInv(node, index)
endfunction

function [index]=StubSubst_VisitExp(node, index)
endfunction

function [index]=StubSubst_VisitElemExp(node, index)
endfunction

function [index]=StubSubst_VisitLog(node, index)
endfunction

function [index]=StubSubst_VisitMonome(node, index)
endfunction

function [index]=StubSubst_VisitElemMonome(node, index)
endfunction

function [index]=StubSubst_VisitSin(node, index)
endfunction

function [index]=StubSubst_VisitCos(node, index)
endfunction

function [index]=StubSubst_VisitAtan(node, index)
endfunction

function [index]=StubSubst_VisitAbs(node, index)
endfunction


function [index]=StubSubst_VisitStub(node, index)
	// We extract the appropriate sub-expression from the original and insert it in the derivative.
	global SCIAD_VISITOR SCIAD_EXPR

	// Extract and index the sub-expression.
	subexpr = []
	if (SCIAD_VISITOR($).dict(node.root) == 0) then
		subexpr = Expr_ExtractSubExpr(SCIAD_VISITOR($).origExpr, node.root)
		SCIAD_VISITOR($).dict(node.root) = subexpr
	else
		subexpr = SCIAD_VISITOR($).dict(node.root)
	end  // if
	sil = size(node.indexList)
	if (sil == 1) then
		subexpr = subexpr(node.indexList(1))
	elseif (sil == 2) then
		subexpr = subexpr(node.indexList(1), node.indexList(2))
	end  // if

	// Insert the sub-expression into the derivative, effectively replacing the stub.
	SCIAD_EXPR($) = Expr_ReplaceLeaf(SCIAD_EXPR($), index, subexpr)
	index = index + Expr_GetNodeCount(subexpr) - 1
endfunction


// End of module.
