//
// Sciad-specific unit testing tools.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


function []=assertExpr(varargin)
	[lhs,rhs] = argn()
	if (rhs < 2) then
		error("assertExpr expects at least two parameters.");
	end  // if
	expr = varargin(1)
	value = varargin(2)
	z = 1e-7
	if (rhs == 3) then
		z = varargin(3)
	end  // if

	assertSize(expr, size(value))
	ev = Expr_GetEvaluation(expr)
	assert(and(abs(ev(:) - value(:)) < z))
endfunction


function []=assertFiniteDiff(expr, decrement, numzero, expected, precision)
	assert(and(abs(Expr_CalcFiniteDiff(expr, decrement, numzero) - expected) < precision));
endfunction


function []=assertNumGradient(expr, n, gradValue, precision)
	assert(and(abs(Expr_Gradient(expr, n, %t) - gradValue) < precision));
endfunction


function []=assertNumJacobian(expr, n, initAdj, jacValue, precision)
	assert(and(abs(Expr_Jacobian(expr, n, initAdj, %t) - jacValue) < precision));
endfunction


function []=assertTaylor(expr, order, anchor, expectedCoeff, precision)
	taylorCoeff = Expr_TaylorCoeff(expr, order, anchor)
	assert(and(size(taylorCoeff) == size(expectedCoeff)))
	assert(and(abs(taylorCoeff(:) - expectedCoeff(:)) < precision))
endfunction


function []=assertExprEdges(expr, edges)
	assert(and(Expr_GetEdges(expr) == edges));
endfunction


function []=assertReeval(expr, newIVarValue, expectedValue, precision)
	assertExpr(Expr_Reevaluate(expr, newIVarValue), expectedValue, precision)
endfunction


// End of module.
