//
// Useful tools that can be used along Sciad functions.
//
// Author : Benoit Hamelin
// 2002 Universit de Sherbrooke
//


function [z]=%ADEXPR_zeros(expr)
	// Gives an hypermatrix of zeros that has same size as the given expression hypermatrix.
	s = size(expr)
	s = s(:)'
	z = matrix(zeros(prod(s), 1), s)
endfunction


function [o]=%ADEXPR_ones(expr)
	// Gives an hypermatrix of ones that has same size as the given expression hypermatrix.
	s = size(expr)
	s = s(:)'
	o = matrix(ones(prod(s), 1), s)
endfunction


function [r]=%ADEXPR_rand(expr)
	// Gives an hypermatrix of independant uniformly distributed on [0,1[ random numbers that has same size as the given
	// expression hypermatrix.
	s = size(expr)
	s = s(:)'
	r = matrix(rand(prod(s), 1), s)
endfunction


function [i]=%ADEXPR_eye(expr)
	// Gives an identity matrix that matches dimension of the given expression hypermatrix.
	s = size(expr)
	if (size(s, "*") > 2) then
		error("Cannot make an identity matrix out of an hypermatrix (more than 2 dimensions).")
	end  // if
	i = eye(s(1), s(2))
endfunction


function [newExpr]=%ADEXPR_matrix(varargin)
	// Reformulates the given expression hypermatrix into one that has another size.
	// TODO - This solution is not efficient, expression-graph-wise.
	// TODO - Only works with two-dimensional objects.

	[lhs,rhs] = argn();
	expr = []
	newSize = []
	if (rhs == 2) then
		newSize = varargin(2)
	elseif (rhs == 3) then
		newSize = [varargin(2),varargin(3)]
	else
		error("Must have two or three arguments.  See apropos matrix.")
	end  // if
	expr = varargin(1)
	oldSize = size(expr)
	if (prod(oldSize) <> prod(newSize)) then
		error("New matrix size must hold same number of elements as original.");
	end  // if

	newExpr = zeros(newSize(1), newSize(2))
	for r = 1:newSize(1) do
		fact = zeros(newSize(1), 1)
		fact(r) = 1
		newExpr = newExpr + fact * expr(((r - 1) * newSize(2) + 1):(r * newSize(2)))
	end  // for
endfunction


// End of module.

