//
// Tests on derivation of expressions to expressions -- base.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Additions.
y = ad_ivar(4);
F = y - y + y + y;
DF = Expr_Gradient(F, 1, %f);
assertExpr(DF, 2);
DDF = Expr_Gradient(DF, 1, %f);
assertExpr(DDF, 0);
DDDF = Expr_Gradient(DDF, 1, %f);
assertExpr(DDDF, 0);
DF = DF + y + y + y;
DDF = Expr_Gradient(DF, 1, %f);
assertExpr(DDF, 3);
DDDF = Expr_Gradient(DDF, 1, %f);
assertExpr(DDDF, 0);

// Regular products.
x = ad_ivar([7;-1;1;-2;3]);
xt = x';
F = x' * x([5;4;5;3;3]);
DF = Expr_Gradient(F, 5, %f);
assertExpr(DF, [3,-2,4,0,9]);
DDF = Expr_Jacobian(DF, 5, eye(5,5), %f);
assertExpr(DDF, [0,0,0,0,1 ; 0,0,0,1,0 ; 0,0,0,1,2 ; 0,1,1,0,0 ; 1,0,2,0,0]);

F = x([1,2,3]) * xt([3,2]) * x([4,1]) * xt([1,3,5]) * [1;-2;-1];
DF = Expr_Jacobian(F, 5, eye(3,3), %f);
assertExpr(DF, ...
	[...
		-95, 98, 98, 14, 63 ; ...
		 11,-32,-14,- 2,- 9 ; ...
		-11, 14,- 4,  2,  9   ...
	]);
G = DF * ones(5,1);
DG = Expr_Jacobian(G, 5, eye(3,3), %f);
assertExpr(DG, ...
	[...
		 80,-70,-14, 2,-26 ; ...
		-20, 54, 18, 2, 14 ; ...
		- 2,  2, 38, 2,  4   ...
	]);
H = DG * ones(5,1);
DH = Expr_Jacobian(H, 5, eye(3,3), %f);
assertExpr(DH, ...
	[...
		 6	,-52	,-92	,-28	,-38	; ...
		18	,-32	,  4	,  4	,- 6	; ...
		18	,-28	,-32	,- 4	,-14	  ...
	]);

F = [x(2) ; 5 + x(1) ; x(2)*x(3) - 3 * x(1) ; x(5)*x(4)*x(4)];
DF = Expr_Jacobian(F, 5, eye(4,4), %f);
assertExpr(DF, ...
	[ ...
		 0, 1,  0,   0, 0 ; ...
		 1, 0,  0,   0, 0 ; ...
		-3, 1, -1,   0, 0 ; ...
		 0, 0,  0, -12, 4   ...
	]);
G = [0, 2, x(1)*x(1), -x(2)] * DF;
DG = Expr_Jacobian(G, 5, eye(5,5), %f);
assertExpr(DG, ...
	[ ...
		-42,   0,  0,  0,  0 ; ...
		 14,   0, 49,  0,  0 ; ...
		-14,  49,  0,  0,  0 ; ...
		  0,  12,  0,  6, -4 ; ...
		  0, - 4,  0, -4,  0   ...
	]);


clear y;
clear F;
clear DF;
clear DDF;
clear DDDF;
clear G;
clear DG;
clear H;
clear DH;

printf("Deriv. expr. - Base : all unit tests passed.\n");


// End of script.
