//
// Tests on derivation of expressions to expressions -- element-wise product.
//
// Author : Benoit Hamelin
// 2004 Universit de Sherbrooke
//


x = ad_ivar([7;-1;1;-2;3]);
xt = x';
F = [x(1) ; 5 ; -x(3)] .* [x(2) + 3 ; -x(4)*x(5) ; x(4)+x(2)];
DF = Expr_Jacobian(F, 5, eye(3,3), %f);
assertExpr(DF, ...
	[...
		2,  7, 0,   0,  0 ; ...
		0,  0, 0, -15, 10 ; ...
		0, -1, 3, - 1,  0   ...
	]);
G = DF * x([5,4,3,2,1]);
DG = Expr_Jacobian(G, 5, ones(1,3), %f);
assertExpr(DG, [8, -14, 6, -30, 7]);
H = DG';
DH = Expr_Jacobian(H, 5, ones(1,5), %f);
assertExpr(DH, [-4, -6, -4, -6, -4]);

F = (xt .* xt([3,5,2,1,4])) * [5 ; x(2) * x(3) ; -2*x(1) ; 3 ; -1];
DF = Expr_Gradient(F, 5, %f);
assertExpr(DF, [1, -20, 52, 18, 3]);
DDF = Expr_Jacobian(DF, 5, eye(5,5), %f);
assertExpr(DDF, ...
	[ ...
		 0, - 2,   7,  3,  0 ; ...
		-2,   6, -20,  0, -2 ; ...
		 7, -20,   0,  0,  1 ; ...
		 3,   0,   0,  0, -1 ; ...
		 0, - 2,   1, -1,  0   ...
	]);

F = ([x(1), 5, x(5) ; x(2), 0, 2*x(1) ; x(3), x(4), -x(3)] .* [x(2), x(3), -x(5) ; -x(1), x(3), 0 ; x(4), 2, -3]) * x([3,4,1]);
DF = Expr_Jacobian(F, 5, eye(3,3), %f);
assertExpr(DF, ...
	[ ...
		-10,  7, -17,  5, -42 ; ...
		  1, -7,   7,  0,   0 ; ...
		  3,  0,  17, -7,   0   ...
	]);

// Test for bug regarding transposition applied to an element-wise scalar
// operation.
x = ad_ivar([7;-1;1;-2;3]);
xt = x';
F = (xt * x) .* (xt * x);
assertExpr(F, 4096, %eps);
G = Expr_Gradient(F, 5, %f);
assertExpr(G, [1792, -256, 256, -512, 768], %eps);
H = Expr_Jacobian(G, 5, eye(5,5), %f);
assertExpr(H, ...
	[...
		 648, - 56,   56, -112,  168 ; ...
		- 56,  264, -  8,   16, - 24 ; ...
		  56, -  8,  264, - 16,   24 ; ...
		-112,   16, - 16,  288, - 48 ; ...
		 168, - 24,   24, - 48,  328   ...
	], %eps);


clear x;
clear xt;
clear F;
clear DF;
clear DDF;
clear G;
clear DG;
clear H;
clear DH;

printf("Deriv. expr. - Element-wise product : all unit tests passed.\n");


// End of script.
