//
// Tests on derivation of expressions to expressions -- exponentials and logarithms.
//
// Author : Benoit Hamelin
// 2004 Universit de Sherbrooke
//


// Tests on exponentials.
x = ad_ivar(3);
F = 2 ^ x;
DF = Expr_Gradient(F, 1, %f);
assertExpr(DF, 8*log(2));
DDF = Expr_Gradient(DF, 1, %f);
assertExpr(DDF, 8*log(2)^2);
DDDF = Expr_Gradient(DDF, 1, %f);
assertExpr(DDDF, 8*log(2)^3);
DDDDF = Expr_Gradient(DDDF, 1, %f);
assertExpr(DDDDF, 8*log(2)^4);
F = exp(x);
DF = Expr_Gradient(F, 1, %f);
assertExpr(DF, exp(3));
DDF = Expr_Gradient(DF, 1, %f);
assertExpr(DDF, exp(3));
DDDF = Expr_Gradient(DDF, 1, %f);
assertExpr(DDDF, exp(3));
DDDDF = Expr_Gradient(DDDF, 1, %f);
assertExpr(DDDDF, exp(3));

x = ad_ivar([7;-1;1;-2;3]);
F = [3 ; 1 ; 0; %e] .^ [x(1)*x(3) ; x(2) ; x(5) ; x(4)^2];
DF = Expr_Jacobian(F, 5, eye(4,4), %f);
assertExpr(DF, ...
	[ ...
		3^7*log(3), 0, 7*(3^7)*log(3), 0        , 0 ; ...
		0         , 0, 0             , 0        , 0 ; ...
		0         , 0, 0             , 0        , 0 ; ...
		0         , 0, 0             , -4*exp(4), 0   ...
	]);
G = [1,0,0,0 ; 0,0,0,1] * DF * ones(5,1);
DG = Expr_Jacobian(G, 5, eye(2,2), %f);
assertExpr(DG, ...
	[ ...
		(3^7)*(log(3)^2) + 3^7*log(3) + 7*(3^7)*(log(3)^2), ...
			0, ...
				(3^7)*log(3) + 7*(3^7)*(log(3)^2) + 49*(3^7)*(log(3)^2), ...
					0, ...
						0 ; ...
		0, ...
			0, ...
				0, ...
					18*exp(4), ...
						0 ...
	]);
F = ([3 , 1 , 0, %e] .^ [x(1)*x(3) , x(2) , x(5) , x(4)^2])';
DF = Expr_Jacobian(F, 5, eye(4,4), %f);
assertExpr(DF, ...
	[ ...
		3^7*log(3), 0, 7*(3^7)*log(3), 0        , 0 ; ...
		0         , 0, 0             , 0        , 0 ; ...
		0         , 0, 0             , 0        , 0 ; ...
		0         , 0, 0             , -4*exp(4), 0   ...
	]);
F = exp([x([1,5]), x([3,4])]) * [1;-1];
DF = Expr_Jacobian(F, 5, eye(2,2), %f);
assertExpr(DF, ...
	[ ...
		exp(7), 0, -%e, 0       , 0     ; ...
		0     , 0,  0 , -exp(-2), exp(3)  ...
	]);
G = DF * [1 ; 1 ; -1 ; 1 ; 1];
DG = Expr_Jacobian(G, 5, eye(2,2), %f);
assertExpr(DG, ...
	[ ...
		exp(7), 0, %e, 0       , 0     ; ...
		0     , 0, 0 , -exp(-2), exp(3)  ...
	]);

// Tests on logarithms.
x = ad_ivar(3);
F = log(x^2);
DF = Expr_Gradient(F, 1, %f);
assertExpr(DF, 2/3);
DDF = Expr_Gradient(DF, 1, %f);
assertExpr(DDF, -2/9);

x = ad_ivar([7;-1;1;-2;3]);
F = log([4 ; x(1)*x(5) ; x(3) ; log(x(5))]);
DF = Expr_Jacobian(F, 5, eye(4,4), %f);
assertExpr(DF, ...
	[ ...
		0  , 0, 0, 0, 0            ; ...
		1/7, 0, 0, 0, 1/3          ; ...
		0  , 0, 1, 0, 0            ; ...
		0  , 0, 0, 0, 1/(3*log(3))   ...
	]);
G = [0,1,0,0 ; 0,0,1,0 ; 0,0,0,1] * DF * ones(5,1);
DG = Expr_Jacobian(G, 5, eye(3,3), %f);
assertExpr(DG, ...
	[ ...
		-1/49, 0,  0, 0, -1/9                       ; ...
		 0   , 0, -1, 0,  0                         ; ...
		 0   , 0,  0, 0, -(log(3) + 1)/(3*log(3))^2   ...
	]);


clear x;
clear F;
clear DF;
clear DDF;
clear DDDF;
clear DDDDF;
clear G;
clear DG;

printf("Deriv. expr. - Exponentials and logarithms : all unit tests passed.\n");


// End of script.

