//
// Tests on derivation of expressions to expressions -- inversion and monomes.
//
// Author : Benoit Hamelin
// 2004 Universit de Sherbrooke
//


// Tests on inversion.
x = ad_ivar(3);
F = 1 / x;
DF = Expr_Gradient(F, 1, %f);
assertExpr(DF, -1/9);
DDF = Expr_Gradient(DF, 1, %f);
assertExpr(DDF, 2/27);
DDDF = Expr_Gradient(DDF, 1, %f);
assertExpr(DDDF, -2/27);
F = 1 ./ x;
DF = Expr_Gradient(F, 1, %f);
assertExpr(DF, -1/9);
DDF = Expr_Gradient(DF, 1, %f);
assertExpr(DDF, 2/27);
DDDF = Expr_Gradient(DDF, 1, %f);
assertExpr(DDDF, -2/27);

x = ad_ivar([7;-1;1;-2;3]);
xt = x';
F = [x(2) * x(3) ; -1 ; x(5)] ./ [x(4) ; x(1)*x(3) ; x(2)*x(2)];
DF = Expr_Jacobian(F, 5, ones(1,3), %f);
assertExpr(DF, [1/49, 5.5, 9/14, 1/4, 1]);
G = DF';
DG = Expr_Jacobian(G, 5, ones(1,5), %f);
assertExpr(DG, [-9/343, 19.25, -15/49-1/4, 1/4, 2]);
F = ([x(2) * x(3) , -1 , x(5)] ./ [x(4) , x(1)*x(3) , x(2)*x(2)])';
DF = Expr_Jacobian(F, 5, ones(1,3), %f);
assertExpr(DF, [1/49, 5.5, 9/14, 1/4, 1]);
F = ((x([1,5]) * xt([3,2])) ./ (x([2,4]) * xt([1,3]))) * [1;1];
DF = Expr_Jacobian(F, 5, eye(2,2), %f);
assertExpr(DF, ...
	[ ...
		1   , -1  , - 8  ,  0   ,  0  ; ...
		3/98, -3/2, -12/7,  9/14,  3/7  ...
	]);

// Tests on monomes.
x = ad_ivar(3);
F = x ^ 4;
DF = Expr_Gradient(F, 1, %f);
assertExpr(DF, 108);
DDF = Expr_Gradient(DF, 1, %f);
assertExpr(DDF, 108);
DDDF = Expr_Gradient(DDF, 1, %f);
assertExpr(DDDF, 72);
DDDDF = Expr_Gradient(DDDF, 1, %f);
assertExpr(DDDDF, 24);
x = ad_ivar(9);
F = x ^ (3/2);
DF = Expr_Gradient(F, 1, %f);
assertExpr(DF, 9/2);
DDF = Expr_Gradient(DF, 1, %f);
assertExpr(DDF, 1/4);
DDDF = Expr_Gradient(DDF, 1, %f);
assertExpr(DDDF, -1/72);
x = ad_ivar(3);
F = x .^ 4;
DF = Expr_Gradient(F, 1, %f);
assertExpr(DF, 108);
DDF = Expr_Gradient(DF, 1, %f);
assertExpr(DDF, 108);
DDDF = Expr_Gradient(DDF, 1, %f);
assertExpr(DDDF, 72);
DDDDF = Expr_Gradient(DDDF, 1, %f);
assertExpr(DDDDF, 24);
x = ad_ivar(9);
F = x .^ (3/2);
DF = Expr_Gradient(F, 1, %f);
assertExpr(DF, 9/2);
DDF = Expr_Gradient(DF, 1, %f);
assertExpr(DDF, 1/4);
DDDF = Expr_Gradient(DDF, 1, %f);
assertExpr(DDDF, -1/72);

x = ad_ivar([7;-1;1;-2;3]);
F = [x(1) ; x(3)*x(4) ; 2*x(5) ; -x(2)] .^ [5/4 ; 3 ; -2 ; 4];
DF = Expr_Jacobian(F, 5, ones(1,4), %f);
assertExpr(DF, [(5/4) * 7^(1/4), -4, -24, 12, -1/54]);
G = DF';
DG = Expr_Jacobian(G, 5, ones(1,5), %f);
assertExpr(DG, [5 / (16 * 7^(3/4)), 12, -12, 24, 3/162]);
F = ([x(1) , x(3)*x(4) , 2*x(5) , -x(2)] .^ [5/4 , 3 , -2 , 4])';
DF = Expr_Jacobian(F, 5, ones(1,4), %f);
assertExpr(DF, [(5/4) * 7^(1/4), -4, -24, 12, -1/54]);
F = ([x([1,4,3]), x([2,3,5])] .^ [1, -2 ; 0, 3 ; -1/2, 1/2]) * [1;-1];
DF = Expr_Jacobian(F, 5, ones(1,3), %f);
assertExpr(DF, [1, -2, -7/2, 0, -1/(2*sqrt(3))]);
G = DF';
DG = Expr_Jacobian(G, 5, eye(5,5), %f);
assertExpr(DG, ...
	[ ...
		0,  0,   0  , 0, 0              ; ...
		0, -6,   0  , 0, 0              ; ...
		0,  0, -21/4, 0, 0              ; ...
		0,  0,   0  , 0, 0              ; ...
		0,  0,   0,   0, 1/(12*sqrt(3))   ...
	]);


clear x;
clear xt;
clear F;
clear DF;
clear DDF;
clear DDDF;
clear DDDDF;
clear G;
clear DG;

printf("Deriv. expr. - Inversion and monomes : all unit tests passed.\n");


// End of script.

