//
// Tests on numerical evaluation of derivatives : constructed vectors and matrices (concatenation and
// insertion).
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Vertical concatenation : gradient and jacobian.
y = ad_ivar([7;-1;1;-2;3]);
yt = y';
assertNumJacobian([y(2);y(4);y(1)], 5, eye(3,3), [0,1,0,0,0 ; 0,0,0,1,0 ; 1,0,0,0,0], %eps);
assertNumJacobian([y(2) + y(3) - y(4); y(2) * y(5); -y([4,2]) .* y([1,2])], 5, eye(4,4), ...
	[...
		0, 1, 1, -1,  0 ; ...
		0, 3, 0,  0, -1 ; ...
		2, 0, 0, -7,  0 ; ...
		0, 2, 0,  0,  0   ...
	], %eps);
assertNumJacobian([yt([5,3,4]) ; 2 * yt([1,2,3]) + yt([3,4,1])] * [1;y(3);2], 5, eye(2,2), ...
	[...
		0, 0,  2, 2, 1 ; ...
		4, 2,  1, 1, 0   ...
	], %eps);
assertNumGradient(yt([3,4,2]) * [y(5);2;y(1) + y(2)], 5, [-1,5,3,2,1], %eps);
assertNumGradient(yt([1,2,3]) * [yt([1,3]) ; yt([3,5]) ; yt([4,1]) + [2,3]] * [y(5) + y(4)*y(3) ; 6], 5, [26,19,5,49,42], %eps);

// Horizontal concatenation : gradient and jacobian.
assertNumJacobian([y(2),y(4),y(1)], 5, eye(3,3), [0,1,0,0,0 ; 0,0,0,1,0 ; 1,0,0,0,0], %eps);
assertNumJacobian([y(2) + y(3) - y(4), y(2) * y(5), -yt([4,2]) .* yt([1,2])], 5, eye(4,4), ...
	[...
		0, 1, 1, -1,  0 ; ...
		0, 3, 0,  0, -1 ; ...
		2, 0, 0, -7,  0 ; ...
		0, 2, 0,  0,  0   ...
	], %eps);
assertNumJacobian([1,y(3),2] * [y([5,3,4]) , 2 * y([1,2,3]) + y([3,4,1])], 5, eye(2,2), ...
	[...
		0, 0,  2, 2, 1 ; ...
		4, 2,  1, 1, 0   ...
	], %eps);
assertNumGradient([y(5),2,y(1) + y(2)] * y([3,4,2]), 5, [-1,5,3,2,1], %eps);
assertNumGradient([y(5) + y(4)*y(3), 6] * [y([1,3]) , y([3,5]) , y([4,1]) + [2;3]] * y([1,2,3]), 5, [26,19,5,49,42], %eps);

// Mixed concatenation : gradient and jacobian.
assertNumJacobian([y(2), y(3) + 4*y(2), y(5)*y(4) ; -y(1), y(2) .* y(5), -y(3) - 2 ; -y(4) + y(1), 6 * y(1), 3] * ...
	[y(4) ; -2 ; -y(1)], 5, eye(3,3), ...
	[...
		  6, -10, -2, -22, 14 ; ...
		  5, - 6,  7, - 7,  2 ; ...
		-17,   0,  0,  11,  0   ...
	], %eps);
assertNumGradient([y(3),1,2] * [y(2), y(3) + 4*y(2), y(5)*y(4) ; -y(1), y(2) .* y(5), -y(3) - 2 ; -y(4) + y(1), 6 * y(1), 3] * ...
    [y(4) ; -2 ; -y(1)], 5, [-23,-16,55,-7,16], %eps);

// Insertion : jacobian.
A = [y([1,2,3]), [4,8,9,-1 ; yt([1,3,4,2]) ; 3,-1,y(5),-2]];
A(2,2) = y(5);
assertNumJacobian(A * [1;1;1;1;1], 5, eye(3,3), [1,0,0,0,0 ; 0,2,1,1,1 ; 0,0,1,0,1], %eps);
A($,$) = y(1) * y(3);
assertNumJacobian(A * [1;1;1;1;1], 5, eye(3,3), [1,0,0,0,0 ; 0,2,1,1,1 ; 1,0,8,0,1], %eps);
A(:,2) = y([5,1,2]);
assertNumJacobian(A * [1;1;1;1;1], 5, eye(3,3), [1,0,0,0,1 ; 1,2,1,1,0 ; 1,1,8,0,1], %eps);
A(2,6) = -y(4);
assertNumJacobian(A * [1;1;1;1;1;1], 5, eye(3,3), [1,0,0,0,1 ; 1,2,1,0,0 ; 1,1,8,0,1], %eps);
A(5,:) = [yt,2];
assertNumJacobian(A * [1;1;1;1;1;1], 5, eye(5,5), [1,0,0,0,1 ; 1,2,1,0,0 ; 1,1,8,0,1 ; 0,0,0,0,0 ; 1,1,1,1,1], %eps);


clear y;
clear yt;
clear A;

printf("Deriv. num. - Constructions : all unit tests passed.\n");


// End of script.

