//
// Tests on numerical evaluation of derivatives : element-wise product.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Scalars.
x = ad_ivar(3);
assertNumGradient(x .* x, 1, 6, %eps);
assertNumGradient(3 .* x .* 2 .* x .* x, 1, 162, %eps);

// Column vectors.
y = ad_ivar([7;-1;1;-2;3]);
assertNumJacobian(y .* y, 5, eye(5,5), [14,0,0,0,0 ; 0,-2,0,0,0 ; 0,0,2,0,0 ; 0,0,0,-4,0 ; 0,0,0,0,6], %eps);
assertNumJacobian(y(5:-1:1) .* y .* [1;5;3;2;3] .* y, 5, eye(5,5), ...
	[ ...
		42, 0,0, 0, 49 ; ...
		 0,20,0, 5,  0 ; ...
		 0, 0,9, 0,  0 ; ...
		 0, 8,0, 8,  0 ; ...
		27, 0,0, 0,126   ...
	], %eps);
assertNumJacobian(y(2) .* y, 5, [0,1,0,0,0 ; 0,0,1,0,0], [0,-2,0,0,0 ; 0,1,-1,0,0], %eps);

// Row vectors.
yt = y';
assertNumJacobian(yt .* yt, 5, eye(5,5), [14,0,0,0,0 ; 0,-2,0,0,0 ; 0,0,2,0,0 ; 0,0,0,-4,0 ; 0,0,0,0,6], %eps);
assertNumJacobian(yt(5:-1:1) .* yt .* [1,5,3,2,3] .* yt, 5, eye(5,5), ...
	[ ...
		42, 0,0, 0, 49 ; ...
		 0,20,0, 5,  0 ; ...
		 0, 0,9, 0,  0 ; ...
		 0, 8,0, 8,  0 ; ...
		27, 0,0, 0,126   ...
	], %eps);
assertNumJacobian(y(2) .* yt, 5, [0,0 ; 1,0 ; 0,1 ; 0,0 ; 0,0], [0,-2,0,0,0 ; 0,1,-1,0,0], %eps);

// Matrices.
assertNumJacobian(((y([1,2,3]) * yt([2,5,4])) .* (y([2,2,4]) * yt([1,3,1]))) * [1;1;1], 5, [0,0,1], [6,-14,30,-32,-2], %eps);


clear x;
clear y;
clear yt;

printf("Deriv. num. - Element-wise product : all unit tests passed.\n");


// End of script.
