//
// Tests on numerical evaluation of derivatives : exponentials and logarithms.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Regular exponential.
y = ad_ivar(2);
x = ad_ivar([7;-1;1;-2;3]);
assertNumGradient(3 ^ y, 1, 9 * log(3), 1e-7);
assertNumGradient(exp(y), 1, exp(2), 1e-7);
assertNumGradient(exp(x(2)) + exp(x(1) * x(4)) + exp(x(2)) * exp(x(5)) + exp(exp(x(3))), 5, ...
	[-0.0000017,7.7569355,41.193556,0.0000058,7.3890561], 1e-6);

// Element-wise exponential.
assertNumJacobian(2 .^ x, 5, eye(5,5), log(2) * [128,0,0,0,0 ; 0,1/2,0,0,0 ; 0,0,2,0,0 ; 0,0,0,1/4,0 ; 0,0,0,0,8], 1e-7);
assertNumJacobian(exp(x([3,5,1])) .* 1.5 .^ [x(3);7;x(4)], 5, eye(3,3), ...
	[...
		  0      ,0,5.7306754,  0      ,  0       ; ...
		  0      ,0,0        ,  0      ,343.18023 ; ...
		487.39251,0,0        ,197.62066,  0         ...
	], 1e-5);

// Element-wise logarithm.
assertNumGradient(log(y), 1, 1/2, %eps);
assertNumGradient(log(exp(x(1))) + exp(log(x(1))) + log(x(3)*x(5) + log(-x(4))), 5, [2,0,0.8123153,-0.1353859,0.2707718], 1e-7);
assertNumJacobian(log(x .* [4 ; x(2)-x(5) ; x(3)^3 ; exp(x(4)) ; log(x(5)^2)]), 5, eye(5,5), ...
	[ ...
		1/7, 0   , 0, 0  , 0         ; ...
	 	0  ,-5/4 , 0, 0  , 1/4       ; ...
	 	0  , 0   , 4, 0  , 0         ; ...
	 	0  , 0   , 0, 1/2, 0         ; ...
	 	0  , 0   , 0, 0  , 0.6367464   ...
	], 1e-7);


clear x;
clear y;

printf("Deriv. num. - Exponentials and logarithms : all unit tests passed.\n");


// End of script.

