//
// Tests on numerical evaluation of derivatives : monomes and inversions.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Scalar regular monomes and inversions.
y = ad_ivar(3);
x = ad_ivar([7;-1;1;-2;3]);
assertNumGradient(inv(y), 1, -1/9, %eps);
assertNumGradient((x(2) * x(3)) / (x(1) + x(5) * x(2)), 5, [1/16, 7/16, -1/4, 0, -1/16], %eps);
assertNumGradient((x(1) + x(5) * x(2)) \ (x(2) * x(3)), 5, [1/16, 7/16, -1/4, 0, -1/16], %eps);
assertNumGradient(y^3 - 3*y^2 + 5*y + 2, 1, 14, %eps);
assertNumGradient(x(5)^2.5 * x(1)^(-4), 5, [-0.0037100,0,0,0,0.0054104], 1e-7);

// Element-wise monomes and inversions.
assertNumJacobian(1 ./ x, 5, eye(5,5), ...
	[...
		-1/49, 0, 0, 0  , 0   ; ...
		 0   ,-1, 0, 0  , 0   ; ...
		 0   , 0,-1, 0  , 0   ; ...
		 0   , 0, 0,-1/4, 0   ; ...
		 0   , 0, 0, 0  ,-1/9   ...
	], %eps);
assertNumJacobian(x(2) ./ (x(1:3) + x(5:-1:3)), 5, eye(3,3), ...
	[...
		1/100,  1/10, 0  , 0  , 1/100 ; ...
		0    , -2/9 , 0  , 1/9, 0     ; ...
		0    ,  1/2 , 1/2, 0  , 0       ...
	], %eps);
assertNumJacobian((x + x(5:-1:1)) .^ [-2;0;3.5;-1;-2.5], 5, eye(5,5), ...
	[...
		-0.002    , 0,  0       , 0  ,-0.002     ; ...
		 0        , 0  ,0       , 0  , 0         ; ...
		 0        , 0  ,39.59798, 0  , 0         ; ...
		 0        ,-1/9,0       ,-1/9, 0         ; ...
		-0.0007906, 0  ,0       , 0  ,-0.0007906   ...
	], 1e-5);


clear x;
clear y;

printf("Deriv. num. - Monomes and inversions : all unit tests passed.\n");


// End of script.

