//
// Tests on numerical evaluation of derivatives : regular product.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


x = ad_ivar(2);
y = ad_ivar([7;-1;1;-2;3]);

// Scalar * Scalar rule : gradient.
assertNumGradient(5 * x, 1, 5, %eps);
assertNumGradient(x * x, 1, 4, %eps);
assertNumGradient(y(1) * y(3) - y(2) * y(2) * y(5), 5, [1,6,7,0,-1], %eps);

// Scalar * Vector rule : jacobian.
assertNumJacobian(5 * y', 5, eye(5,5), 5 * eye(5,5), %eps);
assertNumJacobian(y(2) * y', 5, eye(5,5), [-1,7,0,0,0 ; 0,-2,0,0,0 ; 0,1,-1,0,0 ; 0,-2,0,-1,0 ; 0,3,0,0,-1], %eps);
assertNumJacobian(y(3) * y(1) * y', 5, [1,0 ; 0,0 ; 0,0 ; 0,1 ; 0,0], [14,0,49,0,0 ; -2,0,-14,7,0], %eps);

// Vector * Scalar rule : jacobian.
assertNumJacobian(y * 5, 5, eye(5,5), 5 * eye(5,5), %eps);
assertNumJacobian(y * y(2), 5, eye(5,5), [-1,7,0,0,0 ; 0,-2,0,0,0 ; 0,1,-1,0,0 ; 0,-2,0,-1,0 ; 0,3,0,0,-1], %eps);
assertNumJacobian(y * y(1) * y(3), 5, [1,0,0,0,0 ; 0,0,0,1,0], [14,0,49,0,0 ; -2,0,-14,7,0], %eps);

// Inner product rule : gradient.
assertNumGradient(y' * y, 5, [14,-2,2,-4,6], %eps);
assertNumGradient(y([4,5,2])' * y([1,5,3]), 5, [-2,1,-1,7,6], %eps);

// Matrix * Vector and Vector * Matrix rules : jacobian.
A = matrix(1:25, [5,5]);
assertNumJacobian(A * y, 5, eye(5,5), A, %eps);
assertNumJacobian(y([4,3,4,1,2])' * A, 5, [0,0 ; 0,1 ; 1,0 ; 0,0 ; 0,0], [14,15,12,24,0 ; 9,10,7,14,0], %eps);

// External product for variable matrices; Matrix * Vector rules : jacobian.
assertNumJacobian((y([1,2]) * [1,0,0] + y([2,3]) * [0,1,0] + y([4,5]) * [0,0,1]) * [3;-2;-1], 5, eye(2,2), ...
	[...
		3,-2, 0,-1, 0 ; ...
		0, 3,-2, 0,-1   ...
	], %eps);
assertNumJacobian((y([1,2]) * [1,0,0] + y([2,3]) * [0,1,0] + y([4,5]) * [0,0,1]) * y([5,3,4]), 5, eye(2,2), ...
	[...
		 3,  1, -1, -4,  7 ; ...
		 0,  3,  2,  3, -3   ...
	], %eps);
yt = y';
assertNumJacobian(y([1,4,2]) * yt([1,3]) * y([5,3]), 5, eye(3,3), ...
	[...
		 43,   0,  14,   0,  49 ; ...
		- 6,   0, - 4,  22, -14 ; ...
		- 3,  22, - 2,   0, - 7   ...
	], %eps);

// External product for variable matrices; Vector * Matrix rules : jacobian.
assertNumJacobian([3,-2,-1] * (y([1,2]) * [1,0,0] + y([2,3]) * [0,1,0] + y([4,5]) * [0,0,1])', 5, eye(2,2), ...
	[...
		3,-2, 0,-1, 0 ; ...
		0, 3,-2, 0,-1   ...
	], %eps);
assertNumJacobian(yt([5,3,4]) * (y([1,2]) * [1,0,0] + y([2,3]) * [0,1,0] + y([4,5]) * [0,0,1])', 5, eye(2,2), ...
	[...
		 3,  1, -1, -4,  7 ; ...
		 0,  3,  2,  3, -3   ...
	], %eps);
yt = y';
assertNumJacobian(yt([5,3]) * (y([1,4,2]) * yt([1,3]))', 5, eye(3,3), ...
	[...
		 43,   0,  14,   0,  49 ; ...
		- 6,   0, - 4,  22, -14 ; ...
		- 3,  22, - 2,   0, - 7   ...
	], %eps);

// External product for variable matrices; Matrix * Matrix rules : jacobian.
Y1 = y([1,2,5]) * yt([3,2]);
Y2 = y([4,1]) * yt([1,3,5,4]);
assertNumJacobian((Y1 * Y2) * [1;-2;-1;3], 5, [1,0,0], [1,-196,182,-217,63], %eps);
assertNumJacobian((y([1,2])' * [-1,0,0 ; 0,0,-1] + [0,2,0]) * (Y1 * Y2), 5, [0;1;0;0], [174,-327,528,-48,-9], %eps);

// Other tests.
assertNumGradient(0.5 * y' * [2,1,3,-2,-5 ; 1,3,-8,0,0 ; 3,-8,1,-3,4 ; -2,0,-3,4,9 ; -5,0,4,9,1] * y, 5, [5,-4,48,2,-46], %eps);
assertNumGradient(([2,0,0,0,0 ; 0,-1,0,0,0 ; 0,0,3,0,0 ; 0,0,0,1,0 ; 0,0,0,0,-4] * y)' * y(5:-1:1), 5, [-6,0,6,0,-14], %eps);
assertNumGradient((yt([1,5,3]) * y([4,1,3])) * (yt([5,5]) * y([2,3])), 5, [0,24,24,0,0], %eps);
assertNumGradient(yt([1,5,3]) * (y([4,1,3]) * yt([5,5])) * y([2,3]), 5, [0,24,24,0,0], %eps);


clear x;
clear y;
clear A;
clear yt;

printf("Deriv. num. - Regular product : all unit tests passed.\n");


// End of script.

