//
// Tests on re-evaluation of expression graphs.
//
// Author : Benoit Hamelin
// 2004 Universite de Sherbrooke
//


// Simple variables.
x = ad_ivar([7;-1;1;-2;3]);
fx = x(2);
assertReeval(fx, [3;5;1;0;2], 5, %eps);
assertReeval(fx, [7;90;-89;1;0], 90, %eps);
fx = x;
assertReeval(fx, [8;-2;0;0;1], [8;-2;0;0;1], %eps);
assertReeval(fx, [7;90;-89;1;0], [7;90;-89;1;0], %eps);
fx = x([5,1,$]);
assertReeval(fx, [3;4;1;-2;0], [0;3;0], %eps);
assertReeval(fx, [-5;67;8;12;-9], [-9;-5;-9], %eps);

// Sums and constants.
fx = x(3) - 6 + x(5);
assertReeval(fx, [3;1;0;5;-7], -13, %eps);
assertReeval(fx, [1;7;8;4;-9], -7, %eps);
fx = x + [8;2;5;0;-2] + x([4;3;3;1;5]) + x([2;3;1;5;2]);
assertReeval(fx, [8;0;-2;-3;10], [13;-2;9;15;18], %eps);
assertReeval(fx, [10;4;-5;0;3], [22;-4;5;13;8], %eps);

// Additive inverse.
fx = -x([4,2]);
assertReeval(fx, [5;0;3;-4;6], [4;0], %eps);
assertReeval(fx, [9;2;0;-5;6], [5;-2], %eps);
fx = x(3) + x(5) - x(1) + x(3) - x(4) + 3;
assertReeval(fx, [2;9;0;1;-3], -3, %eps);
assertReeval(fx, [12;-4;8;-9;2], 18, %eps);

// Transposition.
fx = x';
assertReeval(fx, [5;7;8;0;-2], [5,7,8,0,-2], %eps);
assertReeval(fx, [0;2;-3;4;5], [0,2,-3,4,5], %eps);
fx = (x([5,2,3])' + [4,0,2] - x([1,4,2])')';
assertReeval(fx, [6;0;-9;3;-1], [-3;-3;-7], %eps);
assertReeval(fx, [1;-5;13;-3;8], [11;-2;20], %eps);

// Regular product.
fx = x(3) * x(4) - x(1) * x(2) * x(4);
assertReeval(fx, [6;8;-2;5;3], -250, %eps);
assertReeval(fx, [2;-2;3;1;0], 7, %eps);
fx = x * (5 * x(2));
assertReeval(fx, [8;9;0;-4;-2], [360;405;0;-180;-90], %eps);
assertReeval(fx, [-2;3;1;4;-1], [-30;45;15;60;-15], %eps);
fx = (2 - x(3)) * x([4,3,1,5,2])';
assertReeval(fx, [9;0;-3;3;-1], [15,-15,45,-5,0], %eps);
assertReeval(fx, [8;-1;-2;2;3], [8,-8,32,12,-4], %eps);
fx = x' * x;
assertReeval(fx, [-1;-3;0;-5;2], 39, %eps);
assertReeval(fx, [5;-6;-2;-2;3], 78, %eps);
fx = x * x';
assertReeval(fx, [8;0;-2;3;1], ...
	[...
		 64,  0, -16,  24,  8 ; ...
		  0,  0,   0,   0,  0 ; ...
		-16,  0,   4, - 6, -2 ; ...
		 24,  0, - 6,   9,  3 ; ...
		  8,  0, - 2,   3,  1   ...
	], %eps);
assertReeval(fx, [10;3;-4;-6;2], ...
	[...
		 100,  30, -40, -60,  20 ; ...
		  30,   9, -12, -18,   6 ; ...
		- 40, -12,  16,  24, - 8 ; ...
		- 60, -18,  24,  36, -12 ; ...
		  20,   6, - 8, -12,   4   ...
	], %eps);
fx = (x([3,5]) * x([1;2;4])') * [5;-1;3];
assertReeval(fx, [5;0;2;9;-3], [104;-156], %eps);
assertReeval(fx, [-3;4;1;2;-5], [-13;65], %eps);
fx = x([3,2,1])' * ([2;-1;4] * x([1,3,2,5])');
assertReeval(fx, [6;-2;-3;-4;1], [120,-60,-40,20], %eps);
assertReeval(fx, [7;-3;-5;1;-2], [147,-105,-63,-42], %eps);
fx = (x([1,3]) * [3,-2,1]) * ([-2;4;-1] * x([4,5])');
assertReeval(fx, [-8;2;1;4;-3], [480,-360 ; -60,45], %eps);
assertReeval(fx, [2;0;-4;3;-1], [-90,30 ; 180,-60], %eps);

// Concatenations, extraction, insertion.
fx = [x([2,3,5]), [5 ; x(2)*x(4) ; x(3) - x(1)]];
assertReeval(fx, [2;4;7;-1;-2], [4, 5 ; 7, -4 ; -2, 5], %eps);

fx = x - [x(2)+3 ; 4 ; x(3)*x(1) ; x(2) - 3*x(3) ; 9 + x(4)];
extr = fx(1);
assertReeval(extr, [6;0;-9;2;-3], 3, %eps);
assertReeval(extr, [-8;2;-3;3;5], -13, %eps);
extr = fx($);
assertReeval(extr, [6;0;-9;2;-3], -14, %eps);
assertReeval(extr, [-8;2;-3;3;5], -7, %eps);
extr = fx([4,3]);
assertReeval(extr, [6;0;-9;2;-3], [-25;45], %eps);
assertReeval(extr, [-8;2;-3;3;5], [-8;-27], %eps);
extr = fx(:);
assertReeval(extr, [6;0;-9;2;-3], [3;-4;45;-25;-14], %eps);
assertReeval(extr, [-8;2;-3;3;5], [-13;-2;-27;-8;-7], %eps);
extr = fx(1,1);
assertReeval(extr, [6;0;-9;2;-3], 3, %eps);
assertReeval(extr, [-8;2;-3;3;5], -13, %eps);
extr = fx($,:);
assertReeval(extr, [6;0;-9;2;-3], -14, %eps);
assertReeval(extr, [-8;2;-3;3;5], -7, %eps);
extr = fx([4,3], $);
assertReeval(extr, [6;0;-9;2;-3], [-25;45], %eps);
assertReeval(extr, [-8;2;-3;3;5], [-8;-27], %eps);
extr = fx(:, :);
assertReeval(extr, [6;0;-9;2;-3], [3;-4;45;-25;-14], %eps);
assertReeval(extr, [-8;2;-3;3;5], [-13;-2;-27;-8;-7], %eps);
fx = fx';
extr = fx(1);
assertReeval(extr, [6;0;-9;2;-3], 3, %eps);
assertReeval(extr, [-8;2;-3;3;5], -13, %eps);
extr = fx($);
assertReeval(extr, [6;0;-9;2;-3], -14, %eps);
assertReeval(extr, [-8;2;-3;3;5], -7, %eps);
extr = fx([4,3]);
assertReeval(extr, [6;0;-9;2;-3], [-25,45], %eps);
assertReeval(extr, [-8;2;-3;3;5], [-8,-27], %eps);
extr = fx(:);
assertReeval(extr, [6;0;-9;2;-3], [3;-4;45;-25;-14], %eps);
assertReeval(extr, [-8;2;-3;3;5], [-13;-2;-27;-8;-7], %eps);
extr = fx(1,1);
assertReeval(extr, [6;0;-9;2;-3], 3, %eps);
assertReeval(extr, [-8;2;-3;3;5], -13, %eps);
extr = fx(:,$);
assertReeval(extr, [6;0;-9;2;-3], -14, %eps);
assertReeval(extr, [-8;2;-3;3;5], -7, %eps);
extr = fx($, [4,3]);
assertReeval(extr, [6;0;-9;2;-3], [-25,45], %eps);
assertReeval(extr, [-8;2;-3;3;5], [-8,-27], %eps);
extr = fx(:, :);
assertReeval(extr, [6;0;-9;2;-3], [3,-4,45,-25,-14], %eps);
assertReeval(extr, [-8;2;-3;3;5], [-13,-2,-27,-8,-7], %eps);

fx = x([3,4,2]) * x([5,1,2])' + [5,x(2),0 ; -2,x(3),x(5) ; x(1),2,3];
extr = fx(2, 3);
assertReeval(extr, [6;0;-9;2;-3], -3, %eps);
assertReeval(extr, [-8;2;-3;3;5], 11, %eps);
extr = fx([3,1], [2,1]);
assertReeval(extr, [6;0;-9;2;-3], [2, 6 ; -54, 32], %eps);
assertReeval(extr, [-8;2;-3;3;5], [-14, 2 ; 26, -10], %eps);
extr = fx(:, 2);
assertReeval(extr, [6;0;-9;2;-3], [-54 ; 3 ; 2], %eps);
assertReeval(extr, [-8;2;-3;3;5], [26 ; -27 ; -14], %eps);
extr = fx($, :);
assertReeval(extr, [6;0;-9;2;-3], [6, 2, 3], %eps);
assertReeval(extr, [-8;2;-3;3;5], [2, -14, 7], %eps);
assert(and(Expr_Reevaluate(fx, [6;0;-9;2;-3]) == ...
		Expr_Reevaluate(fx(:,:), [6;0;-9;2;-3])));
assert(and(Expr_Reevaluate(fx, [-8;2;-3;3;5]) == ...
		Expr_Reevaluate(fx(:,:), [-8;2;-3;3;5])));

fx(3,1) = 1000;
assertReeval(fx, [6;0;-9;2;-3], [32,-54,0 ; -8,3,-3 ; 1000,2,3], %eps);
assertReeval(fx, [-8;2;-3;3;5], [-10,26,-6 ; 13,-27,11 ; 1000,-14,7], %eps);
fx(5, [2,4]) = [1000*x(3), 2000];
assertReeval(fx, [6;0;-9;2;-3], ...
	[...
		   32,-  54, 0,    0 ; ...
		-   8,    3,-3,    0 ; ...
		 1000,    2, 3,    0 ; ...
		    0,    0, 0,    0 ; ...
			0,-9000, 0, 2000   ...
	], %eps);
assertReeval(fx, [-8;2;-3;3;5], ...
	[...
		-  10,    26, - 6,    0 ; ...
		   13, -  27,  11,    0 ; ...
		 1000, -  14,   7,    0 ; ...
		    0,     0,   0,    0 ; ...
		    0, -3000,   0, 2000   ...
	], %eps);
fx(:, 4) = 100*x;
assertReeval(fx, [6;0;-9;2;-3], ...
	[...
		   32,-  54, 0,  600 ; ...
		-   8,    3,-3,    0 ; ...
		 1000,    2, 3,- 900 ; ...
		    0,    0, 0,  200 ; ...
			0,-9000, 0,- 300   ...
	], %eps);
assertReeval(fx, [-8;2;-3;3;5], ...
	[...
		-  10,    26, - 6,- 800 ; ...
		   13, -  27,  11,  200 ; ...
		 1000, -  14,   7,- 300 ; ...
		    0,     0,   0,  300 ; ...
		    0, -3000,   0,  500   ...
	], %eps);
fx($, :) = -x([1:4])';
assertReeval(fx, [6;0;-9;2;-3], ...
	[...
		   32,-  54, 0,  600 ; ...
		-   8,    3,-3,    0 ; ...
		 1000,    2, 3,- 900 ; ...
		    0,    0, 0,  200 ; ...
		-   6,    0, 9,-   2   ...
	], %eps);
assertReeval(fx, [-8;2;-3;3;5], ...
	[...
		-  10,    26, - 6,- 800 ; ...
		   13, -  27,  11,  200 ; ...
		 1000, -  14,   7,- 300 ; ...
		    0,     0,   0,  300 ; ...
		    8, -   2,   3,-   3   ...
	], %eps);

// Element-wise product.
fx = x .* x([2,3,4,5,1]);
assertReeval(fx, [5;1;-3;-4;2], [5;-3;12;-8;10], %eps);
assertReeval(fx, [6;-2;-1;-3;6], [-12;2;3;-18;36], %eps);

// Inversion.
fx = inv(x(3));
assertReeval(fx, [4;2;-3;2;1], -1/3, 1e-12);
assertReeval(fx, [8;-9;2;0;1], 1/2, 1e-12);
fx = inv([x(2), x(1), x(3) ; x(3), x(4), x(5) ; x(5), x(2), x(1)]);
assertReeval(fx, [1;2;-3;4;5], ...
	[ ...
		-0.0638298, -0.0744681,  0.1808511 ; ...
		 0.2978723,  0.1808511, -0.0106383 ; ...
		-0.2765957,  0.0106383,  0.1170213   ...
	], 1e-7);
assertReeval(fx, [-2;6;7;2;8], ...
	[ ...
	   0.1818182, -0.1328671,  0.1048951 ; ...
	  -0.2727273,  0.2377622, -0.0034965 ; ...
	  -0.0909091,  0.1818182, -0.0909091   ...
	], 1e-7);

// Element-wise inversion.
fx = [x(3) ; x(1)*x(2) ; 5 + x(1) ; 7 ; -2*x(4)] ./ x;
assertReeval(fx, [4;-6;2;1;-2], [1/2 ; 4 ; 9/2 ; 7 ; 1], %eps);
assertReeval(fx, [7;-2;-1;5;6], [-1/7 ; 7 ; -12 ; 7/5 ; -5/3], 1e-12);

// Exponentials.
fx = 2 ^ (x(2) * x(3));
assertReeval(fx, [8;-1;2;5;0], 1/4, %eps);
assertReeval(fx, [1;3;4;5;0], 4096, %eps);

// Element-wise exponentials.
fx = exp(x - x([2,3,4,5,1]));
assertReeval(fx, [6;0;-1;2;6], exp([6;1;-3;-4;0]), 1e-12);
assertReeval(fx, [2;-2;5;0;-3], exp([4;-7;5;3;-5]), 1e-12);
fx = [1;3;%e;1/2] .^ [x(1) - x(2) ; x(2)*x(3) ; x(4)/x(5) ; -x(3)];
assertReeval(fx, [2;5;-3;1;6], [1 ; 3^(-15) ; exp(1/6) ; 2^(-3)], 1e-12);
assertReeval(fx, [6;9;2;-3;4], [1 ; 3^18 ; exp(-3/4) ; 4], 1e-12);

// Logarithms (element-wise).
fx = log(x([3,2,5]) .* [5 ; x(4) - 3 ; -x(2)/x(1)]);
assertReeval(fx, [5;-3;8;0;2], log([40 ; 9 ; 6/5]), 1e-12);
assertReeval(fx, [9;-1;4;-2;1/2], log([20 ; 5 ; 1/18]), 1e-12);

// Monomes.
fx = (x(5) + x(1)*x(2))^4;
assertReeval(fx, [6;-2;0;-1;0], 81*256, %eps);
assertReeval(fx, [-2;3;7;0;9], 81, %eps);
fx = (x(5) + x(1)*x(2))^(-4);
assertReeval(fx, [6;-2;0;-1;0], inv(81*256), %eps);
assertReeval(fx, [-2;3;7;0;9], inv(81), %eps);
fx = (x(4) + x(3)/x(2))^(-3.5);
assertReeval(fx, [6;8;-2;3;0], (2+3/4)^(-3.5), 1e-12);
assertReeval(fx, [9;2;10;-2;0], 1/(27*sqrt(3)), 1e-12);

// Element-wise monomes.
fx = sqrt(x');
assertReeval(fx, [6;10;4;0;3], [sqrt(6), sqrt(10), 2, 0, sqrt(3)], 1e-12);
assertReeval(fx, [8;16;7;9;36], [2*sqrt(2), 4, sqrt(7), 3, 6], 1e-12);
fx = [x(3); x(2)*x(4) ; x(5) - x(3) ; 2^x(1)] .^ [0 ; -4 ; 1/3 ; 5];
assertReeval(fx, [2;9;-1;3;8], [1 ; 27^(-4) ; 9^(1/3) ; 1024], 1e-12);
assertReeval(fx, [-3;4;2;-1;10], [1 ; 1/256 ; 2 ; 1/32768], 1e-12);

// Circular trigonometrics.
fx = sin([x(1)^2 ; x(3)+x(4) ; x(5)/x(1) ; x(2)]);
assertReeval(fx, [6;0;-2;7;-9], [sin([36 ; 5 ; -3/2]) ; 0], 1e-12);
assertReeval(fx, [3 ; -%pi/4 ; 0 ; -%pi/3 ; %pi/2], ...
		[sin(9) ; -sqrt(3)/2 ; 1/2 ; -sqrt(2)/2], 1e-12);
fx = cos([x(1)^2 ; x(3)+x(4) ; x(5)/x(1) ; x(2)]);
assertReeval(fx, [6;0;-2;7;-9], [cos([36 ; 5 ; -3/2]) ; 1], 1e-12);
assertReeval(fx, [3 ; -%pi/4 ; 0 ; -%pi/3 ; %pi/2], ...
		[cos(9) ; 1/2 ; sqrt(3)/2 ; sqrt(2)/2], 1e-12);
fx = atan(x);
assertReeval(fx, [-1 ; sqrt(3)/3 ; 100000000 ; 0 ; -sqrt(3)], ...
		[-%pi/4 ; %pi/6 ; %pi/2 ; 0 ; -%pi/3], 1e-7);
assertReeval(fx, [-100000000 ; sqrt(3) ; 2 ; 1 ; -sqrt(3)/3], ...
		[-%pi/2 ; %pi/3 ; atan(2) ; %pi/4 ; -%pi/6], 1e-7);

// Absolute value.
fx = abs([x(1) * x(3) - x(4) , x(5)^3 ; x(2) - x(4)^4 , exp(x(1)*x(5))]);
assertReeval(fx, [6;0;3;-2;-4], [20, 64 ; 16, exp(-24)], 1e-12);
assertReeval(fx, [-3;1;2;7;5], [13, 125 ; 2400, exp(-15)], 1e-12);


clear x;
clear fx;
clear extr;

printf("Graph re-evaluation : all unit tests passed.\n");


// End of script.

