//
// Tests : expression nodes.
//
// Author : Benoit Hamelin
// 2002 Universit de Sherbrooke
//


// Independant variable node.
n = makeNodeIVar(1, 5);
assert(n.display == "Indep. variable (1)");
assert(and(size(n) == [1,1]));
assert(n.value == 5);
n = makeNodeIVar(1:5, 5:-1:1);
assert(n.display == "Indep. variable [1,2,3,4,5]");
assert(and(size(n) == [5,1]));
assert(and(n.value == [5;4;3;2;1]));
n = makeNodeIVar([1,3;5,7], [10,11 ; 12,13]);
assert(n.display == "Indep. variable [1,5,3,7]");
assert(and(size(n) == [4,1]));
assert(and(n.value == [10;12;11;13]));


// Constant node.
n = makeNodeConst(0);
assert(n.value == 0);
assert(and(size(n) == [1,1]));
n = makeNodeConst([2,3 ; 4,5]);
assert(and(n.value == [2,3 ; 4,5]));
assert(and(size(n) == [2,2]));
n = makeNodeConst(hypermat([2,2,2], 1:8));
sn = size(n.value);
assert(and(n.value(:)' == (1:8)));
assert(and(size(n) == [2,2,2]));

// Tests on node size acquisition.
n = makeNodeConst([2,3,4 ; 5,6,7]);
[l,c] = size(n);
assert((l == 2) & (c == 3));
assert(size(n, "r") == 2);
assert(size(n, "c") == 3);
assert(size(n, "*") == 6);
assert(size(n, 1) == 2);
assert(size(n, 2) == 3);
n = makeNodeConst(hypermat([2,2,3], 1:12));
assert(size(n, 2) == 2);
assert(size(n, 3) == 3);


// Addition node.
n = makeNodeAdd([3,4;5,8]);
assert(and(size(n) == [2,2]));
assert(and(n.value == [3,4;5,8]));


clear n;
clear sn;
clear s1;
clear s2;
clear l;
clear c;

printf("ExprNode : all unit tests passed.\n");


// End of module.

