//
// Tests : addition of expression hypermatrices.
//
// Author : Benoit Hamelin
// 2002 Universit de Sherbrooke
//


// Addition and unary plus.
x = ad_ivar(1);
px = +x;
assert(and(size(x) == size(px)));
assert(Expr_GetEvaluation(x) == Expr_GetEvaluation(px));
assert(Expr_GetNodeCount(x) == Expr_GetNodeCount(px));

y = ad_ivar([1;3;-2]);
py = +y;
assert(and(size(y) == size(py)));
assert(and(Expr_GetEvaluation(y) == Expr_GetEvaluation(py)));
assert(Expr_GetNodeCount(y) == Expr_GetNodeCount(py));

z = y + y;
assert(and(size(z) == size(y)));
assert(and(Expr_GetEvaluation(z) == [2;6;-4]));
sz = z([1,3]);
assert(and(size(sz) == [2,1]));
assert(and(Expr_GetEvaluation(sz) == [2;-4]));
z = (z + (+y)) + y;
assert(and(size(z) == [3,1]));
assert(and(Expr_GetEvaluation(z) == [4;12;-8]));

z = y + [3;2;4];
assert(and(size(z) == [3,1]));
assert(and(Expr_GetEvaluation(z) == [4;5;2]));

z = y + [0;0;0];
assert(and(size(z) == size(y)));
assert(Expr_GetNodeCount(z) == 1);
assert(and(Expr_GetEvaluation(z) == Expr_GetEvaluation(y)));
z = [0;0;0] + y;
assert(and(size(z) == size(y)));
assert(Expr_GetNodeCount(z) == 1);
assert(and(Expr_GetEvaluation(z) == Expr_GetEvaluation(y)));

z = y' + [60,80,100];
pz = z(2:$);
assert(and(size(pz) == [1,2]));
assert(and(Expr_GetEvaluation(pz) == [83,98]));

m = ad_const([1,2,3 ; 4,5,6 ; 7,8,9 ; 10,11,12]);
mm = m + m;
assert(size(mm) == size(m));
assert(and(Expr_GetEvaluation(mm) == 2 .* [1,2,3 ; 4,5,6 ; 7,8,9 ; 10,11,12]));
mm1 = mm([2,4], 2:3);
assert(and(size(mm1) == [2 , 2]));
assert(and(Expr_GetEvaluation(mm1) == 2 .* [5,6 ; 11,12]));

mm = ad_const([1,2,3 ; 4,5,6 ; 7,8,9; 10,11,12]);
mm = mm + mm;
mmt = mm';
assert(and(size(mmt) == [3,4]));
assert(and(Expr_GetEvaluation(mmt) == [2,8,14,20 ; 4,10,16,22 ; 6,12,18,24]));
assert(and(Expr_GetEvaluation(mmt(:,[2,4])) == [8,20 ; 10,22 ; 12,24]));

z = ad_const([0,0,0]);
s = z + z;
assert(and(size(s) == [1,3]));
assert(and(Expr_GetEvaluation(s) == [0,0,0]));
r = ad_const([-2,3,1]);
mr = ad_const([2,-3,-1]);
s = r + mr;
assert(and(size(s) == [1,3]));
assert(and(Expr_GetEvaluation(s) == [0,0,0]));

assertError("[1;2;3] + mm");

clear x;
clear px;
clear y;
clear py;
clear z;
clear pz;
clear m;
clear mm;
clear mm1;
clear s;
clear r;
clear mr;

printf("Expr - Addition : all unit tests passed.\n");


// End of module.

