//
// Tests : construction of an expression.
//
// Author : Benoit Hamelin
// 2002 Universit de Sherbrooke
//


// Simple expression.
expr = makeExpr(makeNodeIVar(1, 7));
assert(Expr_GetNodeCount(expr) == 1);
assert(Expr_GetPredecessors(expr, 1) == []);
assert(Expr_GetSuccessors(expr, 1) == []);
assert(Expr_GetEdges(expr) == []);
assert(Expr_GetEvaluation(expr) == 7);
expr = makeExpr(makeNodeIVar(1:5, 5:9));
assert(Expr_GetNodeCount(expr) == 1);
assert(Expr_GetPredecessors(expr, 1) == []);
assert(Expr_GetSuccessors(expr, 1) == []);
assert(Expr_GetEdges(expr) == []);
assert(and(Expr_GetEvaluation(expr) == [5;6;7;8;9]));


// Expression extension.
expr = extendExpr(expr, makeNodeConst(matrix(1:25, [5,5])));
assert(Expr_GetNodeCount(expr) == 2);
assert(and(Expr_GetPredecessors(expr, 1) == []));
assert(and(Expr_GetPredecessors(expr, 2) == [1]));
assert(and(Expr_GetSuccessors(expr, 1) == [2]));
assert(and(Expr_GetSuccessors(expr, 2) == []));
assert(and(Expr_GetEdges(expr) == [1 ; 2]));
assert(and(Expr_GetEvaluation(expr) == matrix(1:25, [5,5])));

expr = extendExpr(expr, makeNodeAdd([56,57]));
assert(Expr_GetNodeCount(expr) == 3);
assert(and(Expr_GetPredecessors(expr, 1) == []));
assert(and(Expr_GetPredecessors(expr, 2) == [1]));
assert(and(Expr_GetPredecessors(expr, 3) == [2]));
assert(and(Expr_GetSuccessors(expr, 1) == [2]));
assert(and(Expr_GetSuccessors(expr, 2) == [3]));
assert(and(Expr_GetSuccessors(expr, 3) == []));
assert(and(Expr_GetEdges(expr) == [1,2 ; 2,3]));
assert(and(Expr_GetEvaluation(expr) == [56,57]));

expr = extendExpr(extendExpr(expr, makeNodeAdd(3)), makeNodeAdd([23,34;45,56]));
assert(Expr_GetNodeCount(expr) == 5);
assert(and(Expr_GetPredecessors(expr, 1) == []));
assert(and(Expr_GetPredecessors(expr, 2) == [1]));
assert(and(Expr_GetPredecessors(expr, 3) == [2]));
assert(and(Expr_GetPredecessors(expr, 4) == [3]));
assert(and(Expr_GetPredecessors(expr, 5) == [4]));
assert(and(Expr_GetSuccessors(expr, 1) == [2]));
assert(and(Expr_GetSuccessors(expr, 2) == [3]));
assert(and(Expr_GetSuccessors(expr, 3) == [4]));
assert(and(Expr_GetSuccessors(expr, 4) == [5]));
assert(and(Expr_GetSuccessors(expr, 5) == []));
assert(and(Expr_GetEdges(expr) == [1,2,3,4 ; 2,3,4,5]));
assert(and(Expr_GetEvaluation(expr) == [23,34;45,56]));


// Expression composition.
expr = composeExpr(makeExpr(makeNodeIVar(1:5, 1:5)), makeExpr(makeNodeConst([1,2;3,4])), makeNodeAdd(88));
assert(Expr_GetNodeCount(expr) == 3);
assert(and(Expr_GetPredecessors(expr, 1) == []));
assert(and(Expr_GetPredecessors(expr, 2) == []));
assert(and(Expr_GetPredecessors(expr, 3) == [1,2]));
assert(and(Expr_GetSuccessors(expr, 1) == [3]));
assert(and(Expr_GetSuccessors(expr, 2) == [3]));
assert(and(Expr_GetSuccessors(expr, 3) == []));
assert(and(Expr_GetEdges(expr) == [1,2 ; 3,3]));
assert(and(Expr_GetEvaluation(expr) == 88));

expr = composeExpr(expr, extendExpr(makeExpr(makeNodeConst(3)), makeNodeAdd(2)), makeNodeAdd(88));
assert(Expr_GetNodeCount(expr) == 6);
assert(and(Expr_GetPredecessors(expr, 1) == []));
assert(and(Expr_GetPredecessors(expr, 2) == []));
assert(and(Expr_GetPredecessors(expr, 3) == [1,2]));
assert(and(Expr_GetPredecessors(expr, 4) == []));
assert(and(Expr_GetPredecessors(expr, 5) == [4]));
assert(and(Expr_GetPredecessors(expr, 6) == [3,5]));
assert(and(Expr_GetSuccessors(expr, 1) == [3]));
assert(and(Expr_GetSuccessors(expr, 2) == [3]));
assert(and(Expr_GetSuccessors(expr, 3) == [6]));
assert(and(Expr_GetSuccessors(expr, 4) == [5]));
assert(and(Expr_GetSuccessors(expr, 5) == [6]));
assert(and(Expr_GetSuccessors(expr, 6) == []));
assert(and(Expr_GetEdges(expr) == [1,2,4,3,5 ; 3,3,5,6,6]));
assert(and(Expr_GetEvaluation(expr) == 88));


clear expr;

printf("Expr - Base : all unit tests passed.\n");


// End of module.

