//
// Tests on circular trigonometric functions of expression hypermatrices.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Tests for sines.
assertExpr(sin(ad_ivar(%pi / 6)), 0.5, 1e-15);
assertExpr(sin(ad_ivar(4 * %pi / 3)), -sqrt(3) / 2, 1e-15);

p = sin(4 * ad_ivar(%pi) / 3);
assertExpr(p(1), -sqrt(3) / 2, 1e-15);
assertExpr(p($), -sqrt(3) / 2, 1e-15);
assertExpr(p(:), -sqrt(3) / 2, 1e-15);
assertExpr(p(1,1), -sqrt(3) / 2, 1e-15);
assertExpr(p($,:), -sqrt(3) / 2, 1e-15);

y = ad_ivar([%pi / 3; %pi; 3 * %pi / 4; 3 * %pi / 2; 11 * %pi / 6]);
M = sin([y'; ...
	2 * %pi, y(3), 7 * %pi / 4, 5 * %pi / 6, 5 * %pi / 3;...
	y(4)   , y(1), 3 * %pi    , -%pi / 6   , 5 * %pi/2]);
assertExpr(M, [sqrt(3)/2, 0, sqrt(2)/2, -1, -1/2; 0, sqrt(2)/2, -sqrt(2)/2, 1/2, -sqrt(3)/2; -1, sqrt(3)/2, 0, -1/2, 1], 1e-15);
//assertExpr(M(3:7), [-1; 0; sqrt(2)/2; sqrt(3)/2; sqrt(2)/2], 1e-15);  // Single-index extraction not supported anymore on matrices.
//assertExpr(M([10,$,8]), [-1; 1; -sqrt(2)/2], 1e-15);  // Single-index extraction not supported anymore on matrices.
//assertExpr(M(:), [sqrt(3)/2; 0; -1; 0; sqrt(2)/2; sqrt(3)/2; sqrt(2)/2; -sqrt(2)/2; 0; -1; 1/2; -1/2; -1/2; -sqrt(3)/2; 1], 1e-15);  // Single-index extraction not supported anymore on matrices.
assertExpr(M(3,4), -1/2);
assertExpr(M(2,:), [0, sqrt(2)/2, -sqrt(2)/2, 1/2, -sqrt(3)/2], 1e-15);
assertExpr(M(:,4), [-1; 1/2; -1/2], 1e-15);
assertExpr(M($,$), 1, 1e-15);


// Tests for cosines.
assertExpr(cos(ad_ivar(%pi / 6)), sqrt(3)/2, 1e-15);
assertExpr(cos(ad_ivar(4 * %pi / 3)), -1 / 2, 1e-15);

p = cos(4 * ad_ivar(%pi) / 3);
assertExpr(p(1), -1 / 2, 1e-15);
assertExpr(p($), -1 / 2, 1e-15);
assertExpr(p(:), -1 / 2, 1e-15);
assertExpr(p(1,1), -1 / 2, 1e-15);
assertExpr(p($,:), -1 / 2, 1e-15);

y = ad_ivar([%pi / 3; %pi; 3 * %pi / 4; 3 * %pi / 2; 11 * %pi / 6]);
M = cos([y'; ...
	2 * %pi, y(3), 7 * %pi / 4, 5 * %pi / 6, 5 * %pi / 3;...
	y(4)   , y(1), 3 * %pi    , -%pi / 6   , 5 * %pi/2]);
assertExpr(M, [1/2, -1, -sqrt(2)/2, 0, sqrt(3)/2; 1, -sqrt(2)/2, sqrt(2)/2, -sqrt(3)/2, 1/2; 0, 1/2, -1, sqrt(3)/2, 0], 1e-15);
//assertExpr(M(3:7), [0; -1; -sqrt(2)/2; 1/2; -sqrt(2)/2], 1e-15);  // Single-index extraction not supported anymore on matrices.
//assertExpr(M([10,$,8]), [0; 0; sqrt(2)/2], 1e-15);  // Single-index extraction not supported anymore on matrices.
//assertExpr(M(:), [1/2; 1; 0; -1; -sqrt(2)/2; 1/2; -sqrt(2)/2; sqrt(2)/2; -1; 0; -sqrt(3)/2; sqrt(3)/2; sqrt(3)/2; 1/2; 0], 1e-15);  // Single-index extraction not supported anymore on matrices.
assertExpr(M(3,4), sqrt(3)/2);
assertExpr(M(2,:), [1, -sqrt(2)/2, sqrt(2)/2, -sqrt(3)/2, 1/2], 1e-15);
assertExpr(M(:,4), [0; -sqrt(3)/2; sqrt(3)/2], 1e-15);
assertExpr(M($,$), 0, 1e-15);


// Tests for arc-tangent.
x = ad_ivar(3);
assertExpr(atan(sqrt(x)), %pi / 3, 1e-16);
assertExpr(atan(-sqrt(x) / 3), -%pi / 6, 1e-16);

p = atan(-sqrt(x) / 3);
assertExpr(p(1), -%pi / 6, 1e-16);
assertExpr(p($), -%pi / 6, 1e-16);
assertExpr(p(:), -%pi / 6, 1e-16);
assertExpr(p(1,1), -%pi / 6, 1e-16);
assertExpr(p(:,$), -%pi / 6, 1e-16);

z = ad_ivar([3; sqrt(3); 1; -1; 0]);
x = z(1);
y = z(2:$);
M = atan([y'; 2, -sqrt(x) / 3, 0, 1]);
assertExpr(M, [%pi/3, %pi/4, -%pi/4, 0 ; atan(2), -%pi/6, 0, %pi/4], 1e-16);
//assertExpr(M(3), %pi/4, 1e-16);  // Single-index extraction not supported anymore on matrices.
//assertExpr(M(4:7), [-%pi/6; -%pi/4; 0; 0], 1e-16);  // Single-index extraction not supported anymore on matrices.
//assertExpr(M([$,2]), [%pi/4; atan(2)], 1e-16);  // Single-index extraction not supported anymore on matrices.
//assertExpr(M(:), [%pi/3; atan(2); %pi/4; -%pi/6; -%pi/4; 0; 0; %pi/4], 1e-16);  // Single-index extraction not supported anymore on matrices.
assertExpr(M(2,3), 0, 1e-16);
assertExpr(M(1,:), [%pi/3, %pi/4, -%pi/4, 0], 1e-16);
assertExpr(M(:,4), [0; %pi/4], 1e-16);
assertExpr(M($,2), -%pi/6, 1e-16);


clear x;
clear p;
clear y;
clear M;
clear z;

printf("Expr - Circular trigonometrics : all unit tests passed.\n");


// End of script.

