//
// Tests : concatenation of expression hypermatrices.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Horizontal concatenation.
s = ad_const(5);
assert(and(Expr_GetEvaluation([s,s,s,s,s]) == [5,5,5,5,5]));
assert(and(Expr_GetEvaluation([s,8,s]) == [5,8,5]));
assert(and(Expr_GetEvaluation([s,[]]) == Expr_GetEvaluation(s)));
assert(and(Expr_GetEvaluation([[],s]) == Expr_GetEvaluation(s)));
h = ad_const(1:3);
assert(and(Expr_GetEvaluation([h,h]) == [1,2,3,1,2,3]));
assert(and(Expr_GetEvaluation([1:2,h,5:7]) == [1,2,1,2,3,5,6,7]));
assert(and(Expr_GetEvaluation([h,[]]) == Expr_GetEvaluation(h)));
assert(and(Expr_GetEvaluation([[],h]) == Expr_GetEvaluation(h)));
v = ad_const((1:3)');
assert(and(Expr_GetEvaluation([v,v]) == [1,1;2,2;3,3]));
assert(and(Expr_GetEvaluation([[10;11;12],v]) == [10,1;11,2;12,3]));
assert(and(Expr_GetEvaluation([v,[]]) == Expr_GetEvaluation(v)));
assert(and(Expr_GetEvaluation([[],v]) == Expr_GetEvaluation(v)));
m = ad_const([1,2;3,4;5,6]);
assert(and(Expr_GetEvaluation([m,m,m]) == [1,2,1,2,1,2;3,4,3,4,3,4;5,6,5,6,5,6]));
assert(and(Expr_GetEvaluation([m,[22;44;66]]) == [1,2,22;3,4,44;5,6,66]));
assert(and(Expr_GetEvaluation([s,h,s]) == [5,1,2,3,5]));
assert(and(Expr_GetEvaluation([v,m]) == [1,1,2;2,3,4;3,5,6]));
assert(and(Expr_GetEvaluation([m,[]]) == Expr_GetEvaluation(m)));
assert(and(Expr_GetEvaluation([[],m]) == Expr_GetEvaluation(m)));
assertError("[s,v]");
assertError("[v,s]");
assertError("[h,v]");
assertError("[v,h]");
assertError("[h,m]");
assertError("[m,s]");

// Vertical concatenation.
s = ad_const(5);
assert(and(Expr_GetEvaluation([s;s;s;s;s]) == [5;5;5;5;5]));
assert(and(Expr_GetEvaluation([s;8;s]) == [5;8;5]));
assert(and(Expr_GetEvaluation([s;[]]) == Expr_GetEvaluation(s)));
assert(and(Expr_GetEvaluation([[];s]) == Expr_GetEvaluation(s)));
h = ad_const(1:3);
assert(and(Expr_GetEvaluation([h;h]) == [1,2,3;1,2,3]));
assert(and(Expr_GetEvaluation([2:4;h;10:12]) == [2,3,4;1,2,3;10,11,12]));
assert(and(Expr_GetEvaluation([h;[]]) == Expr_GetEvaluation(h)));
assert(and(Expr_GetEvaluation([[];h]) == Expr_GetEvaluation(h)));
v = ad_const((1:3)');
assert(and(Expr_GetEvaluation([v;v]) == [1;2;3;1;2;3]));
assert(and(Expr_GetEvaluation([[10;11;12];v]) == [10;11;12;1;2;3]));
assert(and(Expr_GetEvaluation([v;[]]) == Expr_GetEvaluation(v)));
assert(and(Expr_GetEvaluation([[];v]) == Expr_GetEvaluation(v)));
m = ad_const([1,2;3,4;5,6]);
assert(and(Expr_GetEvaluation([m;m;m]) == [1,2;3,4;5,6;1,2;3,4;5,6;1,2;3,4;5,6]));
assert(and(Expr_GetEvaluation([m;[22,44;66,88]]) == [1,2;3,4;5,6;22,44;66,88]));
assert(and(Expr_GetEvaluation([h(1, 1:2);m]) == [1,2;1,2;3,4;5,6]));
assert(and(Expr_GetEvaluation([m;[]]) == Expr_GetEvaluation(m)));
assert(and(Expr_GetEvaluation([[];m]) == Expr_GetEvaluation(m)));
vt = v';
assert(and(Expr_GetEvaluation([vt(1, 2:$) ; m ; vt(1, 1:2)]) == [2,3;1,2;3,4;5,6;1,2]));
assertError("[s;h]");
assertError("[h;s]");
assertError("[h;v]");
assertError("[v;h]");
assertError("[h;m]");
assertError("[m;s]");
assertError("[v;m]");
assertError("[m;v]");

// Mixed concatenation.
s = ad_const(20);
h = ad_const(10:14);
v = ad_const((3:6)');
m = ad_const([1,2,3;4,5,6;7,8,9]);
assert(and(Expr_GetEvaluation([[[m,[[s,s];[100,200;300,400]]];h],v]) == ...
	[...
		 1,  2,  3,  20,  20, 3; ...
		 4,  5,  6, 100, 200, 4; ...
		 7,  8,  9, 300, 400, 5; ...
		10, 11, 12,  13,  14, 6  ...
	]));


clear s;
clear h;
clear v;
clear m;
clear vt;


printf("Expr - Concatenation : all unit tests passed.\n");

// End of module.

