//
// Tests on division (left and right, regular and element-wise) of expression hypermatrices.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Regular division : scalars.
x = ad_ivar(4);
ii = inv(x);
assert(and(size(ii) == size(x)));
assert(and(abs(Expr_GetEvaluation(ii) - 0.25) < 1e-8));
q = x / 10;
assert(and(size(q) == size(x)));
assert(and(abs(Expr_GetEvaluation(q) - 0.4) < 1e-8));
q = x \ 10;
assert(and(size(q) == size(x)));
assert(and(abs(Expr_GetEvaluation(q) - 2.5) < 1e-8));

extr = q(1);
assert(and(size(extr) == [1,1]));
assert(and(abs(Expr_GetEvaluation(extr) - 2.5) < 1e-8));
extr = q(:);
assert(and(size(extr) == [1,1]));
assert(and(abs(Expr_GetEvaluation(extr) - 2.5) < 1e-8));
extr = q(1,$);
assert(and(size(extr) == [1,1]));
assert(and(abs(Expr_GetEvaluation(extr) - 2.5) < 1e-8));
extr = q($,:);
assert(and(size(extr) == [1,1]));
assert(and(abs(Expr_GetEvaluation(extr) - 2.5) < 1e-8));

// Regular division : matrices.
z = ad_ivar([4;2;4;0;-3;4]);
x = z(1);
y = z(2:$);
A = [1,2,3,4,5 ; y'; x,1,x,0,0 ; 5,4,3,2,1 ; 10,2,-2,2,-2];
cmp = ...
[...
	  0.0441176,   0.0352941,   0.1294118, - 0.1441176,   0.1088235 ; ...
	- 0.1372549,   0.0235294, - 0.2470588,   0.4039216, - 0.0941176 ; ...
	- 0.0098039, - 0.0411765,   0.1823529,   0.0431373, - 0.0852941 ; ...
	  0.0882353, - 0.1294118, - 0.1411765,   0.1117647,   0.0176471 ; ...
	  0.1813725,   0.1117647,   0.0764706, - 0.2480392,   0.0529412 ; ...
];
ii = inv(A);
assert(and(size(ii) == size(A)));
assert(and(abs(Expr_GetEvaluation(ii) - cmp) < 1e-7));
q = y' / A;
assert(and(size(q) == [1,5]));
assert(and(abs(Expr_GetEvaluation(q) - [0,1,0,0,0]) < 1e-7));
q = A \ y;
assert(and(size(q) == [5,1]));
assert(and(abs(Expr_GetEvaluation(q) - [1.0970588 ; -1.7686275 ; -0.6549020 ; -0.6058824 ; 1.7656863]) < 1e-7));
q = 5 / A;
assert(and(size(q) == [5,5]));
assert(and(abs(Expr_GetEvaluation(q) - 5 * cmp) < 1e-6));
q = 5 \ A;
assert(and(size(q) == [5,5]));
assert(and(abs(Expr_GetEvaluation(q) - Expr_GetEvaluation(A) ./ 5) < 1e-6));
assert(and(size(A / 5) == size(5 \ A)));
assert(and(abs(Expr_GetEvaluation(A / 5) - Expr_GetEvaluation(5 \ A)) < 1e-7));
assert(and(size(A \ 5) == size(5 / A)));
assert(and(abs(Expr_GetEvaluation(A \ 5) - Expr_GetEvaluation(5 / A)) < 1e-7));

//extr = ii(12);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [1,1]));
//assert(and(abs(Expr_GetEvaluation(extr) - cmp(12)) < 1e-7));
//extr = ii([$,3,17,9]);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [4,1]));
//assert(and(abs(Expr_GetEvaluation(extr) - cmp([$,3,17,9])) < 1e-7));
//extr = ii(:);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [25,1]));
//assert(and(abs(Expr_GetEvaluation(extr) - cmp(:)) < 1e-7));
extr = ii(4,5);
assert(and(size(extr) == [1,1]));
assert(and(abs(Expr_GetEvaluation(extr) - cmp(4,5)) < 1e-7));
extr = ii([$,2], [1,3,4]);
assert(and(size(extr) == [2,3]));
assert(and(abs(Expr_GetEvaluation(extr) - cmp([5,2], [1,3,4])) < 1e-7));
extr = ii(:, [2,1]);
assert(and(size(extr) == [5,2]));
assert(and(abs(Expr_GetEvaluation(extr) - cmp(:, [2,1])) < 1e-7));
extr = ii(3, :);
assert(and(size(extr) == [1,5]));
assert(and(abs(Expr_GetEvaluation(extr) - cmp(3, :)) < 1e-7));
extr = ii(:,:);
assert(and(size(extr) == size(ii)));
assert(and(abs(Expr_GetEvaluation(extr) - cmp) < 1e-7));

q = A \ y;
//extr = q(2);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [1,1]));
//assert(and(abs(Expr_GetEvaluation(extr) + 1.7686275) < 1e-7));
//extr = q([3,$,4]);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [3,1]));
//assert(and(abs(Expr_GetEvaluation(extr) - [-0.6549020 ; 1.7656863 ; -0.6058824]) < 1e-7));
//extr = q(:);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == size(q)));
//assert(and(abs(Expr_GetEvaluation(extr) - Expr_GetEvaluation(q)) < 1e-7));
extr = q(2,1);
assert(and(size(extr) == [1,1]));
assert(and(abs(Expr_GetEvaluation(extr) + 1.7686275) < 1e-7));
extr = q([3,$,4], $);
assert(and(size(extr) == [3,1]));
assert(and(abs(Expr_GetEvaluation(extr) - [-0.6549020 ; 1.7656863 ; -0.6058824]) < 1e-7));
extr = q(:,:);
assert(and(size(extr) == size(q)));
assert(and(abs(Expr_GetEvaluation(extr) - Expr_GetEvaluation(q)) < 1e-7));


// Element-wise right division : scalars.
iii = eleminv(x);
assertExpr(iii, 0.25);
q = 2 ./ x;
assertExpr(q, 0.5);
q = x ./ 2;
assertExpr(q, 2);

extr = iii(1);
assertExpr(extr, 0.25);
extr = iii($);
assertExpr(extr, 0.25);
extr = iii(:);
assertExpr(extr, 0.25);
extr = iii(1,$);
assertExpr(extr, 0.25);
extr = iii(:,:);
assertExpr(extr, 0.25);
extr = q(1);
assertExpr(extr, 2);
extr = q($);
assertExpr(extr, 2);
extr = q(:);
assertExpr(extr, 2);
extr = q(1,$);
assertExpr(extr, 2);
extr = q(:,:);
assertExpr(extr, 2);


// Element-wise right division : column vectors.
iii = eleminv(y+1);
assertExpr(iii, [0.33333333 ; 0.2 ; 1 ; -0.5 ; 0.2]);
q = y ./ [5 ; 4 ; 9 ; -1 ; -2];
assertExpr(q, [0.4 ; 1 ; 0 ; 3 ; -2]);
q = [5 ; 4 ; 9 ; -1 ; -2] ./ (y + 2);
assertExpr(q, [1.25 ; 0.66666667 ; 4.5 ; 1 ; -0.33333333]);
assertExpr(5 ./ (y + 2), 5 ./ (Expr_GetEvaluation(y) + 2));
assertExpr(y ./ 5, Expr_GetEvaluation(y) ./ 5);

extr = q(3);
assertExpr(extr, 4.5);
extr = q([2,$,1]);
assertExpr(extr, [0.66666667 ; -0.33333333 ; 1.25]);
extr = q(:);
assertExpr(extr, Expr_GetEvaluation(q));
extr = q(4,1);
assertExpr(extr, 1);
extr = q([$,4], :);
assertExpr(extr, [-0.33333333 ; 1]);
extr = q(:,:);
assertExpr(extr, Expr_GetEvaluation(q));

// Element-wise right division : row vectors.
yt = y';
iii = eleminv(yt+1);
assertExpr(iii, [0.33333333 , 0.2 , 1 , -0.5 , 0.2]);
q = yt ./ [5 , 4 , 9 , -1 , -2];
assertExpr(q, [0.4 , 1 , 0 , 3 , -2]);
q = [5 , 4 , 9 , -1 , -2] ./ (yt + 2);
assertExpr(q, [1.25 , 0.66666667 , 4.5 , 1 , -0.33333333]);
assertExpr(5 ./ (yt + 2), 5 ./ (Expr_GetEvaluation(yt) + 2));
assertExpr(yt ./ 5, Expr_GetEvaluation(yt) ./ 5);

extr = q(3);
assertExpr(extr, 4.5);
extr = q([2,$,1]);
assertExpr(extr, [0.66666667 , -0.33333333 , 1.25]);
extr = q(:);
assertExpr(extr, Expr_GetEvaluation(q(:)));
extr = q(1,4);
assertExpr(extr, 1);
extr = q(:, [$,4]);
assertExpr(extr, [-0.33333333 , 1]);
extr = q(:,:);
assertExpr(extr, Expr_GetEvaluation(q));


// Element-wise right division : matrices.
M = [y , [5 ; x ; (x+x) ; 0 ; 0] , [-2 ; -9 ; x ; 3 ; 10]] + 1;
iii = eleminv(M);
cmp = ...
[ ...
	 0.3333333  0.1666667   -1.        ; ...
	 0.2        0.2         -0.125     ; ...
	 1.         0.1111111    0.2       ; ...
	-0.5        1.           0.25      ; ...
	 0.2        1.           0.0909091 ...
];
assertExpr(iii, cmp, 1e-6);
q = matrix(1:15, [5,3]) ./ M;
assertExpr(q, [0.33333333,1,-11 ; 0.4,1.4,-1.5 ; 3,0.88888889,2.6 ; -2,9,3.5 ; 1,10,1.36363636]);
q = M ./ matrix(1:15, [5,3]);
cmp2 = ...
[...
	 3         , 1         , -0.09090909 ; ...
	 2.5       , 0.7142857 , -0.66666667 ; ...
	 0.33333333, 1.125     ,  0.3846154  ; ...
	-0.5       , 0.11111111,  0.2857143  ; ...
	 1         , 0.1       ,  0.73333333   ...
];
assertExpr(q, cmp2);
assertExpr(5 ./ M, 5 ./ Expr_GetEvaluation(M));
assertExpr(M ./ 5, Expr_GetEvaluation(M) ./ 5);

assertExpr(iii(2, 2), 0.2);
assertExpr(iii([3,$,2], [$,2]), cmp([3,$,2], [$,2]));
assertExpr(iii(:, [1,2]), cmp(:, [1,2]));
assertExpr(iii([2,5], :), cmp([2,5], :));
assertExpr(iii(:,:), cmp);

assertExpr(q(2, 2), cmp2(2,2));
assertExpr(q([3,$,2], [$,2]), cmp2([3,$,2], [$,2]));
assertExpr(q(:, [1,2]), cmp2(:, [1,2]));
assertExpr(q([2,5], :), cmp2([2,5], :));
assertExpr(q(:,:), cmp2);


// Element-wise left division : scalars.
q = x .\ 2;
assertExpr(q, 0.5);
q = 2 .\ x;
assertExpr(q, 2);

extr = q(1);
assertExpr(extr, 2);
extr = q($);
assertExpr(extr, 2);
extr = q(:);
assertExpr(extr, 2);
extr = q(1,$);
assertExpr(extr, 2);
extr = q(:,:);
assertExpr(extr, 2);


// Element-wise left division : column vectors.
q = [5 ; 4 ; 9 ; -1 ; -2] .\ y;
assertExpr(q, [0.4 ; 1 ; 0 ; 3 ; -2]);
q = (y + 2) .\ [5 ; 4 ; 9 ; -1 ; -2];
assertExpr(q, [1.25 ; 0.66666667 ; 4.5 ; 1 ; -0.33333333]);
assertExpr((y + 2) .\ 5, 5 ./ (Expr_GetEvaluation(y) + 2));
assertExpr(5 .\ y, Expr_GetEvaluation(y) ./ 5);

extr = q(3);
assertExpr(extr, 4.5);
extr = q([2,$,1]);
assertExpr(extr, [0.66666667 ; -0.33333333 ; 1.25]);
extr = q(:);
assertExpr(extr, Expr_GetEvaluation(q));
extr = q(4,1);
assertExpr(extr, 1);
extr = q([$,4], :);
assertExpr(extr, [-0.33333333 ; 1]);
extr = q(:,:);
assertExpr(extr, Expr_GetEvaluation(q));

// Element-wise left division : row vectors.
q = [5 , 4 , 9 , -1 , -2] .\ yt;
assertExpr(q, [0.4 , 1 , 0 , 3 , -2]);
q = (yt + 2) .\ [5 , 4 , 9 , -1 , -2];
assertExpr(q, [1.25 , 0.66666667 , 4.5 , 1 , -0.33333333]);
assertExpr((yt + 2) .\ 5, 5 ./ (Expr_GetEvaluation(yt) + 2));
assertExpr(5 .\ yt, Expr_GetEvaluation(yt) ./ 5);

extr = q(3);
assertExpr(extr, 4.5);
extr = q([2,$,1]);
assertExpr(extr, [0.66666667 , -0.33333333 , 1.25]);
extr = q(:);
assertExpr(extr, Expr_GetEvaluation(q'));
extr = q(1,4);
assertExpr(extr, 1);
extr = q(:, [$,4]);
assertExpr(extr, [-0.33333333 , 1]);
extr = q(:,:);
assertExpr(extr, Expr_GetEvaluation(q));


// Element-wise left division : matrices.
q = M .\ matrix(1:15, [5,3]);
assertExpr(q, [0.33333333,1,-11 ; 0.4,1.4,-1.5 ; 3,0.88888889,2.6 ; -2,9,3.5 ; 1,10,1.36363636]);
q = matrix(1:15, [5,3]) .\ M;
assertExpr(q, cmp2);
assertExpr(M .\ 5, 5 ./ Expr_GetEvaluation(M));
assertExpr(5 .\ M, Expr_GetEvaluation(M) ./ 5);

assertExpr(q(2, 2), cmp2(2,2));
assertExpr(q([3,$,2], [$,2]), cmp2([3,$,2], [$,2]));
assertExpr(q(:, [1,2]), cmp2(:, [1,2]));
assertExpr(q([2,5], :), cmp2([2,5], :));
assertExpr(q(:,:), cmp2);


clear x;
clear ii;
clear q;
clear extr;
clear y;
clear A;
clear cmp;
clear iii;
clear yt;

printf("Expr - Division : all unit tests passed.\n");


// End of module.

