//
// Tests : element-wise product of expression matrices.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


x = ad_ivar(4);
pr = x .* x;
assert(and(size(pr) == [1,1]));
assert(Expr_GetEvaluation(pr) == 16);
pr = x .* 1;
assert(Expr_GetNodeCount(pr) == Expr_GetNodeCount(x));
pr = x .* 0;
assert(Expr_GetNodeCount(pr) == 1);
assert(Expr_GetEvaluation(pr) == 0);

assert(Expr_GetEvaluation(pr) == Expr_GetEvaluation(pr(1)));
assert(Expr_GetEvaluation(pr) == Expr_GetEvaluation(pr'));

y = ad_ivar([1;3;-9;0;2]);
pr = y .* y;
assert(and(size(pr) == [5,1]));
assert(and(Expr_GetEvaluation(pr) == [1;9;81;0;4]));
pr = [3;-2;1;3;0] .* pr;
assert(and(size(pr) == [5,1]));
assert(and(Expr_GetEvaluation(pr) == [3;-18;81;0;0]));
assert(and(Expr_GetNodeCount(pr) > 1));
assert(and(Expr_GetNodeCount(pr .* zeros(5,1)) == 1));
assert(and(Expr_GetNodeCount(ones(5,1) .* pr) == Expr_GetNodeCount(pr)));
pr2 = ad_const([4;2;1]) .* ad_const([0.25;0.5;1]);
assert(and(Expr_GetNodeCount(pr2) == 1));
assert(and(Expr_GetEvaluation(pr2) == [1;1;1]));

ipr = pr(:);
assert(and(size(ipr) == size(pr)) & and(Expr_GetEvaluation(ipr) == Expr_GetEvaluation(pr)));
ipr = pr(3:$);
assert(and(size(ipr) == [3,1]));
assert(and(Expr_GetEvaluation(ipr) == [81;0;0]));
ipr = pr([2,4]);
assert(and(size(ipr) == [2,1]));
assert(and(Expr_GetEvaluation(ipr) == [-18;0]));
//assert(and(Expr_GetNodeCount(ipr) == Expr_GetNodeCount(pr)));

prt = pr';
assert(and(size(prt) == [1,5]));
assert(and(Expr_GetEvaluation(prt) == [3,-18,81,0,0]));

pr = [1,2,3,4,5] .* y' .* [5,4,3,2,1];
assert(and(size(pr) == [1,5]));
assert(and(Expr_GetEvaluation(pr) == [5,24,-81,0,10]));
ipr = pr([2,3,5]);
assert(and(size(ipr) == [1,3]));
assert(and(Expr_GetEvaluation(ipr) == [24,-81,10]));
prt = ipr';
assert(and(size(prt) == [3,1]));
assert(and(Expr_GetEvaluation(prt) == [24;-81;10]));
ipr = prt([1,$]);
assert(and(size(ipr) == [2,1]));
assert(and(Expr_GetEvaluation(ipr) == [24;10]));

M = ad_const([1,2,0,4 ; -2,0,3,5]);
pr = M .* M;
assert(and(size(pr) == size(M)));
assert(and(Expr_GetEvaluation(pr) == [1,4,0,16 ; 4,0,9,25]));
pr = M' .* [3,0 ; -1,-1 ; 9,2 ; 0,1];
assert(and(size(pr) == [4,2]));
assert(and(Expr_GetEvaluation(pr) == [3,0 ; -2,0 ; 0,6 ; 0,5]));

ipr = pr(:,:);
assert(and(size(ipr) == size(pr)));
assert(and(Expr_GetEvaluation(ipr) == Expr_GetEvaluation(pr)));
//ipr = pr(:);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(ipr) == [8,1]));
//assert(and(Expr_GetEvaluation(ipr) == [3;-2;0;0;0;0;6;5]));
//ipr = pr(3:2:$);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(ipr) == [3,1]));
//assert(and(Expr_GetEvaluation(ipr) == [0;0;6]));
ipr = pr([2,3],:);
assert(and(size(ipr) == [2,2]));
assert(and(Expr_GetEvaluation(ipr) == [-2,0 ; 0,6]));
ipr = pr(:,$);
assert(and(size(ipr) == [4,1]));
assert(and(Expr_GetEvaluation(ipr) == [0;0;6;5]));

// NOTE - Hypermatrices not supported anymore in SCIAD expressions.
//H = ad_const(hypermat([2,2,3], 1:12));
//pr = hypermat([2,2,3], [1,0,2,0,3,0,4,0,5,0,6,0]) .* H .* hypermat([2,2,3], ones(12,1));
//assert(and(size(pr) == [2,2,3]));
//ev = Expr_GetEvaluation(pr);
//assert(and(ev(:) == [1;0;6;0;15;0;28;0;45;0;66;0]));
//
//ipr = pr(:);
//assert(and(size(ipr) == [12,1]));
//assert(and(Expr_GetEvaluation(ipr) == [1;0;6;0;15;0;28;0;45;0;66;0]));
//ipr = pr(:,1,$);
//assert(and(size(ipr) == [2,1]));
//assert(and(Expr_GetEvaluation(ipr) == [45;0]));
//ipr = pr(:,2,:);
//assert(and(size(ipr) == [2,1,3]));
//ev = Expr_GetEvaluation(ipr);
//assert(and(ev(:) == [6;0;28;0;66;0]));

pr = y .* y;
assert(and(size(pr) == [5,1]));
assert(and(Expr_GetEvaluation(pr) == [1;9;81;0;4]));
pr = [3;-2;1;3;0] .* pr;
assert(and(size(pr) == [5,1]));
assert(and(Expr_GetEvaluation(pr) == [3;-18;81;0;0]));
assert(and(Expr_GetNodeCount(pr) > 1));
assert(and(Expr_GetNodeCount(pr .* zeros(5,1)) == 1));
assert(and(Expr_GetNodeCount(ones(5,1) .* pr) == Expr_GetNodeCount(pr)));
pr2 = ad_const([4;2;1]) .* ad_const([0.25;0.5;1]);
assert(and(Expr_GetNodeCount(pr2) == 1));
assert(and(Expr_GetEvaluation(pr2) == [1;1;1]));

ipr = pr(:);
assert(and(size(ipr) == size(pr)) & and(Expr_GetEvaluation(ipr) == Expr_GetEvaluation(pr)));
ipr = pr(3:$);
assert(and(size(ipr) == [3,1]));
assert(and(Expr_GetEvaluation(ipr) == [81;0;0]));
ipr = pr([2,4]);
assert(and(size(ipr) == [2,1]));
assert(and(Expr_GetEvaluation(ipr) == [-18;0]));
//assert(and(Expr_GetNodeCount(ipr) == Expr_GetNodeCount(pr)));

prt = pr';
assert(and(size(prt) == [1,5]));
assert(and(Expr_GetEvaluation(prt) == [3,-18,81,0,0]));

pr = [1,2,3,4,5] .* y' .* [5,4,3,2,1];
assert(and(size(pr) == [1,5]));
assert(and(Expr_GetEvaluation(pr) == [5,24,-81,0,10]));
ipr = pr([2,3,5]);
assert(and(size(ipr) == [1,3]));
assert(and(Expr_GetEvaluation(ipr) == [24,-81,10]));
prt = ipr';
assert(and(size(prt) == [3,1]));
assert(and(Expr_GetEvaluation(prt) == [24;-81;10]));
ipr = prt([1,$]);
assert(and(size(ipr) == [2,1]));
assert(and(Expr_GetEvaluation(ipr) == [24;10]));

M = ad_const([1,2,0,4 ; -2,0,3,5]);
pr = M .* M;
assert(and(size(pr) == size(M)));
assert(and(Expr_GetEvaluation(pr) == [1,4,0,16 ; 4,0,9,25]));
pr = M' .* [3,0 ; -1,-1 ; 9,2 ; 0,1];
assert(and(size(pr) == [4,2]));
assert(and(Expr_GetEvaluation(pr) == [3,0 ; -2,0 ; 0,6 ; 0,5]));

ipr = pr(:,:);
assert(and(size(ipr) == size(pr)));
assert(and(Expr_GetEvaluation(ipr) == Expr_GetEvaluation(pr)));
//ipr = pr(:);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(ipr) == [8,1]));
//assert(and(Expr_GetEvaluation(ipr) == [3;-2;0;0;0;0;6;5]));
//ipr = pr(3:2:$);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(ipr) == [3,1]));
//assert(and(Expr_GetEvaluation(ipr) == [0;0;6]));
ipr = pr([2,3],:);
assert(and(size(ipr) == [2,2]));
assert(and(Expr_GetEvaluation(ipr) == [-2,0 ; 0,6]));
ipr = pr(:,$);
assert(and(size(ipr) == [4,1]));
assert(and(Expr_GetEvaluation(ipr) == [0;0;6;5]));

// NOTE - Hypermatrices not supported anymore in SCIAD expressions.
//H = ad_const(hypermat([2,2,3], 1:12));
//pr = hypermat([2,2,3], [1,0,2,0,3,0,4,0,5,0,6,0]) .* H .* hypermat([2,2,3], ones(12,1));
//assert(and(size(pr) == [2,2,3]));
//ev = Expr_GetEvaluation(pr);
//assert(and(ev(:) == [1;0;6;0;15;0;28;0;45;0;66;0]));
//
//ipr = pr(:);
//assert(and(size(ipr) == [12,1]));
//assert(and(Expr_GetEvaluation(ipr) == [1;0;6;0;15;0;28;0;45;0;66;0]));
//ipr = pr(:,1,$);
//assert(and(size(ipr) == [2,1]));
//assert(and(Expr_GetEvaluation(ipr) == [45;0]));
//ipr = pr(:,2,:);
//assert(and(size(ipr) == [2,1,3]));
//ev = Expr_GetEvaluation(ipr);
//assert(and(ev(:) == [6;0;28;0;66;0]));

// Tests for multiplication to a scalar.
z = ad_ivar([4;1;3;-9;0;2]);
x = z(1);
y = z(2:$);
pr = y .* x;
assert(and(size(pr) == size(y)));
assert(and(Expr_GetEvaluation(pr) == [4;12;-36;0;8]));
pr = 2 .* y;
assert(and(size(pr) == size(y)));
assert(and(Expr_GetEvaluation(pr) == [2;6;-18;0;4]));
pr = x .* y';
assert(and(size(pr) == size(y')));
assert(and(Expr_GetEvaluation(pr) == [4,12,-36,0,8]));
pr = y' .* 2;
assert(and(size(pr) == size(y')));
assert(and(Expr_GetEvaluation(pr) == [2,6,-18,0,4]));
pr = x .* M;
assert(and(size(pr) == size(M)));
assert(and(Expr_GetEvaluation(pr) == [4,8,0,16 ; -8,0,12,20]));
pr = M .* (1/2);
assert(and(size(pr) == size(M)));
assert(and(Expr_GetEvaluation(pr) == [0.5,1,0,2 ; -1,0,1.5,2.5]));

// TODO - Implement element-wise product of n-dimensional hypermatrices (n > 2) to scalars.
//pr = H .* x;
//assert(and(size(pr) == size(H)));
//ev = Expr_GetEvaluation(pr);
//assert(and(ev(:) == (4:4:48)'));
//pr = 2 .* H;
//assert(and(size(pr) == size(H)));
//ev = Expr_GetEvaluation(pr);
//assert(and(ev(:) == (2:2:24)'));


clear x;
clear y;
clear pr;
clear pr2;
clear ipr;
clear prt;
clear M;
clear H;
clear ev;

printf("Expr - Element-wise product : all unit tests passed.\n");


// End of module.

