//
// Tests on exponentials (regular and element-wise) of expression hypermatrices.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Regular exponential.
ivar = ad_ivar([3; 0.5; -0.8; -4; 0; 1]);
x1 = ivar(1);
x2 = ivar(2);
x3 = ivar(3);
x4 = ivar(4);
zz = ivar(5);
oo = ivar(6);

assertExpr(2 ^ x1, 8);
assertExpr(2 ^ x2, sqrt(2), 1e-8);
assertExpr(2 ^ x3, 0.574349177, 1e-8);
assertExpr(2 ^ x4, 0.0625);
assertExpr(2 ^ zz, 1);
assertExpr(2 ^ oo, 2);

M = [2,-1 ; -1,2];
assertExpr(M ^ x1, [14,-13 ; -13,14]);
assertExpr(M ^ x2, [1.3660254, -0.3660254 ; -0.3660254, 1.3660254], 1e-7);
assertExpr(M ^ x3, [0.7076218, 0.2923782 ; 0.2923782, 0.7076218], 1e-7);
assertExpr(M ^ x4, [0.5061728, 0.4938272 ; 0.4938272, 0.5061728], 1e-7);
assertExpr(M ^ zz, eye(2,2));
assertExpr(M ^ oo, M);

p = 5 ^ (-x2);
s = p(1);
assertExpr(s, Expr_GetEvaluation(p));
s = p($);
assertExpr(s, Expr_GetEvaluation(p));
s = p(:);
assertExpr(s, Expr_GetEvaluation(p));
s = p(1,1);
assertExpr(s, Expr_GetEvaluation(p));
s = p(:,$);
assertExpr(s, Expr_GetEvaluation(p));

p = M ^ (x4 + x1);
//assertExpr(p(2), 1/3);  // Single-index extraction not supported anymore on matrices.
//assertExpr(p(2:$), [1/3 ; 1/3 ; 2/3]);  // Single-index extraction not supported anymore on matrices.
//assertExpr(p(:), [2/3 ; 1/3 ; 1/3 ; 2/3]);  // Single-index extraction not supported anymore on matrices.
assertExpr(p(2,1), 1/3);
assertExpr(p(2,:), [1/3, 2/3]);
assertExpr(p(:,1), [2/3 ; 1/3]);
assertExpr(p(:,:), Expr_GetEvaluation(p));


// Element-wise exponential - scalars.
assertExpr(2 .^ x1, 8);
assertExpr(2 .^ x2, sqrt(2), 1e-8);
assertExpr(2 .^ x3, 0.574349177, 1e-8);
assertExpr(2 .^ x4, 0.0625);
assertExpr(2 .^ zz, 1);
assertExpr(2 .^ oo, 2);

assertExpr(exp(x1), 20.08553692, 1e-8);
assertExpr(exp(x2 + 0.1), 1.822118800, 1e-9);
assertExpr(exp(x3), 0.449328964, 1e-9);
assertExpr(exp(2 * x4 / 3), 0.069483451, 1e-9);
assertExpr(exp(zz), 1);
assertExpr(exp(oo), %e);

p = 5 .^ (-x2);
s = p(1);
assertExpr(s, Expr_GetEvaluation(p));
s = p($);
assertExpr(s, Expr_GetEvaluation(p));
s = p(:);
assertExpr(s, Expr_GetEvaluation(p));
s = p(1,1);
assertExpr(s, Expr_GetEvaluation(p));
s = p(:,$);
assertExpr(s, Expr_GetEvaluation(p));

// Element-wise exponential - column vectors.
y = ad_ivar([0 ; -2 ; 0.25 ; 3]);
assertExpr(exp(y), [1 ; 0.1353353 ; 1.2840254 ; 20.085537], 1e-7);
assertExpr([1+0.2 ; 3 ; 16 ; -0.2] .^ y, [1 ; 0.1111111 ; 2 ; -0.008], 1e-7);
assertExpr([1+0.2 ; 3 ; 16 ; -0.2] ^ y, [1 ; 0.1111111 ; 2 ; -0.008], 1e-7);
assertExpr(2 .^ y, [1 ; 0.25 ; 1.1892071 ; 8], 1e-7);
assertExpr(2 ^ y, [1 ; 0.25 ; 1.1892071 ; 8], 1e-7);

p = [0;0.5;4;2] .^ y;
assertExpr(p(2), 4);
assertExpr(p([$,3,1]), [8; 1.4142136; 1]);
assertExpr(p(:), Expr_GetEvaluation(p));
assertExpr(p(2,1), 4);
assertExpr(p(2:$, $), [4; 1.4142136; 8]);
assertExpr(p(:,:), Expr_GetEvaluation(p));

// Element-wise exponential - row vectors.
yt = y';
assertExpr(exp(yt), [1 , 0.1353353 , 1.2840254 , 20.085537], 1e-7);
assertExpr([1+0.2 , 3 , 16 , -0.2] .^ yt, [1 , 0.1111111 , 2 , -0.008], 1e-7);
assertExpr([1+0.2 , 3 , 16 , -0.2] ^ yt, [1 , 0.1111111 , 2 , -0.008], 1e-7);
assertExpr(2 .^ yt, [1 , 0.25 , 1.1892071 , 8], 1e-7);
assertExpr(2 ^ yt, [1 , 0.25 , 1.1892071 , 8], 1e-7);

p = [0,0.5,4,2] .^ yt;
assertExpr(p(2), 4);
assertExpr(p([$,3,1]), [8, 1.4142136, 1]);
assertExpr(p(:), Expr_GetEvaluation(p)');
assertExpr(p(1,2), 4);
assertExpr(p($, 2:$), [4, 1.4142136, 8]);
assertExpr(p(:,:), Expr_GetEvaluation(p));

// Element-wise exponential - 2-dim matrices.
M = [5, 1, 1, 2 ; 1, 6, -1, 2 ; 1, -1, 10, 0 ; 2, 2, 0, 7];
p = M .^ [[5, x1, -0.5 ; x2, x3, 0 ; 0.5, 3, 3 ; x4, zz, oo], y];
assertExpr(p, ...
	[...
		3125     ,  1        ,    1,   1   ; ...
		   1     ,  0.2384948,    1,   0.25; ...
		   1     , -1        , 1000,   0   ; ...
		   0.0625,  1        ,    0, 343     ...
	], 1e-7);

//assertExpr(p(7), -1);  // Single-index extraction not supported anymore on matrices.
//assertExpr(p([11,$,2]), [1000 ; 343 ; 1]);  // Single-index extraction not supported anymore on matrices.
//assertExpr(p(:), [3125; 1; 1; 0.0625; 1; 0.2384948; -1; 1; 1; 1; 1000; 0; 1; 0.25; 0; 343]);  // Single-index extraction not supported anymore on matrices.
assertExpr(p(2,4), 0.25);
assertExpr(p([1,4], [2,3,1]), [1,1,3125 ; 1,0,0.0625]);
assertExpr(p($,:), [0.0625, 1, 0, 343]);
assertExpr(p(:,$), [1; 0.25; 0; 343]);
assertExpr(p(:,:), Expr_GetEvaluation(p));


clear ivars;
clear x1;
clear x2;
clear x3;
clear x4;
clear zz;
clear oo;
clear y;
clear yt;
clear p;
clear M;


printf("Expr - Exponentials : all unit tests passed.\n");


// End of module.

