//
// Tests : extraction out of matrices that result of a product.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


x = ad_ivar(5);
p = x * 3;
extr = p(1);
assert(and(size(extr) == [1,1]));
assert(Expr_GetEvaluation(extr) == 15);
extr = p($,$);
assert(and(size(extr) == [1,1]));
assert(Expr_GetEvaluation(extr) == 15);

y = ad_ivar([2;5;-1;0;8]);
p = [2,-2,0,3,4] * y;
extr = p(:);
assert(and(size(extr) == [1,1]));
assert(Expr_GetEvaluation(extr) == 26);
extr = p($,:);
assert(and(size(extr) == [1,1]));
assert(Expr_GetEvaluation(extr) == 26);

p = [2,-2,0,3,4 ; 1,1,2,0,0 ; 1,1,1,1,1 ; -2,0,0,-2,-3] * y;
extr = p([2,3]);
assert(and(size(extr) == [2,1]));
assert(and(Expr_GetEvaluation(extr) == [5;14]));
extr = p([1,3,$]);
assert(and(size(extr) == [3,1]));
assert(and(Expr_GetEvaluation(extr) == [26;14;-28]));
extr = p(:);
assert(and(size(extr) == size(p)));
assert(and(Expr_GetEvaluation(extr) == Expr_GetEvaluation(p)));
extr = p(1,1);
assert(and(size(extr) == [1,1]));
assert(and(Expr_GetEvaluation(extr) == 26));
extr = p([2:$], :);
assert(and(size(extr) == [3,1]));
assert(and(Expr_GetEvaluation(extr) == [5;14;-28]));
extr = p(:,:);
assert(and(size(extr) == size(p)));
assert(and(Expr_GetEvaluation(extr) == Expr_GetEvaluation(p)));

p = y' * [2,1,1,-2 ; -2,1,1,0 ; 0,2,1,0 ; 3,0,1,-2 ; 4,0,1,-3];
extr = p([2,3]);
assert(and(size(extr) == [1,2]));
assert(and(Expr_GetEvaluation(extr) == [5,14]));
extr = p([1,3,$]);
assert(and(size(extr) == [1,3]));
assert(and(Expr_GetEvaluation(extr) == [26,14,-28]));
extr = p(:);
assert(and(size(extr) == size(p')));
assert(and(Expr_GetEvaluation(extr) == Expr_GetEvaluation(p')));
extr = p(1,1);
assert(and(size(extr) == [1,1]));
assert(and(Expr_GetEvaluation(extr) == 26));
extr = p(:, 2:$);
assert(and(size(extr) == [1,3]));
assert(and(Expr_GetEvaluation(extr) == [5,14,-28]));
extr = p(:,:);
assert(and(size(extr) == size(p)));
assert(and(Expr_GetEvaluation(extr) == Expr_GetEvaluation(p)));

p = y * y';
//extr = p(8);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [1,1]));
//assert(and(Expr_GetEvaluation(extr) == -5));
//extr = p([19:22]);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [4,1]));
//assert(and(Expr_GetEvaluation(extr) == [0;0;16;40]));
//extr = p([2,12,$,17,24]);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [5,1]));
//assert(and(Expr_GetEvaluation(extr) == [10;-5;64;0;0]));
//extr = p(:);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [25,1]));
//assert(and(Expr_GetEvaluation(extr) == [4;10;-2;0;16 ; 10;25;-5;0;40 ; -2;-5;1;0;-8 ; 0;0;0;0;0 ; 16;40;-8;0;64]));
extr = p(3,5);
assert(and(size(extr) == [1,1]));
assert(and(Expr_GetEvaluation(extr) == -8));
extr = p(3:$, [1,2]);
assert(and(size(extr) == [3,2]));
assert(and(Expr_GetEvaluation(extr) == [-2,-5 ; 0,0 ; 16,40]));
extr = p([2,5], [3,1]);
assert(and(size(extr) == [2,2]));
assert(and(Expr_GetEvaluation(extr) == [-5,10 ; -8,16]));
extr = p([2,3], :);
assert(and(size(extr) == [2,5]));
assert(and(Expr_GetEvaluation(extr) == [10,25,-5,0,40 ; -2,-5,1,0,-8]));
extr = p(:, [$,2]);
assert(and(size(extr) == [5,2]));
assert(and(Expr_GetEvaluation(extr) == [16,10 ; 40,25 ; -8,-5 ; 0,0 ; 64,40]));
extr = p(:,:);
assert(and(size(extr) == size(p)));
assert(and(Expr_GetEvaluation(extr) == Expr_GetEvaluation(p)));

// Special test : old derivation bug!
xt = ad_ivar([5;-2;0;3])';
M = [1;0;0] * xt([2,3,1]) + [0;1;0] * xt([4,4,1]) + [0;0;1] * xt([4,3,2]);
assertExpr(M(:,3), [5;5;-2]);
assertExpr(M(1,:), [-2,0,5]);
assertExpr(M(2,:), [3,3,5]);
assertExpr(M(3,:), [3,0,-2]);


clear x;
clear extr;
clear xt;
clear y;
clear yt;
clear M;

printf("Expr - Product extraction : all unit tests passed.\n");


// End of module.

