//
// Tests on insertion in expression matrices.
//
// Author : Benoit Hamelin
// 2002 Universit de Sherbrooke
//


// Unidimensional indexing insertion.

z = ad_ivar([4;6]);
x = z(1);
y = z(2);
x(1) = y;
assert(and(size(x) == [1,1]));
assert(and(Expr_GetEvaluation(x) == 6));

z = ad_ivar([4;2;3;6;8;9]);
x = z(1);
y = z(2:$);
v = [y ; 100 ; x ; 200 ; x];
assert(and(size(v) == [9,1]));
v(2) = 1000;
assert(and(size(v) == [9,1]));
assert(and(Expr_GetEvaluation(v) == [2 ; 1000 ; 6 ; 8 ; 9 ; 100 ; 4 ; 200 ; 4]));
v([6,7,8]) = [x ; 2000 ; x];
assert(and(size(v) == [9,1]));
assert(and(Expr_GetEvaluation(v) == [2 ; 1000 ; 6 ; 8 ; 9 ; 4 ; 2000 ; 4 ; 4]));
v([12,13,14,15,16]) = y(:);
assert(and(size(v) == [16,1]));
assert(and(Expr_GetEvaluation(v) == [2 ; 1000 ; 6 ; 8 ; 9 ; 4 ; 2000 ; 4 ; 4 ; 0 ; 0 ; 2 ; 3 ; 6 ; 8 ; 9]));
v([10,11]) = [34,67];
assert(and(size(v) == [16,1]));
assert(and(Expr_GetEvaluation(v) == [2 ; 1000 ; 6 ; 8 ; 9 ; 4 ; 2000 ; 4 ; 4 ; 34 ; 67 ; 2 ; 3 ; 6 ; 8 ; 9]));
v([15,16,17,18]) = [y(2);x;y(3);43];
assert(and(size(v) == [18,1]));
assert(and(Expr_GetEvaluation(v) == [2 ; 1000 ; 6 ; 8 ; 9 ; 4 ; 2000 ; 4 ; 4 ; 34 ; 67 ; 2 ; 3 ; 6 ; 3 ; 4 ; 6 ; 43]));
v = [23;43;12];
v([3,4,5,6,7]) = y;
assert(and(size(v) == [7,1]));
assert(and(Expr_GetEvaluation(v) == [23 ; 43 ; 2 ; 3 ; 6 ; 8 ; 9]));
v([1,5,$+3,2:3]) = (1000:1000:5000)';
assert(and(size(v) == [10,1]));
assert(and(Expr_GetEvaluation(v) == [1000 ; 4000 ; 5000 ; 3 ; 2000 ; 8 ; 9 ; 0 ; 0 ; 3000]));

y = y';
v = [y , 100 , x , 200 , x];
assert(and(size(v) == [1,9]));
v(2) = 1000;
assert(and(size(v) == [1,9]));
assert(and(Expr_GetEvaluation(v) == [2 , 1000 , 6 , 8 , 9 , 100 , 4 , 200 , 4]));
v([6,7,8]) = [x , 2000 , x];
assert(and(size(v) == [1,9]));
assert(and(Expr_GetEvaluation(v) == [2 , 1000 , 6 , 8 , 9 , 4 , 2000 , 4 , 4]));
v([12,13,14,15,16]) = y;
assert(and(size(v) == [1,16]));
assert(and(Expr_GetEvaluation(v) == [2 , 1000 , 6 , 8 , 9 , 4 , 2000 , 4 , 4 , 0 , 0 , 2 , 3 , 6 , 8 , 9]));
v([10,11]) = [34;67];
assert(and(size(v) == [1,16]));
assert(and(Expr_GetEvaluation(v) == [2 , 1000 , 6 , 8 , 9 , 4 , 2000 , 4 , 4 , 34 , 67 , 2 , 3 , 6 , 8 , 9]));
v([15,16,17,18]) = [y(2),x,y(3),43];
assert(and(size(v) == [1,18]));
assert(and(Expr_GetEvaluation(v) == [2 , 1000 , 6 , 8 , 9 , 4 , 2000 , 4 , 4 , 34 , 67 , 2 , 3 , 6 , 3 , 4 , 6 , 43]));
v = [23,43,12];
v([3,4,5,6,7]) = y;
assert(and(size(v) == [1,7]));
assert(and(Expr_GetEvaluation(v) == [23 , 43 , 2 , 3 , 6 , 8 , 9]));
v([1,5,$+3,2:3]) = (1000:1000:5000);
assert(and(size(v) == [1,10]));
assert(and(Expr_GetEvaluation(v) == [1000 , 4000 , 5000 , 3 , 2000 , 8 , 9 , 0 , 0 , 3000]));

M = [y ; 100, x, 34, 45, x ; y(2:4), 90, x];
assert(and(size(M) == [3,5]));
assert(and(Expr_GetEvaluation(M) == [2, 3, 6, 8, 9 ; 100, 4, 34, 45, 4 ; 3, 6, 8, 90, 4]));
//M(11) = x;  // Single-index insertion not supported anymore on matrices.
//assert(and(size(M) == [3,5]));
//assert(and(Expr_GetEvaluation(M) == [2, 3, 6, 8, 9 ; 100, 4, 34, 4, 4 ; 3, 6, 8, 90, 4]));
//M(2:3) = [1000;2000];  // Single-index insertion not supported anymore on matrices.
//assert(and(size(M) == [3,5]));
//assert(and(Expr_GetEvaluation(M) == [2, 3, 6, 8, 9 ; 1000, 4, 34, 4, 4 ; 2000, 6, 8, 90, 4]));
//M(5:8) = (300:100:600)';  // Single-index insertion not supported anymore on matrices.
//assert(and(size(M) == [3,5]));
//assert(and(Expr_GetEvaluation(M) == [2, 3, 500, 8, 9 ; 1000, 300, 600, 4, 4 ; 2000, 400, 8, 90, 4]));
//y = y';
//M(2:11) = [2 ; 3 ; 6 ; 8 ; 9 ; 2 ; 3 ; 6 ; 8 ; 9];  // Single-index insertion not supported anymore on matrices.
//assert(and(size(M) == [3,5]));
//assert(and(Expr_GetEvaluation(M) == [2, 6, 2, 8, 9 ; 2, 8, 3, 9, 4 ; 3, 9, 6, 90, 4]));
//M([$,3,7]) = [1000;2000;3000];  // Single-index insertion not supported anymore on matrices.
//assert(and(size(M) == [3,5]));
//assert(and(Expr_GetEvaluation(M) == [2, 6, 3000, 8, 9 ; 2, 8, 3, 9, 4 ; 2000, 9, 6, 90, 1000]));

// Two-dimensional indexing insertion.
z = ad_ivar([4;6]);
x = z(1);
y = z(2);
x(1,1) = y;
assert(and(size(x) == [1,1]));
assert(Expr_GetEvaluation(x) == 6);
x($,$) = 45;
assert(and(size(x) == [1,1]));
assert(Expr_GetEvaluation(x) == 45);

z = ad_ivar([4;2;3;6;8;9]);
x = z(1);
y = z(2:$);
v = [y ; 100 ; x ; 200 ; x];
v(2, 1) = 1000;
assert(and(size(v) == [9,1]));
assert(and(Expr_GetEvaluation(v) == [2;1000;6;8;9;100;4;200;4]));
v(11, 1) = 2000;
assert(and(size(v) == [11,1]));
assert(and(Expr_GetEvaluation(v) == [2;1000;6;8;9;100;4;200;4;0;2000]));
v([3,4,5], 1) = [50;60;70];
assert(and(size(v) == [11,1]));
assert(and(Expr_GetEvaluation(v) == [2;1000;50;60;70;100;4;200;4;0;2000]));
v([$,$+1], 1) = [23;34];
assert(and(size(v) == [12,1]));
assert(and(Expr_GetEvaluation(v) == [2;1000;50;60;70;100;4;200;4;0;23;34]));
v([3,6,8,$+1], 1) = [-1;-2;-3;-4];
assert(and(size(v) == [13,1]));
assert(and(Expr_GetEvaluation(v) == [2;1000;-1;60;70;-2;4;-3;4;0;23;34;-4]));
v(:, 1) = [1;x;2;x;3;4;x;5;(x+x);6;x;8;10];
assert(and(size(v) == [13,1]));
assert(and(Expr_GetEvaluation(v) == [1;4;2;4;3;4;4;5;8;6;4;8;10]));
v(5, :) = x;
assert(and(size(v) == [13,1]));
assert(and(Expr_GetEvaluation(v) == [1;4;2;4;4;4;4;5;8;6;4;8;10]));
v([1,3], 3) = [-1;-2];
assert(and(size(v) == [13,3]));
assert(and(Expr_GetEvaluation(v) == [[1;4;2;4;4;4;4;5;8;6;4;8;10], zeros(13,1), [-1;0;-2;zeros(10,1)]]));

v = [y' , 100 , x , 200 , x];
v(1, 2) = 1000;
assert(and(size(v) == [1,9]));
assert(and(Expr_GetEvaluation(v) == [2,1000,6,8,9,100,4,200,4]));
v(1, 11) = 2000;
assert(and(size(v) == [1,11]));
assert(and(Expr_GetEvaluation(v) == [2,1000,6,8,9,100,4,200,4,0,2000]));
v(1, [3,4,5]) = [50,60,70];
assert(and(size(v) == [1,11]));
assert(and(Expr_GetEvaluation(v) == [2,1000,50,60,70,100,4,200,4,0,2000]));
v(1, [$,$+1]) = [23,34];
assert(and(size(v) == [1,12]));
assert(and(Expr_GetEvaluation(v) == [2,1000,50,60,70,100,4,200,4,0,23,34]));
v(1, [3,6,8,$+1]) = [-1,-2,-3,-4];
assert(and(size(v) == [1,13]));
assert(and(Expr_GetEvaluation(v) == [2,1000,-1,60,70,-2,4,-3,4,0,23,34,-4]));
v(1, :) = [1,x,2,x,3,4,x,5,(x+x),6,x,8,10];
assert(and(size(v) == [1,13]));
assert(and(Expr_GetEvaluation(v) == [1,4,2,4,3,4,4,5,8,6,4,8,10]));
v(:, 5) = x;
assert(and(size(v) == [1,13]));
assert(and(Expr_GetEvaluation(v) == [1,4,2,4,4,4,4,5,8,6,4,8,10]));
v(3, [1,3]) = [-1,-2];
assert(and(size(v) == [3,13]));
assert(and(Expr_GetEvaluation(v) == [[1,4,2,4,4,4,4,5,8,6,4,8,10]; zeros(1,13); [-1,0,-2,zeros(1,10)]]));

yt = y';
v = [yt; 1000:1000:5000; x, 2, 3, 10, x; yt];
v(2,3) = 0;
assert(and(size(v) == [4,5]));
assert(and(Expr_GetEvaluation(v) == [2,3,6,8,9 ; 1000,2000,0,4000,5000 ; 4,2,3,10,4 ; 2,3,6,8,9]));
v(:,4) = y(2:$);
assert(and(size(v) == [4,5]));
assert(and(Expr_GetEvaluation(v) == [2,3,6,3,9 ; 1000,2000,0,6,5000 ; 4,2,3,8,4 ; 2,3,6,9,9]));
v(3,:) = yt;
assert(and(size(v) == [4,5]));
assert(and(Expr_GetEvaluation(v) == [2,3,6,3,9 ; 1000,2000,0,6,5000 ; 2,3,6,8,9 ; 2,3,6,9,9]));
v(2:$, [3,4]) = matrix(-1:-1:-6, [3,2]);
assert(and(size(v) == [4,5]));
assert(and(Expr_GetEvaluation(v) == [2,3,6,3,9 ; 1000,2000,-1,-4,5000 ; 2,3,-2,-5,9 ; 2,3,-3,-6,9]));
v([1,3], [1,3,5]) = matrix(100:100:600, [2,3]);
assert(and(size(v) == [4,5]));
assert(and(Expr_GetEvaluation(v) == [100,3,300,3,500 ; 1000,2000,-1,-4,5000 ; 200,3,400,-5,600 ; 2,3,-3,-6,9]));
v(6,6) = x;
assert(and(size(v) == [6,6]));
assert(and(Expr_GetEvaluation(v) == ...
[...
	 100,   3, 300, 3, 500,0 ; ...
	1000,2000,-  1,-4,5000,0 ; ...
	 200,   3, 400,-5, 600,0 ; ...
	   2,   3,-  3,-6,   9,0 ; ...
	   0,   0,   0, 0,   0,0 ; ...
	   0,   0,   0, 0,   0,4   ...
]));
v([5,7], [4,8]) = [10,20 ; 30,40];
assert(and(size(v) == [7,8]));
assert(and(Expr_GetEvaluation(v) == ...
[...
	 100,   3, 300,  3, 500,0,0, 0 ; ...
	1000,2000,-  1,- 4,5000,0,0, 0 ; ...
	 200,   3, 400,- 5, 600,0,0, 0 ; ...
	   2,   3,-  3,- 6,   9,0,0, 0 ; ...
	   0,   0,   0, 10,   0,0,0,20 ; ...
	   0,   0,   0,  0,   0,4,0, 0 ; ...
	   0,   0,   0, 30,   0,0,0,40   ...
]));

// Insertion into a constant or empty matrix.
MM = matrix(1:20, [4,5]);
MM(2,:) = yt;
assertExpr(MM, [1,5,9,13,17 ; 2,3,6,8,9 ; 3,7,11,15,19 ; 4,8,12,16,20]);
EMP = [];
EMP(1) = x;
assertExpr(EMP, Expr_GetEvaluation(x));
EMP = [];
EMP(4) = x;
assertExpr(EMP, [0;0;0;4]);
EMP = [];
EMP(1,1) = x;
assertExpr(EMP, 4);
EMP = [];
EMP(1,6) = x;
assertExpr(EMP, [0,0,0,0,0,4]);
EMP = [];
EMP(3,1) = x;
assertExpr(EMP, [0;0;4]);
EMP = [];
EMP(3,2) = x;
assertExpr(EMP, [0,0 ; 0,0 ; 0,4]);


clear x;
clear y;
clear v;
clear M;
clear z;

printf("Expr - Insertion : all unit tests passed.\n");


// End of script.

