//
// Tests : regular product of expression matrices.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


x = ad_ivar(5);
pr = x * x;
assert(and(size(pr) == [1,1]));
assert(and(Expr_GetEvaluation(pr) == 25));
assert(and(Expr_GetNodeCount(pr) == 3));
pr = pr * 5;
assert(and(size(pr) == [1,1]));
assert(and(Expr_GetEvaluation(pr) == 125));

y = ad_ivar([2;3;-1;0;5]);
pr = [1,2,3,4,5] * y;
assert(and(size(pr) == [1,1]));
assert(and(Expr_GetEvaluation(pr) == 30));
pr = y * [1,2,3,4,5];
assert(and(size(pr) == [5,5]));
assert(and(Expr_GetEvaluation(pr) == ...
	[...
		 2,   4,   6,   8,  10; ...
		 3,   6,   9,  12,  15; ...
		-1, - 2, - 3, - 4, - 5; ...
		 0,   0,   0,   0,   0; ...
		 5,  10,  15,  20,  25 ...
	]));
pr = y' * y;
assert(and(size(pr) == [1,1]));
assert(and(Expr_GetEvaluation(pr) == 39));
pr = 5 * y;
assert(and(size(pr) == size(y)));
assert(and(Expr_GetEvaluation(pr) == [10;15;-5;0;25]));
pr2 = y * 5;
assert(and(size(pr) == size(y)));
assert(and(Expr_GetEvaluation(pr) == Expr_GetEvaluation(pr2)));

M1 = ad_const([1,2,3,4,5 ; 5,6,7,8,9]);
pr = M1 * y;
assert(and(size(pr) == [2,1]));
assert(and(Expr_GetEvaluation(pr) == [30;66]));
pr = M1 * M1';
assert(and(size(pr) == [2,2]));
assert(and(Expr_GetEvaluation(pr) == [55,115 ; 115,255]));
pr = M1' * M1;
assert(and(size(pr) == [5,5]));
assert(and(Expr_GetEvaluation(pr) == ...
	[...
		26, 32, 38, 44,  50; ...
		32, 40, 48, 56,  64; ...
		38, 48, 58, 68,  78; ...
		44, 56, 68, 80,  92; ...
		50, 64, 78, 92, 106; ...
	]));
pr = 2 * M1;
assert(and(size(pr) == size(M1)));
assert(and(Expr_GetEvaluation(pr) == [2,4,6,8,10 ; 10,12,14,16,18]));
pr2 = M1 * 2;
assert(and(size(pr) == size(M1)));
assert(and(Expr_GetEvaluation(pr2) == Expr_GetEvaluation(pr)));


clear x;
clear pr;
clear y;
clear pr2;
clear M1;

printf("Expr - Inner product : all unit tests passed.\n");


// End of script.

