//
// Tests on pseudo-polynomes (regular and element-wise elevation of functions to constant real exponents) of expression
// hypermatrices.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Regular scalar pseudo-polynomes.
ivar = ad_ivar([3; 0.5; -0.8; -2.5; 0; 1]);
x1 = ivar(1);
x2 = ivar(2);
x3 = ivar(3);
x4 = ivar(4);
zz = ivar(5);
oo = ivar(6);

assertExpr(x1 ^ 3, 27);
assertExpr(x1 ^ 0.5, sqrt(3));
assertExpr(x1 ^ (-1), 1/3);
assertExpr(x1 ^ (-1.5), 0.1924501, 1e-7);
assertExpr(x1 ^ 1, Expr_GetEvaluation(x1));
assertExpr(x1 ^ 0, 1);

assertExpr(x2 ^ 3, 1/8);
assertExpr(x2 ^ 0.5, 1 / sqrt(2));
assertExpr(x2 ^ (-1), 2);
assertExpr(x2 ^ (-1.5), 2.8284271, 1e-7);
assertExpr(x2 ^ 1, Expr_GetEvaluation(x2));
assertExpr(x2 ^ 0, 1);

assertExpr(x3 ^ 3, -0.512);
assertExpr(x3 ^ (-1), -1.25);
assertExpr(x3 ^ 1, Expr_GetEvaluation(x3));
assertExpr(x3 ^ 0, 1);

assertExpr(x4 ^ 3, -15.625);
assertExpr(x4 ^ (-1), -0.4);
assertExpr(x4 ^ 1, Expr_GetEvaluation(x4));
assertExpr(x4 ^ 0, 1);

p = x1 ^ 0.5;
assertExpr(p(1), Expr_GetEvaluation(p));
assertExpr(p($), Expr_GetEvaluation(p));
assertExpr(p(:), Expr_GetEvaluation(p));
assertExpr(p(1,1), Expr_GetEvaluation(p));
assertExpr(p($,:), Expr_GetEvaluation(p));


// Regular matricial pseudo-polynomes.
M = [x1, -2 ; -2, x1];
assertExpr(M ^ 3, [63, -62 ; -62, 63]);
assertExpr(M ^ 0.5, [1.618034, -0.6180340; -0.6180340, 1.618034], 1e-7);
assertExpr(M ^ (-1), [0.6, 0.4 ; 0.4, 0.6]);
assertExpr(M ^ (-1.5), [0.5447214, 0.4552786; 0.4552786, 0.5447214]);
assertExpr(M ^ 1, Expr_GetEvaluation(M));
assertExpr(M ^ 0, eye(2,2));

p = M ^ 3;
//assertExpr(p(2), -62);  // Single-index extraction not supported anymore on matrices.
//assertExpr(p(2:$), [-62 ; -62; 63]);  // Single-index extraction not supported anymore on matrices.
//assertExpr(p(:), [63; -62; -62; 63]);  // Single-index extraction not supported anymore on matrices.
assertExpr(p(2,1), -62);
assertExpr(p(2,:), [-62, 63]);
assertExpr(p(:,1), [63; -62]);
assertExpr(p(:,:), Expr_GetEvaluation(p));


// Element-wise scalar pseudo-polynomes.
assertExpr(x1 .^ 3, 27);
assertExpr(x1 .^ 0.5, sqrt(3));
assertExpr(x1 .^ (-1), 1/3);
assertExpr(x1 .^ (-1.5), 0.1924501, 1e-7);
assertExpr(x1 .^ 1, Expr_GetEvaluation(x1));
assertExpr(x1 .^ 0, 1);

assertExpr(x2 .^ 3, 1/8);
assertExpr(x2 .^ 0.5, 1 / sqrt(2));
assertExpr(x2 .^ (-1), 2);
assertExpr(x2 .^ (-1.5), 2.8284271, 1e-7);
assertExpr(x2 .^ 1, Expr_GetEvaluation(x2));
assertExpr(x2 .^ 0, 1);

assertExpr(x3 .^ 3, -0.512);
assertExpr(x3 .^ (-1), -1.25);
assertExpr(x3 .^ 1, Expr_GetEvaluation(x3));
assertExpr(x3 .^ 0, 1);

assertExpr(x4 .^ 3, -15.625);
assertExpr(x4 .^ (-1), -0.4);
assertExpr(x4 .^ 1, Expr_GetEvaluation(x4));
assertExpr(x4 .^ 0, 1);

p = x1 .^ 0.5;
assertExpr(p(1), Expr_GetEvaluation(p));
assertExpr(p($), Expr_GetEvaluation(p));
assertExpr(p(:), Expr_GetEvaluation(p));
assertExpr(p(1,1), Expr_GetEvaluation(p));
assertExpr(p($,:), Expr_GetEvaluation(p));


// Element-wise column vector pseudo-polynomes.
y = ad_ivar([-1; 2; 0; 3.5]);
assertExpr(y .^ [2; -1; 2; -0.5], [1; 0.5; 0; 0.5345225], 1e-7);
assertExpr(y ^ [2; -1; 2; -0.5], [1; 0.5; 0; 0.5345225], 1e-7);
assertExpr(y .^ 3, [-1; 8; 0; 42.875]);
assertExpr(y ^ 3, [-1; 8; 0; 42.875]);
assertExpr(y .^ 1, Expr_GetEvaluation(y));
assertExpr(y .^ 0, ones(4,1));

p = y .^ [2; -1; 2; -0.5];
assertExpr(p(2), 0.5);
assertExpr(p([$,3,1]), [0.5345225; 0; 1], 1e-7);
assertExpr(p(:), Expr_GetEvaluation(p));
assertExpr(p(2,1), 0.5);
assertExpr(p(2:$, $), [0.5; 0; 0.5345225], 1e-7);
assertExpr(p(:,:), Expr_GetEvaluation(p));


// Element-wise row vector pseudo-polynomes.
yt = y';
assertExpr(yt .^ [2, -1, 2, -0.5], [1, 0.5, 0, 0.5345225], 1e-7);
assertExpr(yt ^ [2, -1, 2, -0.5], [1, 0.5, 0, 0.5345225], 1e-7);
assertExpr(yt .^ 3, [-1, 8, 0, 42.875]);
assertExpr(yt ^ 3, [-1, 8, 0, 42.875]);
assertExpr(yt .^ 1, Expr_GetEvaluation(yt));
assertExpr(yt .^ 0, ones(1,4));

p = yt .^ [2, -1, 2, -0.5];
assertExpr(p(2), 0.5);
assertExpr(p([$,3,1]), [0.5345225, 0, 1], 1e-7);
assertExpr(p(:), Expr_GetEvaluation(p)');
assertExpr(p(1,2), 0.5);
assertExpr(p($, 2:$), [0.5, 0, 0.5345225], 1e-7);
assertExpr(p(:,:), Expr_GetEvaluation(p));


// Element-wise matricial pseudo-polynomes.
M = [yt(4:-1:1); [y(3:-1:1), [3,0,1; 0,5,-2; 1,-2,6]]];
assertExpr(M .^ 2, ...
	[...
		12.25, 0,  4,  1; ...
	     0   , 9,  0,  1; ...
	     4   , 0, 25,  4; ...
	     1   , 1,  4, 36  ...
	]);
p = M .^ [-1, 0.5, 2, 2; 1.5, -1.5, 0, 4; -4, 3, 0, 1; -1, 2, 3, -4];
assertExpr(p, ...
	[...
	     0.2857143, 0        ,  4,  1;        ...
	     0        , 0.1924501,  1,  1;        ...
	     0.0625   , 0        ,  1, -2;        ...
	    -1        , 1        , -8,  0.0007716 ...
	], 1e-7);

//assertExpr(p(7), 0);  // Single-index extraction not supported anymore on matrices.
//assertExpr(p([11,$,2]), [1; 0.0007716; 0]);  // Single-index extraction not supported anymore on matrices.
//assertExpr(p(:), [0.2857143; 0; 0.0625; -1; 0; 0.1924501; 0; 1; 4; 1; 1; -8; 1; 1; -2; 0.0007716], 1e-7);  // Single-index extraction not supported anymore on matrices.
assertExpr(p(2,4), 1);
assertExpr(p([1,4], [2,3,1]), [0,4,0.2857143; 1,-8,-1]);
assertExpr(p($,:), [-1, 1, -8, 0.0007716], 1e-7);
assertExpr(p(:,$), [1; 1; -2; 0.0007716], 1e-7);
assertExpr(p(:,:), Expr_GetEvaluation(p));


clear ivars;
clear x1;
clear x2;
clear x3;
clear x4;
clear zz;
clear oo;
clear p;
clear y;
clear yt;
clear M;

printf("Expr - Polynoms : all unit tests passed.\n");


// End of module.

