//
// Tests on subtraction of expression hypermatrices.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


// Scalars.
x = ad_ivar(4);
opp = -x;
assert(and(size(opp) == size(x)));
assert(Expr_GetEvaluation(opp) == -4);
d = x - 5;
assert(and(size(d) == size(x)));
assert(Expr_GetEvaluation(d) == -1);
d = 5 - x;
assert(and(size(d) == size(x)));
assert(Expr_GetEvaluation(d) == 1);

extr = d(1);
assert(and(size(extr) == size(d)));
assert(Expr_GetEvaluation(extr) == Expr_GetEvaluation(d));
extr = d(:,$);
assert(and(size(extr) == size(d)));
assert(Expr_GetEvaluation(extr) == Expr_GetEvaluation(d));


// Column vectors.
z = ad_ivar([4;2;6;-2;0;5]);
x = z(1);
y = z(2:$);
opp = -y;
assert(and(size(opp) == size(y)));
assert(and(Expr_GetEvaluation(opp) == [-2;-6;2;0;-5]));
d = y - [1;-3;5;2;0];
assert(and(size(d) == size(y)));
assert(and(Expr_GetEvaluation(d) == [1;9;-7;-2;5]));
d = [x;x;x;x;x] - y;
assert(and(size(d) == size(y)));
assert(and(Expr_GetEvaluation(d) == [2;-2;6;4;-1]));
d2 = x - y;
assert(and(size(d2) == size(y)));
assert(and(Expr_GetEvaluation(d2) == Expr_GetEvaluation(d)));
d2 = y - x;
assert(and(size(d2) == size(y)));
assert(and(Expr_GetEvaluation(d2) == -Expr_GetEvaluation(d)));

d = y - [1;-3;5;2;0];
extr = d(3);
assert(and(size(extr) == [1,1]));
assert(and(Expr_GetEvaluation(extr) == -7));
extr = d(2:$);
assert(and(size(extr) == [4,1]));
assert(and(Expr_GetEvaluation(extr) == [9;-7;-2;5]));
extr = d([1,4]);
assert(and(size(extr) == [2,1]));
assert(and(Expr_GetEvaluation(extr) == [1;-2]));
extr = d(:);
assert(and(size(extr) == size(d)));
assert(and(Expr_GetEvaluation(extr) == Expr_GetEvaluation(d)));
extr = d(5, 1);
assert(and(size(extr) == [1,1]));
assert(and(Expr_GetEvaluation(extr) == 5));
extr = d([3,$], 1);
assert(and(size(extr) == [2,1]));
assert(and(Expr_GetEvaluation(extr) == [-7;5]));
extr = d(:,:);
assert(and(size(extr) == size(d)));
assert(and(Expr_GetEvaluation(extr) == Expr_GetEvaluation(d)));


// Row vectors.
z = ad_ivar([4;2;6;-2;0;5]);
x = z(1);
y = z(2:$)';
opp = -y;
assert(and(size(opp) == size(y)));
assert(and(Expr_GetEvaluation(opp) == [-2,-6,2,0,-5]));
d = y - [1,-3,5,2,0];
assert(and(size(d) == size(y)));
assert(and(Expr_GetEvaluation(d) == [1,9,-7,-2,5]));
d = [x,x,x,x,x] - y;
assert(and(size(d) == size(y)));
assert(and(Expr_GetEvaluation(d) == [2,-2,6,4,-1]));
d2 = x - y;
assert(and(size(d2) == size(y)));
assert(and(Expr_GetEvaluation(d2) == Expr_GetEvaluation(d)));
d2 = y - x;
assert(and(size(d2) == size(y)));
assert(and(Expr_GetEvaluation(d2) == -Expr_GetEvaluation(d)));

d = y - [1,-3,5,2,0];
extr = d(3);
assert(and(size(extr) == [1,1]));
assert(and(Expr_GetEvaluation(extr) == -7));
extr = d(2:$);
assert(and(size(extr) == [1,4]));
assert(and(Expr_GetEvaluation(extr) == [9,-7,-2,5]));
extr = d([1,4]);
assert(and(size(extr) == [1,2]));
assert(and(Expr_GetEvaluation(extr) == [1,-2]));
extr = d(:);
assert(and(size(extr) == size(d')));
assert(and(Expr_GetEvaluation(extr) == Expr_GetEvaluation(d')));
extr = d(1, 5);
assert(and(size(extr) == [1,1]));
assert(and(Expr_GetEvaluation(extr) == 5));
extr = d(1, [3,$]);
assert(and(size(extr) == [1,2]));
assert(and(Expr_GetEvaluation(extr) == [-7,5]));
extr = d(:,:);
assert(and(size(extr) == size(d)));
assert(and(Expr_GetEvaluation(extr) == Expr_GetEvaluation(d)));


// Matrices.
M = [y ; 5,x,2,(x+x),1 ; y(1:3),x,-9];
opp = -M;
assert(and(size(opp) == size(M)));
assert(and(Expr_GetEvaluation(opp) == [-2,-6,2,0,-5 ; -5,-4,-2,-8,-1 ; -2,-6,2,-4,9]));
d = M - matrix(1:15, [3,5]);
assert(and(size(d) == size(M)));
assert(and(Expr_GetEvaluation(d) == [1,2,-9,-10,-8 ; 3,-1,-6,-3,-13 ; -1,0,-11,-8,-24]));
d = matrix(1:15, [3,5]) - M;
assert(and(size(d) == size(M)));
assert(and(Expr_GetEvaluation(d) == [-1,-2,9,10,8 ; -3,1,6,3,13 ; 1,0,11,8,24]));
d = M - x;
assert(and(size(d) == size(M)));
assert(and(Expr_GetEvaluation(d) == [-2,2,-6,-4,1 ; 1,0,-2,4,-3 ; -2,2,-6,0,-13]));
d = 1 - M;
assert(and(size(d) == size(M)));
assert(and(Expr_GetEvaluation(d) == [-1,-5,3,1,-4 ; -4,-3,-1,-7,0 ; -1,-5,3,-3,10]));

//extr = d(10);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [1,1]));
//assert(and(Expr_GetEvaluation(extr) == 1));
//extr = d([2:8]);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [7,1]));
//assert(and(Expr_GetEvaluation(extr) == [-4;-1;-5;-3;-5;3;-1]));
//extr = d([$,3,9,4]);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [4,1]));
//assert(and(Expr_GetEvaluation(extr) == [10 ; -1 ; 3 ; -5]));
//extr = d(:);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(extr) == [15,1]));
//assert(and(Expr_GetEvaluation(extr) == [-1;-4;-1 ; -5;-3;-5 ; 3;-1;3 ; 1;-7;-3 ; -4;0;10]));
extr = d(3,4);
assert(and(size(extr) == [1,1]));
assert(and(Expr_GetEvaluation(extr) == -3));
extr = d($, [2,4]);
assert(and(size(extr) == [1,2]));
assert(and(Expr_GetEvaluation(extr) == [-5,-3]));
extr = d(:, [1,$]);
assert(and(size(extr) == [3,2]));
assert(and(Expr_GetEvaluation(extr) == [-1,-4 ; -4,0 ; -1,10]));
extr = d(2, :);
assert(and(size(extr) == [1,5]));
assert(and(Expr_GetEvaluation(extr) == [-4,-3,-1,-7,0]));
extr = d(:,:);
assert(and(size(extr) == size(d)));
assert(and(Expr_GetEvaluation(extr) == Expr_GetEvaluation(d)));


// Hypermatrices.
// NOTE : these aren't supported anymore.
//H = ad_const(hypermat([2,2,3], 1:12));
//opp = -H;
//assert(and(size(opp) == size(H)));
//ev = Expr_GetEvaluation(opp);
//assert(and(ev(:) == -(1:12)'));
//d = H - hypermat([2,2,3], 2:13);
//assert(and(size(d) == size(H)));
//ev = Expr_GetEvaluation(d);
//assert(and(ev(:) == -ones(12,1)));
//d = hypermat([2,2,3], 2:13) - H;
//assert(and(size(d) == size(H)));
//ev = Expr_GetEvaluation(d);
//assert(and(ev(:) == ones(12,1)));


clear z;
clear x;
clear opp;
clear d;
clear y;
clear d2;
clear extr;
clear ev;

printf("Expr - Subtraction : all unit tests passed.\n");


// End of module.
