//
// Tests on execution of UNC problem functions and gradients.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


printf("Expr - UNC problems...\n");


// Problem 1 - rose.
assertExpr(objfcn(2, 2, ad_ivar([-1.2;1]), 1), 24.2);
assertExpr(objfcn(2, 2, ad_ivar([3;-4]), 1), 16904);
assertExpr(objfcn(2, 2, ad_ivar([6;8]), 1), 78425);
assertExpr(objfcn(2, 2, ad_ivar([1;1]), 1), 0);

assertExpr(grdfcn(2, 2, ad_ivar([-1.2;1]), 1), [-215.6, -88]);
assertExpr(grdfcn(2, 2, ad_ivar([3;-4]), 1), [15604, -2600]);
assertExpr(grdfcn(2, 2, ad_ivar([6;8]), 1), [67210, -5600]);
assertExpr(grdfcn(2, 2, ad_ivar([1;1]), 1), [0,0]);

printf("     1/34. rose - OK\n");


// Problem 2 - froth

assertExpr(objfcn(2, 2, ad_ivar([0.5;-2]), 2), 400.5);
assertExpr(objfcn(2, 2, ad_ivar([-3;4]), 2), 128);
assertExpr(objfcn(2, 2, ad_ivar([2;7]), 2), 86418);
assertExpr(objfcn(2, 2, ad_ivar([5;4]), 2), 0);

assertExpr(grdfcn(2, 2, ad_ivar([0.5;-2]), 2), [30,-1272]);
assertExpr(grdfcn(2, 2, ad_ivar([-3;4]), 2), [-32,-512]);
assertExpr(grdfcn(2, 2, ad_ivar([2;7]), 2), [288,97932]);
assertExpr(grdfcn(2, 2, ad_ivar([5;4]), 2), [0,0]);

printf("     2/34. froth - OK\n");


// Problem 3 - badscp

assertExpr(objfcn(2, 2, ad_ivar([0;1]), 3), 1.1352617, 1e-7);
assertExpr(objfcn(2, 2, ad_ivar([-3;4]), 3), 1.440e10, 1e7);
assertExpr(objfcn(2, 2, ad_ivar([2;7]), 3), 1.960e10, 1e7);
assertExpr(objfcn(2, 2, ad_ivar([1.098e-5 ; 9.106]), 3), 2.599e-8, 1e-11);

assertExpr(grdfcn(2, 2, ad_ivar([0;1]), 3), [-20000.736,-0.271], 1e-3);
assertExpr(grdfcn(2, 2, ad_ivar([-3;4]), 3), [-9.60008e9,7.20006e9], 1e4);
assertExpr(grdfcn(2, 2, ad_ivar([2;7]), 3), [19.59986e9,5.59996e9], 1e4);
assertExpr(grdfcn(2, 2, ad_ivar([1.098e-5 ; 9.106]), 3), [-29.357744,-0.0000354], 1e-6);

printf("     3/34. badscp - OK\n");


// Problem 4 - badscb

assertExpr(objfcn(2, 3, ad_ivar([1;1]), 4), 1e12, 1e8);
assertExpr(objfcn(2, 3, ad_ivar([-3;4]), 4), 1e12, 1e8);
assertExpr(objfcn(2, 3, ad_ivar([2;-7]), 4), 1e12, 1e8);
assertExpr(objfcn(2, 3, ad_ivar([1e6;2e-6]), 4), 0);

assertExpr(grdfcn(2, 3, ad_ivar([1;1]), 4), [-2000000,-0.000004], 1e-6);
assertExpr(grdfcn(2, 3, ad_ivar([-3;4]), 4), [-2000118,91.999996], 1e-6);
assertExpr(grdfcn(2, 3, ad_ivar([2;-7]), 4), [-1999772,-78.000004], 1e-6);
assertExpr(grdfcn(2, 3, ad_ivar([1e6;2e-6]), 4), [0,0]);

printf("     4/34. badscb - OK\n");


// Problem 5 - beale

assertExpr(objfcn(2, 3, ad_ivar([1;1]), 5), 14.203125, 1e-6);
assertExpr(objfcn(2, 3, ad_ivar([-3;4]), 5), 36619.453, 1e-3);
assertExpr(objfcn(2, 3, ad_ivar([2;-7]), 5), 479602.2, 1e-1);
assertExpr(objfcn(2, 3, ad_ivar([3;0.5]), 5), 0);

assertExpr(grdfcn(2, 3, ad_ivar([1;1]), 5), [0,27.75]);
assertExpr(grdfcn(2, 3, ad_ivar([-3;4]), 5), [-24810.75,55773], 1e-2);
assertExpr(grdfcn(2, 3, ad_ivar([2;-7]), 5), [481202,-408560.5], 1e-1);
assertExpr(grdfcn(2, 3, ad_ivar([3;0.5]), 5), [0,0]);

printf("     5/34. beale - OK\n");


// Problem 6 - jensam

assertExpr(objfcn(2, 10, ad_ivar([0.3;0.4]), 6), 4171.3062, 1e-4);
assertExpr(objfcn(2, 10, ad_ivar([-3;4]), 6), 5.542e34, 1e31);
assertExpr(objfcn(2, 10, ad_ivar([2;-7]), 6), 2.398e17, 1e14);
assertExpr(objfcn(2, 10, ad_ivar([0.2578;0.2578]), 6), 124.36227, 1e-5);

assertExpr(grdfcn(2, 10, ad_ivar([0.3;0.4]), 6), [33796.559,87402.147], 1e-3);
r = Expr_GetEvaluation(grdfcn(2, 10, ad_ivar([-3;4]), 6));
assert(and(abs(r - [6.564e5,1.108e36]) < [1e2,1e33]));
r = Expr_GetEvaluation(grdfcn(2, 10, ad_ivar([2;-7]), 6));
assert(and(abs(r - [4.787e18,0.0063460]) < [1e15,1e-7]));
assertExpr(grdfcn(2, 10, ad_ivar([0.2578;0.2578]), 6), [-3.4224962,-3.4224962], 1e-7);

printf("     6/34. jensam - OK\n");


// Problem 7 - helix
assertExpr(objfcn(3, 3, ad_ivar([-1;0;0]), 7), 2500);
assertExpr(objfcn(3, 3, ad_ivar([-2;3;5]), 7), 948.55089, 1e-5);
assertExpr(objfcn(3, 3, ad_ivar([6;-3;-2]), 7), 3421.6443, 1e-4);
assertExpr(objfcn(3, 3, ad_ivar([1;0;0]), 7), 0);

assertExpr(grdfcn(3, 3, ad_ivar([-1;0;0]), 7), [0,-1591.5494,-1000], 1e-4);
assertExpr(grdfcn(3, 3, ad_ivar([-2;3;5]), 7), [-75.678911,575.84397,322.83296], 1e-5);
assertExpr(grdfcn(3, 3, ad_ivar([6;-3;-2]), 7), [1057.6358,-437.51489,-256.41638], 1e-4);
assertExpr(grdfcn(3, 3, ad_ivar([1;0;0]), 7), [0,0,0]);

printf("     7/34. helix - OK.\n");


// Problem 8 - bard

assertExpr(objfcn(3, 15, ad_ivar([1;1;1]), 8), 41.681696, 1e-6);
assertExpr(objfcn(3, 15, ad_ivar([0;-3;7]), 8), 6.4435155, 1e-7);
assertExpr(objfcn(3, 15, ad_ivar([-2;9;2]), 8), 110.11645, 1e-5);
assertExpr(objfcn(3, 15, ad_ivar([0.0824106;1.13304;2.3437]), 8), 0.0082149, 1e-7);

assertExpr(grdfcn(3, 15, ad_ivar([1;1;1]), 8), [43.765714,-51.871238,-50.559988], 1e-6);
assertExpr(grdfcn(3, 15, ad_ivar([0;-3;7]), 8), [-2.3696013,-57.844354,-25.476418], 1e-6);
assertExpr(grdfcn(3, 15, ad_ivar([-2;9;2]), 8), [-78.24818,2.3523525,2.2949503], 1e-5);
assertExpr(grdfcn(3, 15, ad_ivar([0.0824106;1.13304;2.3437]), 8), [-0.0000566,0.0000376,0.0000373], 1e-7);

printf("     8/34. bard - OK.\n");


// Problem 9 - gauss
assertExpr(objfcn(3, 15, ad_ivar([-0.25;1;-0.3]), 9), 1.4771750, 1e-7);
assertExpr(objfcn(3, 15, ad_ivar([-2;3;-7]), 9), 0.5642234, 1e-7);
assertExpr(objfcn(3, 15, ad_ivar([0.5;-1;0.75]), 9), 17804407, 1);
assertExpr(objfcn(3, 15, ad_ivar([0.4;1;0]), 9), 0.0000039, 1e-7);

assertExpr(grdfcn(3, 15, ad_ivar([-0.25;1;-0.3]), 9), [-4.5380333,-0.2914009,0.1037113], 1e-7);
assertExpr(grdfcn(3, 15, ad_ivar([-2;3;-7]), 9), [-0.0191762e-9,-0.2361727e-9,0.4037110e-9], 1e-16);
assertExpr(grdfcn(3, 15, ad_ivar([0.5;-1;0.75]), 9), [71217673,-3.202E+08,1.510E+08], 1e5);
assertExpr(grdfcn(3, 15, ad_ivar([0.4;1;0]), 9), [0.007414,-0.0007441,0], 1e-6);

printf("     9/34. gauss - OK.\n");


// Problem 10 - meyer
assertExpr(objfcn(3, 16, ad_ivar([0.02;4000;250]), 10), 1.694e9, 1e6);
assertExpr(objfcn(3, 16, ad_ivar([0;0;0]), 10), 3.891e9, 1e6);
assertExpr(objfcn(3, 16, ad_ivar([-2;3;-7]), 10), 3.892e9, 1e6);
assertExpr(objfcn(3, 16, ad_ivar([10;-3;-9]), 10), 3.887e9, 1e6);

assertExpr(grdfcn(3, 16, ad_ivar([0.02;4000;250]), 10), [-872.76663e8,-0.0561936e8,0.7247908e8], 1e3);
assertExpr(grdfcn(3, 16, ad_ivar([0;0;0]), 10), [-397826,0,0]);
assertExpr(grdfcn(3, 16, ad_ivar([-2;3;-7]), 10), [-418629.86,14204.447,-773.91798], 1e-2);
assertExpr(grdfcn(3, 16, ad_ivar([10;-3;-9]), 10), [-377177.12,-65909.768,-3718.8834], 1e6);

printf("    10/34. meyer - OK.\n");

// Problem 11 - gulf
// TODO - Allow for variable-variable exponentiation.
//assertExpr(objfcn(3, 10, ad_ivar([5;2.5;0.15]), 11), 4.1303867, 1e-7);
//assertExpr(objfcn(3, 10, ad_ivar([50;25;1.5]), 11), 0, 1e-16);
//assertExpr(objfcn(3, 10, ad_ivar([-2;-3;-10]), 11), 8.9385, 1e-4);
//assertExpr(objfcn(3, 10, ad_ivar([3;-1;11]), 11), 0.0385, 1e-4);
//
//assertExpr(grdfcn(3, 10, ad_ivar([5;2.5;0.15]), 11), [0.6461703,0.0094972,-12.715378], 1e-6);
//assertExpr(grdfcn(3, 10, ad_ivar([50;25;1.5]), 11), [0,0,0], 1e-15);
//assertExpr(grdfcn(3, 10, ad_ivar([-2;-3;-10]), 11), [0,0,0], 1e-15);
//assertExpr(grdfcn(3, 10, ad_ivar([3;-1;11]), 11), [0,0,0]);

printf("    11/34. gulf - TODO!\n");


// Problem 12 - box
assertExpr(objfcn(3, 10, ad_ivar([0;10;20]), 12), 1031.1538, 1e-4);
assertExpr(objfcn(3, 10, ad_ivar([1;10;1]), 12), 0);
assertExpr(objfcn(3, 10, ad_ivar([4;-5;-3]), 12), 33877.005, 1e-3);
assertExpr(objfcn(3, 10, ad_ivar([-2;10;0]), 12), 161.10144, 1e-4);

assertExpr(grdfcn(3, 10, ad_ivar([0;10;20]), 12), [98.223431,-2.1193742,112.38817], 1e-5);
assertExpr(grdfcn(3, 10, ad_ivar([1;10;1]), 12), [0,0,0]);
assertExpr(grdfcn(3, 10, ad_ivar([4;-5;-3]), 12), [20.484699,-64819.216,301.29877], 1e-3);
assertExpr(grdfcn(3, 10, ad_ivar([-2;10;0]), 12), [-264.81191,0.2595035,-33.796297], 1e-5);

printf("    12/34. box - OK.\n");


// Problem 13 - sing
assertExpr(objfcn(4, 4, ad_ivar([3;-1;0;1]), 13), 215);
assertExpr(objfcn(4, 4, ad_ivar([0;0;0;0]), 13), 0);
assertExpr(objfcn(4, 4, ad_ivar([-3;5;7;0]), 13), 9825);
assertExpr(objfcn(4, 4, ad_ivar([10;-4;-2;-7]), 13), 836235);

assertExpr(grdfcn(4, 4, ad_ivar([3;-1;0;1]), 13), [306,-144,-2,-310]);
assertExpr(grdfcn(4, 4, ad_ivar([0;0;0;0]), 13), [0,0,0,0]);
assertExpr(grdfcn(4, 4, ad_ivar([-3;5;7;0]), 13), [-986,-1976,5902,1010]);
assertExpr(grdfcn(4, 4, ad_ivar([10;-4;-2;-7]), 13), [196460,-600,50,-196570]);

printf("    13/34. sing - OK.\n");


// Problem 14 - wood
assertExpr(objfcn(4, 6, ad_ivar([-3;-1;-3;-1]), 14), 19192);
assertExpr(objfcn(4, 6, ad_ivar([1;1;1;1]), 14), 0);
assertExpr(objfcn(4, 6, ad_ivar([10;-9;4;-5]), 14), 1230441.6, 1e-1);
assertExpr(objfcn(4, 6, ad_ivar([-8;0;-1;7]), 14), 413179.9, 1e-1);

assertExpr(grdfcn(4, 6, ad_ivar([-3;-1;-3;-1]), 14), [-12008,-2080,-10808,-1880]);
assertExpr(grdfcn(4, 6, ad_ivar([1;1;1;1]), 14), [0,0,0,0]);
assertExpr(grdfcn(4, 6, ad_ivar([10;-9;4;-5]), 14), [436018,-22120.8,30246,-4099.2], 1e-1);
assertExpr(grdfcn(4, 6, ad_ivar([-8;0;-1;7]), 14), [-204818,-12701.4,2156,1181.4], 1e-1);

printf("    14/34. wood - OK.\n");


// Problem 15 - kowosb
assertExpr(objfcn(4, 11, ad_ivar([0.25;0.39;0.415;0.39]), 15), 0.0053132, 1e-7);
assertExpr(objfcn(4, 11, ad_ivar([-3;-10;5;7]), 15), 13.363947, 1e-6);
assertExpr(objfcn(4, 11, ad_ivar([0;0;0;0]), 15), 0.1484132, 1e-7);
assertExpr(objfcn(4, 11, ad_ivar([5;-9;4;-3]), 15), 1330.1109, 1e-4);

assertExpr(grdfcn(4, 11, ad_ivar([0.25;0.39;0.415;0.39]), 15), [0.1335765,-0.0007475,-0.0090056,0.0111355], 1e-7);
assertExpr(grdfcn(4, 11, ad_ivar([-3;-10;5;7]), 15), [-9.8369536,-3.6340338,-2.2469233,-1.9846453], 1e-7);
assertExpr(grdfcn(4, 11, ad_ivar([0;0;0;0]), 15), [-2.0624,0,0,0], 1e-7);
assertExpr(grdfcn(4, 11, ad_ivar([5;-9;4;-3]), 15), [531.90886,-323.58983,640.3392,1760.9935], 1e-4);

printf("    15/34. kowosb - OK.\n");


// Problem 16 - bd
assertExpr(objfcn(4, 20, ad_ivar([25;5;-5;-1]), 16), 7926693.3, 1e-1);
assertExpr(objfcn(4, 20, ad_ivar([0;0;0;0]), 16), 16148735, 1);
assertExpr(objfcn(4, 20, ad_ivar([-10;3;0;9]), 16), 15049403, 1);
assertExpr(objfcn(4, 20, ad_ivar([2;-4;-5;1]), 16), 46118096, 1);

assertExpr(grdfcn(4, 20, ad_ivar([25;5;-5;-1]), 16), [1149322.8,1779291.7,-254579.59,-173400.43], 1e-1);
assertExpr(grdfcn(4, 20, ad_ivar([0;0;0;0]), 16), [-1443666.7,-5423109.4,28644.022,-11582.383], 1e-1);
assertExpr(grdfcn(4, 20, ad_ivar([-10;3;0;9]), 16), [-1437868.3,-5192739.3,-53195.248,135477.94], 1e-1);
assertExpr(grdfcn(4, 20, ad_ivar([2;-4;-5;1]), 16), [-3315905.9,-12214371,-309157.09,115137.88], 1);

printf("    16/34. bd - OK.\n");


// Problem 17 - osb1
global FIRSTIME;
global y;
FIRSTIME = 1;
y = [];
assertExpr(objfcn(5, 33, ad_ivar([-1;1.2;-1.2;0;0.5]), 17), 10.27504, 1e-6);
assertExpr(objfcn(5, 33, ad_ivar([1;-1;0;1;-2]), 17), 6.3386366, 1e-7);
assertExpr(objfcn(5, 33, ad_ivar([0.5;1.5;-1;0.01;0.02]), 17), 0.8790263, 1e-7);
assertExpr(objfcn(5, 33, ad_ivar([.3754,1.9358,-1.4647,0.01287,0.02212]'), 17), 0.0000553, 1e-7);

r = grdfcn(5, 33, ad_ivar([-1;1.2;-1.2;0;0.5]), 17);
assert(and(abs(Expr_GetEvaluation(r) - [-30.850281,-30.850281,-3.6977168,4092.3887,-0.1174104]) < [1e-6,1e-6,1e-7,1e-4,1e-7]));
r = grdfcn(5, 33, ad_ivar([1;-1;0;1;-2]), 17);
assert(and(abs(Expr_GetEvaluation(r) - [22.365909,-1.6879917,1.055e+278,0.0000835,0]) < [1e-6,1e-7,1e275,1e-7,1e-8]));
r = grdfcn(5, 33, ad_ivar([0.5;1.5;-1;0.01;0.02]), 17);
assert(and(abs(Expr_GetEvaluation(r) - [10.709952,3.0646452,1.5810648,-411.65597,76.261736]) < [1e-6,1e-7,1e-7,1e-5,1e-6]));
r = grdfcn(5, 33, ad_ivar([.3754,1.9358,-1.4647,0.01287,0.02212]'), 17);
assert(and(abs(Expr_GetEvaluation(r) - [-0.0084585,-0.0027336,-0.0016602,0.3387203,-0.1109241]) < [1e-7,1e-7,1e-7,1e-7,1e-7]));

printf("    17/34. osb1 - OK.\n");


// Problem 18 - biggs
assertExpr(objfcn(6, 13, ad_ivar([1;2;1;1;1;1]), 18), 0.7790701, 1e-7);
assertExpr(objfcn(6, 13, ad_ivar([0;-3;5;8;2;-4]), 18), 334393.69, 1e-2);
assertExpr(objfcn(6, 13, ad_ivar([1;10;1;5;4;3]), 18), 0);
assertExpr(objfcn(6, 13, ad_ivar([-9;3;0;0;4;-1]), 18), 15.380567, 1e-6);

assertExpr(grdfcn(6, 13, ad_ivar([1;2;1;1;1;1]), 18), [-0.1493719,-0.1831635,-1.483958,1.4282775,-0.1493719,-1.483958], 1e-7);
assertExpr(grdfcn(6, 13, ad_ivar([0;-3;5;8;2;-4]), 18), [15225.226,-803085.66,-2911.6115,85038.984,-1457.481,-431.74801], 1e-2);
assertExpr(grdfcn(6, 13, ad_ivar([1;10;1;5;4;3]), 18), [0,0,0,0,0,0]);
assertExpr(grdfcn(6, 13, ad_ivar([-9;3;0;0;4;-1]), 18), [0,0,-132774.43,8.1883412,-1.5270122,-6.3327761], 1e-2);

printf("    18/34. biggs - OK.\n");


// Problem 19 - osb2
FIRSTIME = 1;
y = [];
assertExpr(objfcn(11, 65, ad_ivar([1.3,0.65,0.65,0.7,0.6,3,5,7,2,4.5,5.5]'), 19), 2.0934195, 1e-7);
assertExpr(objfcn(11, 65, ad_ivar([1.31;0.4315;0.6336;0.5993;0.7539;0.9056;1.3651;4.8248;2.3988;4.5689;5.6754]), 19), ...
	0.0401379, 1e-7);
assertExpr(objfcn(11, 65, ad_ivar([0;-2;2;1;-1;0;0;-1;1;-2;0]), 19), 4.104e35, 1e32);
assertExpr(objfcn(11, 65, ad_ivar([3;2;1;1;-0.5;1.3;-1.4;4;6;0;-1]), 19), 6.622e49, 1e46);

// TODO - In current representation, gradient evaluation vs. sciad is MUCH TOO LONG for this problem.
//assertExpr(grdfcn(11, 65, ad_ivar([1.3,0.65,0.65,0.7,0.6,3,5,7,2,4.5,5.5]'), 19), ...
//	[1.630377,4.4861865,-0.9346409,-1.184164,-0.4129027,-0.1005784,0.1575078,0.1178320,- 2.8188567,0.0424960,-1.2184054], 1e-7);
//assertExpr(grdfcn(11, 65, ad_ivar([1.31;0.4315;0.6336;0.5993;0.7539;0.9056;1.3651;4.8248;2.3988;4.5689;5.6754]), 19), ...
//	[0.0001724,0.0004917,-0.0008600,-0.0000684,0.0002598,0.0002404,0.0001110,0.0000043,0.0002630,0.0001364,0.000591], 1e-7);
//r = grdfcn(11, 65, ad_ivar([0;-2;2;1;-1;0;0;-1;1;-2;0]), 19);
//assert(and(abs(r - [9.945e20,1.715e18,1.715e18,8.208e35,0,9.857e19,-2.398e20,-3.353e37,0,0,-1.049e37]) < ...
//	[1e17,1e15,1e15,1e32,1e-8,1e16,1e17,1e34,1e-8,1e-8,1e34]));
//r = grdfcn(11, 65, ad_ivar([3;2;1;1;-0.5;1.3;-1.4;4;6;0;-1]), 19);
//assert(and(abs(r - [4.694e26,1.599e25,1.324e50,0.1779596,-8.985e27,-4.641e24,-5.420e51,-0.2078450,3.137e25,-2.372e51,1.531155]) < ...
//	[1e23,1e22,1e47,1e-7,1e24,1e21,1e48,1e-7,1e22,1e48,1e-7]));

printf("    19/34. osb2 - OK (objective only).\n");


// Problem 20 - watson

assertExpr(objfcn(6, 31, ad_ivar(zeros(6,1)), 20), 30);
assertExpr(objfcn(6, 31, ad_ivar([1;-2;10;0;-4;3]), 20), 7257.8203, 1e-4);
assertExpr(objfcn(6, 31, ad_ivar([-9;0;2;10;7;7]), 20), 131469.53, 1e-2);
assertExpr(objfcn(6, 31, ad_ivar([0.5;10;3;5;-4;-9]), 20), 60220.247, 1e-3);

assertExpr(grdfcn(6, 31, ad_ivar(zeros(6,1)), 20), [0,-60,-60,-61.034483,-62.068966,-63.114929], 1e-6);
assertExpr(grdfcn(6, 31, ad_ivar([1;-2;10;0;-4;3]), 20), [6227.6799,5154.3939,4344.6839,3660.5335,3087.415,2600.8844], 1e-4);
assertExpr(grdfcn(6, 31, ad_ivar([-9;0;2;10;7;7]), 20), [-16000.521,9651.4009,15393.86,16651.379,16621.948,16136.705], 1e-3);
assertExpr(grdfcn(6, 31, ad_ivar([0.5;10;3;5;-4;-9]), 20), [29416.866,20528.079,14701.896,10840.938,8135.3781,6157.1077], 1e-3);

printf("    20/34. watson - OK.\n");


// Problem 21 - rosex (8x8)
assertExpr(objfcn(8, 8, ad_ivar([-1.2;1;-1.2;1;-1.2;1;-1.2;1]), 21), 96.8);
assertExpr(objfcn(8, 8, ad_ivar([-10;3;0;5;-7;-2;3;0]), 21), 1211790);
assertExpr(objfcn(8, 8, ad_ivar([0;0;-10;4;7;-6;1;0.5]), 21), 1224283);
assertExpr(objfcn(8, 8, ad_ivar([5;2;-4;0;-1;1;0.2;-1.3]), 21), 78725.2);

assertExpr(grdfcn(8, 8, ad_ivar([-1.2;1;-1.2;1;-1.2;1;-1.2;1]), 21), [-215.6,-88,-215.6,-88,-215.6,-88,-215.6,-88]);
assertExpr(grdfcn(8, 8, ad_ivar([-10;3;0;5;-7;-2;3;0]), 21), [-388022,-19400,-2,1000,-142816,-10200,10804,-1800]);
assertExpr(grdfcn(8, 8, ad_ivar([0;0;-10;4;7;-6;1;0.5]), 21), [-2,0,-384022,-19200,154012,-11000,200,-100]);
assertExpr(grdfcn(8, 8, ad_ivar([5;2;-4;0;-1;1;0.2;-1.3]), 21), [46008,-4600,-25610,-3200,-4,0,105.6,-268]);

printf("    21/34. rosex - OK.\n");


// Problem 22 - singx (8x8)
assertExpr(objfcn(8, 8, ad_ivar([3;-1;0;1;3;-1;0;1]), 22), 430);
assertExpr(objfcn(8, 8, ad_ivar([0;0;0;0;0;0;0;0]), 22), 0);
assertExpr(objfcn(8, 8, ad_ivar([10;4;-2;6;-7;9;-1;3]), 22), 131086);
assertExpr(objfcn(8, 8, ad_ivar([-5;2;7;-0.5;1;1;-4;-2]), 22), 32854.875);

assertExpr(grdfcn(8, 8, ad_ivar([3;-1;0;1;3;-1;0;1]), 22), [306,-144,-2,-310,306,-144,-2,-310]);
assertExpr(grdfcn(8, 8, ad_ivar([0;0;0;0;0;0;0;0]), 22), [0,0,0,0,0,0,0,0]);
assertExpr(grdfcn(8, 8, ad_ivar([10;4;-2;6;-7;9;-1;3]), 22), [2660,3048,-4176,-2480,-39834,6984,-10688,40040]);
assertExpr(grdfcn(8, 8, ad_ivar([-5;2;7;-0.5;1;1;-4;-2]), 22), [-3615,-6612,13899,3570,1102,3136,-5852,-1060]);

printf("    22/34. singx - OK.\n");


// Problem 23 - pen1
assertExpr(objfcn(6, 6, ad_ivar([1;2;3;4;5;6]), 23), 8235.5631, 1e-4);
assertExpr(objfcn(6, 6, ad_ivar([0;0;0;0;0;0]), 23), 0.06256, 1e-8);
assertExpr(objfcn(6, 6, ad_ivar([10;-2;-5;-9;3;1]), 23), 48290.065, 1e-3);
assertExpr(objfcn(6, 6, ad_ivar([-0.8;7;-2;2;3;-8]), 23), 17001.553, 1e-3);

assertExpr(grdfcn(6, 6, ad_ivar([1;2;3;4;5;6]), 23), [363,726.00002,1089,1452.0001,1815.0001,2178.0001], 1e-4);
assertExpr(grdfcn(6, 6, ad_ivar([0;0;0;0;0;0]), 23), [-0.00002,-0.00002,-0.00002,-0.00002,-0.00002,-0.00002], 1e-8);
assertExpr(grdfcn(6, 6, ad_ivar([10;-2;-5;-9;3;1]), 23), [8790.0002,-1758.0001,-4395.0001,-7911.0002,2637,879], 1e-4);
assertExpr(grdfcn(6, 6, ad_ivar([-0.8;7;-2;2;3;-8]), 23), [-417.24804,3650.9201,-1043.1201,1043.12,1564.68,-4172.4802], 1e-4);

printf("    23/34. pen1 - OK.\n");


// Problem 24 - pen2
assertExpr(objfcn(7, 14, ad_ivar([0.5;0.5;0.5;0.5;0.5;0.5;0.5]), 24), 36.090069, 1e-6);
assertExpr(objfcn(7, 14, ad_ivar([0;0;0;0;0;0;0]), 24), 1.0400803, 1e-7);
assertExpr(objfcn(7, 14, ad_ivar([5;-2;3;-1;-4;-3.5;1]), 24), 102743.29, 1e-2);
assertExpr(objfcn(7, 14, ad_ivar([-2;2;-5;-3;-3;1;3]), 24), 62504.84, 1e-3);

assertExpr(grdfcn(7, 14, ad_ivar([0.5;0.5;0.5;0.5;0.5;0.5;0.5]), 24), ...
	[84.6,71.999999,59.999998,47.999997,35.999995,23.999994,11.999997], 1e-6);
assertExpr(grdfcn(7, 14, ad_ivar([0;0;0;0;0;0;0]), 24), ...
	[-0.4000007,-0.0000016,-0.0000026,-0.0000038,-0.0000050,-0.0000064,-0.0000035], 1e-7);
assertExpr(grdfcn(7, 14, ad_ivar([5;-2;3;-1;-4;-3.5;1]), 24), ...
	[44879.6,-15384,19230,-5128,-15384,-8974,1282], 1e-3);
assertExpr(grdfcn(7, 14, ad_ivar([-2;2;-5;-3;-3;1;3]), 24), ...
	[-14004.4,12000,-25000,-12000,-9000,2000,3000], 1e-3);

printf("    24/34. pen2 - OK.\n");


// Problem 25 - vardim
assertExpr(objfcn(9, 11, ad_ivar([8/9;7/9;6/9;5/9;4/9;3/9;2/9;1/9;0]), 25), 1006569.6, 1e-1);
assertExpr(objfcn(9, 11, ad_ivar([1;1;1;1;1;1;1;1;1]), 25), 0);
assertExpr(objfcn(9, 11, ad_ivar([0;-3;3;2;0;-1;-2;-2;-3]), 25), 88538751, 1);
assertExpr(objfcn(9, 11, ad_ivar([-2;-1;1.5;-2.5;-0.5;1;1;0;2]), 25), 457681.75, 1e-2);

assertExpr(grdfcn(9, 11, ad_ivar([8/9;7/9;6/9;5/9;4/9;3/9;2/9;1/9;0]), 25), ...
	[-127082.07,-254164.15,-381246.22,-508328.3,-635410.37,-762492.44,-889574.52,-1016656.6,-1143738.7], 1e-1);
assertExpr(grdfcn(9, 11, ad_ivar([1;1;1;1;1;1;1;1;1]), 25), zeros(1,9));
assertExpr(grdfcn(9, 11, ad_ivar([0;-3;3;2;0;-1;-2;-2;-3]), 25), ...
	[-3650888,-7301780,-10952654,-14603542,-18254432,-21905320,-25556208,-29207094,-32857982], 1);
assertExpr(grdfcn(9, 11, ad_ivar([-2;-1;1.5;-2.5;-0.5;1;1;0;2]), 25), ...
	[-70362,-140716,-211067,-281431,-351783,-422136,-492492,-562850,-633202], 1e-2);

printf("    25/34. vardim - OK.\n");


// Problem 26 - trig
assertExpr(objfcn(7, 7, ad_ivar([1/7;1/7;1/7;1/7;1/7;1/7;1/7]), 26), 0.0093396, 1e-7);
assertExpr(objfcn(7, 7, ad_ivar([0;0;0;0;0;0;0]), 26), 0);
assertExpr(objfcn(7, 7, ad_ivar([5;-2;10;-3;-5;2;-1]), 26), 1291.3336, 1e-4);
assertExpr(objfcn(7, 7, ad_ivar([3;-10;6;7;-9;-3;4]), 26), 1840.7615, 1e-4);

assertExpr(grdfcn(7, 7, ad_ivar([1/7;1/7;1/7;1/7;1/7;1/7;1/7]), 26), ...
	[0.0427516,0.0110523,-0.0148457,-0.0349426,-0.0492382,-0.0577327,-0.0604259], 1e-7);
assertExpr(grdfcn(7, 7, ad_ivar([0;0;0;0;0;0;0]), 26), [0,0,0,0,0,0,0]);
assertExpr(grdfcn(7, 7, ad_ivar([5;-2;10;-3;-5;2;-1]), 26), ...
	[-205.17117,-204.9349,-125.15585,-12.279277,280.58379,360.00919,-319.95313], 1e-5);
assertExpr(grdfcn(7, 7, ad_ivar([3;-10;6;7;-9;-3;4]), 26), ...
	[56.477213,166.61691,-96.445416,179.08865,-133.98043,-24.186889,-367.48851], 1e-4);

printf("    26/34. trig - OK.\n");


// Problem 27 - almost

printf("    27/34. almost - N/A.\n");


// Problem 28 - bv
assertExpr(objfcn(8, 8, ad_ivar([-8/81;-14/81;-18/81;-20/81;-20/81;-18/81;-14/81;-8/81]), 28), 0.0013750, 1e-7);
assertExpr(objfcn(8, 8, ad_ivar([0;0;0;0;0;0;0;0]), 28), 0.0050490, 1e-7);
assertExpr(objfcn(8, 8, ad_ivar([4;-9;3;2;1;-2;6;0]), 28), 1725.419, 1e-4);
assertExpr(objfcn(8, 8, ad_ivar([-1;4;-10;-7;-4;-3;5;3]), 28), 1072.0172, 1e-4);

assertExpr(grdfcn(8, 8, ad_ivar([-8/81;-14/81;-18/81;-20/81;-20/81;-18/81;-14/81;-8/81]), 28), ...
	[-0.0387261,-0.0019268,-0.0023400,-0.0029476,-0.0037610,-0.0047632,-0.0058736,0.0424946], 1e-7);
assertExpr(grdfcn(8, 8, ad_ivar([0;0;0;0;0;0;0;0]), 28), ...
	[0.0117167,-0.0004942,-0.0002559,0.0001166,0.0006598,0.0014160,0.0024341,0.1025354], 1e-7);
assertExpr(grdfcn(8, 8, ad_ivar([4;-9;3;2;1;-2;6;0]), 28), ...
	[144.35106,-236.79051,118.70385,-30.09075,30.41673,-82.060967,139.40917,-58.429701], 1e-5);
assertExpr(grdfcn(8, 8, ad_ivar([-1;4;-10;-7;-4;-3;5;3]), 28), ...
	[-63.760961,153.63216,-180.18516,32.773207,24.208162,-56.184015,78.558802,-15.434999], 1e-5);

printf("    28/34. bv - OK.\n");


// Problem 29 - ie

assertExpr(objfcn(8, 8, ad_ivar([-8/81;-14/81;-18/81;-20/81;-20/81;-18/81;-14/81;-8/81]), 29), 0.0522958, 1e-7);
assertExpr(objfcn(8, 8, ad_ivar([0;0;0;0;0;0;0;0]), 29), 0.2437548, 1e-7);
assertExpr(objfcn(8, 8, ad_ivar([4;-9;3;2;1;-2;6;0]), 29), 265.73058, 1e-5);
assertExpr(objfcn(8, 8, ad_ivar([-1;4;-10;-7;-4;-3;5;3]), 29), 612.12885, 1e-5);

assertExpr(grdfcn(8, 8, ad_ivar([-8/81;-14/81;-18/81;-20/81;-20/81;-18/81;-14/81;-8/81]), 29), ...
	[-0.1206519,-0.2044007,-0.2523683,-0.2664182,-0.2492865,-0.2051391,-0.1406522,-0.0666416], 1e-7);
assertExpr(grdfcn(8, 8, ad_ivar([0;0;0;0;0;0;0;0]), 29), ...
	[0.1656806,0.3226244,0.4608186,0.5668797,0.6243990,0.6147731,0.5186503,0.3181457], 1e-7);
assertExpr(grdfcn(8, 8, ad_ivar([4;-9;3;2;1;-2;6;0]), 29), ...
	[10.641152,-13.710228,15.865353,15.966154,12.704402,2.5997971,74.140016,5.8845315], 1e-6);
assertExpr(grdfcn(8, 8, ad_ivar([-1;4;-10;-7;-4;-3;5;3]), 29), ...
	[-6.2533034,-48.929839,-229.61649,-112.27467,-31.075744,-13.275308,-38.861524,-4.3606502], 1e-5);

printf("    29/34. ie - OK.\n");


// Problem 30 - trid
assertExpr(objfcn(9, 9, ad_ivar([-1;-1;-1;-1;-1;-1;-1;-1;-1]), 30), 20);
assertExpr(objfcn(9, 9, ad_ivar([0;2;-3;4;6;-10;7;-3;-2]), 30), 70025);
assertExpr(objfcn(9, 9, ad_ivar([10;-3;5;6;2;1;0;0;-7]), 30), 48608);
assertExpr(objfcn(9, 9, ad_ivar([-1;5;-6;-9;-8;2;0;-3;-4]), 30), 57283);

assertExpr(grdfcn(9, 9, ad_ivar([-1;-1;-1;-1;-1;-1;-1;-1;-1]), 30), [-26,-4,-8,-8,-8,-8,-8,-4,-38]);
assertExpr(grdfcn(9, 9, ad_ivar([0;2;-3;4;6;-10;7;-3;-2]), 30), [-28,34,-1044,946,2164,-21146,4054,-610,-104]);
assertExpr(grdfcn(9, 9, ad_ivar([10;-3;5;6;2;1;0;0;-7]), 30), [12154,-642,1770,2794,338,36,-30,326,-7376]);
assertExpr(grdfcn(9, 9, ad_ivar([-1;5;-6;-9;-8;2;0;-3;-4]), 30), [-154,922,-3688,-12352,-9570,504,38,-480,-1448]);

printf("    30/34. trid - OK.\n");


// Problem 31 - band

assertExpr(objfcn(9, 9, ad_ivar([-1;-1;-1;-1;-1;-1;-1;-1;-1]), 31), 324);
assertExpr(objfcn(9, 9, ad_ivar([0;2;-3;4;6;-10;7;-3;-2]), 31), 30186154);
assertExpr(objfcn(9, 9, ad_ivar([10;-3;5;6;2;1;0;0;-7]), 31), 29581361);
assertExpr(objfcn(9, 9, ad_ivar([-1;5;-6;-9;-8;2;0;-3;-4]), 31), 23803429);

assertExpr(grdfcn(9, 9, ad_ivar([-1;-1;-1;-1;-1;-1;-1;-1;-1]), 31), [-144,-132,-132,-132,-144,-156,-168,-180,-192]);
assertExpr(grdfcn(9, 9, ad_ivar([0;2;-3;4;6;-10;7;-3;-2]), 31), ...
	[-8080,-20304,-18884,71954,950462,-15540480,2126580,-110574,-29732]);
assertExpr(grdfcn(9, 9, ad_ivar([10;-3;5;6;2;1;0;0;-7]), 31), ...
	[15098996,-136900,363140,972282,-30270,-19098,-4510,-4210,-2617626]);
assertExpr(grdfcn(9, 9, ad_ivar([-1;5;-6;-9;-8;2;0;-3;-4]), 31), ...
	[13218,279560,-1149266,-9024598,-5072722,-54682,-2648,-79924,-221604]);

printf("    31/34. band - OK.\n");


// Problem 32 - lin
assertExpr(objfcn(6, 10, ad_ivar([1;1;1;1;1;1]), 32), 28);
assertExpr(objfcn(6, 10, ad_ivar([-1;-1;-1;-1;-1;-1]), 32), 4);
assertExpr(objfcn(6, 10, ad_ivar([0;-5;9;2;3;-7]), 32), 182);
assertExpr(objfcn(6, 10, ad_ivar([-2;4;1;10;5;-2]), 32), 192);

assertExpr(grdfcn(6, 10, ad_ivar([1;1;1;1;1;1]), 32), [4,4,4,4,4,4]);
assertExpr(grdfcn(6, 10, ad_ivar([-1;-1;-1;-1;-1;-1]), 32), [0,0,0,0,0,0], 1e-15);
assertExpr(grdfcn(6, 10, ad_ivar([0;-5;9;2;3;-7]), 32), [2,-8,20,6,8,-12]);
assertExpr(grdfcn(6, 10, ad_ivar([-2;4;1;10;5;-2]), 32), [-2,10,4,22,12,-2]);

printf("    32/34. lin - OK.\n");


// Problem 33 - lin1
assertExpr(objfcn(6, 10, ad_ivar([1;1;1;1;1;1]), 33), 167485);
assertExpr(objfcn(6, 10, ad_ivar([-1;-1;-1;-1;-1;-1]), 33), 172105);
assertExpr(objfcn(6, 10, ad_ivar([0;-5;9;2;3;-7]), 33), 1770);
assertExpr(objfcn(6, 10, ad_ivar([-2;4;1;10;5;-2]), 33), 1473130);

assertExpr(grdfcn(6, 10, ad_ivar([1;1;1;1;1;1]), 33), [16060,32120,48180,64240,80300,96360]);
assertExpr(grdfcn(6, 10, ad_ivar([-1;-1;-1;-1;-1;-1]), 33), [-16280,-32560,-48840,-65120,-81400,-97680]);
assertExpr(grdfcn(6, 10, ad_ivar([0;-5;9;2;3;-7]), 33), [-1650,-3300,-4950,-6600,-8250,-9900]);
assertExpr(grdfcn(6, 10, ad_ivar([-2;4;1;10;5;-2]), 33), [47630,95260,142890,190520,238150,285780]);

printf("    33/34. lin1 - OK.\n");

// Problem 34 - lin0
assertExpr(objfcn(6, 10, ad_ivar([1;1;1;1;1;1]), 34), 38986);
assertExpr(objfcn(6, 10, ad_ivar([-1;-1;-1;-1;-1;-1]), 34), 41002);
assertExpr(objfcn(6, 10, ad_ivar([0;-5;9;2;3;-7]), 34), 323530);
assertExpr(objfcn(6, 10, ad_ivar([-2;4;1;10;5;-2]), 34), 1172842);

assertExpr(grdfcn(6, 10, ad_ivar([1;1;1;1;1;1]), 34), [0,11280,16920,22560,28200,0]);
assertExpr(grdfcn(6, 10, ad_ivar([-1;-1;-1;-1;-1;-1]), 34), [0,-11568,-17352,-23136,-28920,0]);
assertExpr(grdfcn(6, 10, ad_ivar([0;-5;9;2;3;-7]), 34), [0,32496,48744,64992,81240,0]);
assertExpr(grdfcn(6, 10, ad_ivar([-2;4;1;10;5;-2]), 34), [0,61872,92808,123744,154680,0]);

printf("    34/34. lin0 - OK.\n");


clear r;
clearglobal FIRSTIME;
clearglobal y;

printf("Expr - UNC problems : all unit tests passed.\n");

// End of script.

