//
// Tests on expressions made solely of a R^n variable or constant matrix.
//
// Author : Benoit Hamelin
// 2002 Universit de Sherbrooke
//


x = ad_ivar(7);
assert(and(size(x) == [1,1]));
assert(Expr_GetNodeCount(x) == 1);
assert(Expr_GetEdges(x) == []);
assert(Expr_GetEvaluation(x) == 7);

ex = x(1);
assert(and(size(ex) == size(x)));
assert(Expr_GetEvaluation(ex) == Expr_GetEvaluation(x));

assert(and(size(x) == size(x')));
assert(and(Expr_GetEvaluation(x) == Expr_GetEvaluation(x')));
assert(and(Expr_GetNodeCount(x) == Expr_GetNodeCount(x')));

y = ad_ivar([10,11,12,13,14]);
assert(and(size(y) == [5,1]));
assert(Expr_GetNodeCount(y) == 1);
assert(Expr_GetEdges(y) == []);
assert(and(Expr_GetEvaluation(y) == [10;11;12;13;14]));

ex = y(:);
assert(and(size(y) == size(ex)));
assert(and(Expr_GetEvaluation(y) == Expr_GetEvaluation(ex)));
ex = y(2);
assert(and(size(ex) == [1,1]));
assert(and(Expr_GetEvaluation(ex) == 11));
ex = y(2:4);
assert(and(size(ex) == [3,1]));
assert(and(Expr_GetEvaluation(ex) == [11;12;13]));
ex = y(3:$);
assert(and(size(ex) == [3,1]));
assert(and(Expr_GetEvaluation(ex) == [12;13;14]));
ex = y(4, 1);
assert(and(size(ex) == [1,1]));
assert(and(Expr_GetEvaluation(ex) == 13));

yt = y';
assert(and(Expr_GetNodeCount(yt) == 2));
assert(and(Expr_GetEvaluation(yt) == [10,11,12,13,14]));
assert(and(Expr_GetEvaluation(yt([2,4])) == [11,13]));
assert(and(size(y(:)) == size(yt(:))));
assert(and(Expr_GetEvaluation(y(:)) == Expr_GetEvaluation(yt(:))));


c = ad_const(5);
assert(and(size(c) == [1,1]));
assert(Expr_GetNodeCount(c) == 1);
assert(Expr_GetEdges(c) == []);
assert(Expr_GetEvaluation(c) == 5);

ex = c(:);
assert(and(size(ex) == size(c)));
assert(and(Expr_GetEvaluation(ex) == 5));
ex = c(1, 1);
assert(and(size(ex) == size(c)));
assert(and(Expr_GetEvaluation(ex) == 5));

assert(and(size(c) == size(c')));
assert(and(Expr_GetEvaluation(c) == Expr_GetEvaluation(c')));


c = ad_const([3;4;5;6]);
assert(and(size(c) == [4,1]));
assert(Expr_GetNodeCount(c) == 1);
assert(Expr_GetEdges(c) == []);
assert(and(Expr_GetEvaluation(c) == [3;4;5;6]));

ex = c(3);
assert(and(size(ex) == [1,1]));
assert(and(Expr_GetEvaluation(ex) == 5));
ex = c(3, 1);
assert(and(size(ex) == [1,1]));
assert(Expr_GetEvaluation(ex) == 5);
ex = c([2, 4]);
assert(and(size(ex) == [2,1]));
assert(and(Expr_GetEvaluation(ex) == [4; 6]));
ex = c(1:$);
assert(and(size(ex) == size(c)));
assert(and(Expr_GetEvaluation(ex) == [3; 4; 5; 6]));
ex = c(:);
assert(and(size(ex) == size(c)));
assert(and(Expr_GetEvaluation(ex) == [3;4;5;6]));

ct = c';
assert(and(size(ct) == [1,4]));
assert(and(Expr_GetNodeCount(ct) == 1));
assert(and(Expr_GetEvaluation(ct) == [3,4,5,6]));
assert(and(Expr_GetEvaluation(c(3)) == Expr_GetEvaluation(ct(3))));


c = ad_const([3,4,5,6]);
assert(and(size(c) == [1,4]));
assert(Expr_GetNodeCount(c) == 1);
assert(Expr_GetEdges(c) == []);
assert(and(Expr_GetEvaluation(c) == [3,4,5,6]));

ex = c(3);
assert(and(size(ex) == [1,1]));
assert(and(Expr_GetEvaluation(ex) == 5));
ex = c(1, 3);
assert(and(size(ex) == [1,1]));
assert(and(Expr_GetEvaluation(ex) == 5));
ex = c(:);
assert(and(size(ex) == [4,1]));
assert(and(Expr_GetEvaluation(ex) == [3; 4; 5; 6]));
ex = c([2,3,4]);
assert(and(size(ex) == [1,3]));
assert(and(Expr_GetEvaluation(ex) == [4,5,6]));

ct = c';
assert(and(size(ct) == [4,1]));
assert(and(Expr_GetNodeCount(ct) == 1));
assert(and(Expr_GetEvaluation(ct) == [3;4;5;6]));
assert(and(Expr_GetEvaluation(c(3)) == Expr_GetEvaluation(ct(3))));


c = ad_const([1,3,6 ; 1,9,1 ; 3,4,7]);
assert(and(size(c) == [3,3]));
assert(Expr_GetNodeCount(c) == 1);
assert(Expr_GetEdges(c) == []);
assert(Expr_GetEvaluation(c) == [1,3,6 ; 1,9,1 ; 3,4,7]);

//ex = c(5);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(ex) == [1,1]));
//assert(and(Expr_GetEvaluation(ex) == 9));
//ex = c(3, 2);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(ex) == [1,1]));
//assert(and(Expr_GetEvaluation(ex) == 4));
//ex = c(:);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(ex) == [9,1]));
//assert(and(Expr_GetEvaluation(ex) == [1; 1; 3; 3; 9; 4; 6; 1; 7]));
//ex = c(6:$);  // Single-index extraction not supported anymore on matrices.
//assert(and(size(ex) == [4,1]));
//assert(and(Expr_GetEvaluation(ex) == [4; 6; 1; 7]));
ex = c(1:2, 2:3);
assert(and(size(ex) == [2,2]));
assert(and(Expr_GetEvaluation(ex) == [3,6 ; 9,1]));
ex = c(3, :);
assert(and(size(ex) == [1,3]));
assert(and(Expr_GetEvaluation(ex) == [3, 4, 7]));
ex = c(:, 1);
assert(and(size(ex) == [3,1]));
assert(and(Expr_GetEvaluation(ex) == [1; 1; 3]));
ex = c(:, 2:$);
assert(and(size(ex) == [3,2]));
assert(and(Expr_GetEvaluation(ex) == [3,6 ; 9,1 ; 4,7]));

ct = c';
assert(and(size(ct) == size(c)));
assert(and(Expr_GetEvaluation(ct(2,2)) == Expr_GetEvaluation(c(2,2))));
//assert(and(Expr_GetEvaluation(ct(:)) == [1;3;6;1;9;1;3;4;7]));  // Single-index extraction not supported anymore on matrices.
//assert(and(Expr_GetEvaluation(ct([2,6,8])) == [3;1;4]));  // Single-index extraction not supported anymore on matrices.
assert(and(Expr_GetEvaluation(ct([2,3],[1,2])) == [3,9 ; 6,1]));


c = ad_const([1,2,3,4 ; 5,6,7,8 ; 9,10,11,12]);
ct = c';
//assert(and(Expr_GetEvaluation(ct([3,8,10])) == [3;8;10]));  // Single-index extraction not supported anymore on matrices.
assert(and(Expr_GetEvaluation(ct(:,2)) == [5;6;7;8]));
assert(and(Expr_GetEvaluation(ct(3,:)) == [3,7,11]));
assert(and(Expr_GetEvaluation(ct([2,3],[2,3])) == [6,10;7,11]));


// Hypermatrices not supported anymore.
//
//c = ad_const(hypermat([2,2,2,2], 1:16));
//assert(and(size(c) == [2,2,2,2]));
//assert(Expr_GetNodeCount(c) == 1);
//assert(Expr_GetEdges(c) == []);
//h = Expr_GetEvaluation(c);
//assert(typeof(h) == "hypermat");
//assert(and(size(h) == [2,2,2,2]));
//assert(and(h(:)' == (1:16)));
//
//ex = c(6);
//assert(and(size(ex) == [1,1]));
//assert(and(Expr_GetEvaluation(ex) == 6));
//ex = c(1,2,2,1);
//assert(and(size(ex) == [1,1]));
//h = Expr_GetEvaluation(ex);
//assert(and(size(h) == size(ex)));
//assert(and(h(:) == 7));
//ex = c(:, 1, 1, :);
//assert(and(size(ex) == [2,1,1,2]));
//h = Expr_GetEvaluation(ex);
//assert(and(size(h) == size(ex)));
//assert(and(h(:) == [1;2;9;10]));
//ex = c($, 1, 2, :);
//assert(and(size(ex) == [1,1,1,2]));
//h = Expr_GetEvaluation(ex);
//assert(and(size(h) == size(ex)));
//assert(and(h(:) == [6;14]));
//ex = c(:, :, 2, 1);
//assert(and(size(ex) == [2,2]));
//assert(and(Expr_GetEvaluation(ex) == [5,7;6,8]));

// Tests for cancellation of transposition.
assert(Expr_GetNodeCount(x) == 1);
assert(Expr_GetNodeCount(x') == 1);
assert(Expr_GetNodeCount((x')') == 1);
assert(Expr_GetNodeCount(((x')')') == 1);
assert(Expr_GetNodeCount((((x')')')') == 1);
assert(Expr_GetNodeCount(y) == 1);
assert(Expr_GetNodeCount(y') == 2);
assert(Expr_GetNodeCount((y')') == 1);
assert(Expr_GetNodeCount(((y')')') == 2);
assert(Expr_GetNodeCount((((y')')')') == 1);
assert(and(size((((y')')')') == size(y)));


clear x;
clear g;
clear n;
clear ex;
clear y;
clear l1;
clear l2;
clear c;
clear h;
clear yt;
clear ct;

printf("Expr - VarConst : all unit tests passed.\n");


// End of script.

