//
// Tests on Taylor development : regular and element-wise exponentiation and logarithm.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


x = ad_ivar(4);

// Tests on regular exponentiation.
assertTaylor(exp(x), 4, 2, [exp(2) ; exp(2);  exp(2)/2 ; exp(2)/6 ; exp(2)/24], 1e-15);
assertTaylor(2^(-x*x), 3, 3, ...
	[ ...
		2^(-9) ; ...
		-3 * 2^(-8) * log(2) ; ...
		2^(-9) * log(2) * (2*9*log(2) - 1) ; ...
		log(2)^2 * 2^(-9) * (6 - 4*9*log(2)) ...
	], 1e-15);

// Tests on element-wise exponentiation.
assertTaylor(2 .^ (-x*x), 3, 3, ...
	[ ...
		2^(-9) ; ...
		-3 * 2^(-8) * log(2) ; ...
		2^(-9) * log(2) * (2*9*log(2) - 1) ; ...
		log(2)^2 * 2^(-9) * (6 - 4*9*log(2)) ...
	], 1e-15);
assertTaylor([3;%e;1;0.5] .^ [x;x*x+3;x;x], 3, 3, ...
	[ ...
		27,			27*log(3),	27*(log(3)^2)/2,	9*(log(3)^3)/2	; ...
		exp(12),	6*exp(12),	19*exp(12),			126*exp(12)/3	; ...
		 1,			0,			0,					0				; ...
		 1/8,		-log(2)/8,	(log(2)^2)/16,		-(log(2)^3)/48	  ...
	], 1e-8);
assertTaylor([3,%e,1,0.5] .^ [x,x*x+3,x,x], 3, 3, ...
	[ ...
		27,			27*log(3),	27*(log(3)^2)/2,	9*(log(3)^3)/2	; ...
		exp(12),	6*exp(12),	19*exp(12),			126*exp(12)/3	; ...
		 1,			0,			0,					0				; ...
		 1/8,		-log(2)/8,	(log(2)^2)/16,		-(log(2)^3)/48	  ...
	]', 1e-8);
assertTaylor([3,1;%e,0.5] .^ [x,x;x*x+3,x], 3, 3, ...
	hypermat([2,2,4], ...
		[ ...
			27;					exp(12);		1;	1/8;			...
			27*log(3);			6*exp(12);		0;	-log(2)/8;		...
			27*(log(3)^2)/2;	19*exp(12);		0;	(log(2)^2)/16;	...
			9*(log(3)^3)/2;		126*exp(12)/3;	0;	-(log(2)^3)/48	...
		]), 1e-8);

// Tests on logarithms.
assertTaylor(log(x), 4, 3, [log(3) ; 1/3 ; -1/18 ; 1/81 ; -1/324], 1e-15);
assertTaylor(log(x*x - exp(-x)), 3, 3, [2.1916773 ; 0.6759378 ; -0.1194981 ; 0.0302286], 1e-7);
assertTaylor(log([x*x ; exp(2*x*x) ; 2^x ; x+5]), 4, 3, ...
	[ ...
		log(9),	 2/3,	-1/  9,	2/  81,	-1/  162	; ...
		18,		12,		 2,		0,		 0			; ...
		log(8),	log(2),	 0,		0,		 0			; ...
		log(8),	 1/8,	-1/128,	1/1536,	-1/16384	  ...
	], 1e-15);
assertTaylor(log([x*x ; exp(2*x*x) ; 2^x ; x+5]'), 4, 3, ...
	[ ...
		log(9),	 2/3,	-1/  9,	2/  81,	-1/  162	; ...
		18,		12,		 2,		0,		 0			; ...
		log(8),	log(2),	 0,		0,		 0			; ...
		log(8),	 1/8,	-1/128,	1/1536,	-1/16384	  ...
	]', 1e-15);
assertTaylor([x,-1;2,exp(x)] .* log([log(x),x*x*x ; x-1,exp(exp(x))]), 3, 3, ...
	hypermat([2,2,4], ...
		[ ...
			0.2821435 ;  1.3862944 ; -3.2958369 ; 403.42879 ; ...
			1.0042871 ;  1         ; -1         ; 806.85759 ; ...
			0.0136173 ; -1/4       ;  1/6       ; 806.85759 ; ...
			0.0110758 ;  1/12      ; -1/27      ; 537.90506   ...
		]), 1e-5);

clear x;

printf("Taylor dev. - Exponential and logarithm : all unit tests passed.\n");


// End of script.
