//
// Tests on Taylor development : regular and element-wise exponentiation and logarithm.
//
// Author : Benoit Hamelin
// 2003 Universit de Sherbrooke
//


x = ad_ivar(4);

// Tests on regular pseudo-monomes and pseudo-polynomes.
assertTaylor(x^5, 4, 3, [243 ; 405 ; 270 ; 90 ; 15], 1e-15);
assertTaylor(x ^ (1/2), 4, 3, [sqrt(3) ; inv(2 * sqrt(3)) ; -(3^(-3/2))/8 ; (3^(-5/2))/16 ; -5*(3^(-7/2))/128], 1e-15);
assertTaylor(-2 * x^(-2), 4, 3, [-2/9 ; 4/27 ; -6/81 ; 8/243 ; -10/729], 1e-15);
assertTaylor(-2*x^4 + 5*x^3 - 3*x^2 + 2, 4, 3, [-52 ; -99 ; -66 ; -19 ; -2], 1e-15);
assertTaylor(-x^(4/3) + 2 * x^(-5/2), 3, 3, ...
	[ ...
		-3^(4/3) + 2*3^(-5/2)				; ...
		-4*(3^(1/3))/3 - 5*(3^(-7/2))		; ...
		-2*(3^(-2/3))/9 + 35*(3^(-9/2))/4	; ...
		4*(3^(-5/3))/81 - 105*(3^(-11/2))/8	  ...
	], 1e-8);

// Tests on element-wise pseudo-monomes and pseudo-polynomes.
assertTaylor(x .^ 5, 4, 3, [243 ; 405 ; 270 ; 90 ; 15], 1e-15);
assertTaylor(sqrt(x), 4, 3, [sqrt(3) ; inv(2 * sqrt(3)) ; -(3^(-3/2))/8 ; (3^(-5/2))/16 ; -5*(3^(-7/2))/128], 1e-15);
assertTaylor(-2 .* x .^ (-2), 4, 3, [-2/9 ; 4/27 ; -6/81 ; 8/243 ; -10/729], 1e-15);
assertTaylor(-2 * x .^ 4 + 5 * x .^ 3 - 3 * x .^ 2 + 2, 4, 3, [-52 ; -99 ; -66 ; -19 ; -2], 1e-15);
assertTaylor(- x .^ (4/3) + 2 * x .^ (-5/2), 3, 3, ...
	[ ...
		-3^(4/3) + 2*3^(-5/2)				; ...
		-4*(3^(1/3))/3 - 5*(3^(-7/2))		; ...
		-2*(3^(-2/3))/9 + 35*(3^(-9/2))/4	; ...
		4*(3^(-5/3))/81 - 105*(3^(-11/2))/8	  ...
	], 1e-15);

assertTaylor([x^2-3*x ; x^3 ; exp(x) ; log(2*x) ; x^5 ; log(x)] .^ [1/2 ; 2 ; -1/2 ; 1 ; 0 ; 3], 3, 4, ...
	[ ...
		   2	,             5/4	,       -9/64	,       45/512	; ...
		4096	,          6144		,     3840		,     1280		; ...
		exp(-2)	, -     exp(-2)/2	,  exp(-2)/8	, -exp(-2)/48	; ...
		log(8)	,             1/4	, -      1/32	,        1/192	; ...
		   1	,             0		,        0		,        0		; ...
		log(4)^3,  3*(log(4)^2)/4	, 0.0797603		, -0.0193292	  ...
	], 1e-7);
assertTaylor([x^2-3*x , x^3 , exp(x) , log(2*x) , x^5 , log(x)] .^ [1/2 , 2 , -1/2 , 1 , 0 , 3], 3, 4, ...
	[ ...
		   2	,             5/4	,       -9/64	,       45/512	; ...
		4096	,          6144		,     3840		,     1280		; ...
		exp(-2)	, -     exp(-2)/2	,  exp(-2)/8	, -exp(-2)/48	; ...
		log(8)	,             1/4	, -      1/32	,        1/192	; ...
		   1	,             0		,        0		,        0		; ...
		log(4)^3,  3*(log(4)^2)/4	, 0.0797603		, -0.0193292	  ...
	]', 1e-7);
v1 = [ 2 ; exp(-2) ; 1 ; 4096 ; log(8) ; log(4)^3];
v2 = [ 5/4 ; -exp(-2)/2 ; 0 ; 6144 ; 1/4 ; 3*(log(4)^2)/4];
v3 = [ -9/64 ; exp(-2)/8 ; 0 ; 3840 ; -1/32 ; (6*log(4) - 3*(log(4)^2))/32];
v4 = [ 45/512 ; -exp(-2)/48 ; 0 ; 1280 ; 1/192 ; (1 - 3*log(4) + log(4)^2)/64];
assertTaylor([x^2-3*x , x^3 ; exp(x) , log(2*x) ; x^5 , log(x)] .^ [1/2 , 2 ; -1/2 , 1 ; 0 , 3], 3, 4, ...
	hypermat([3,2,4], [v1 ; v2; v3 ; v4]), 1e-8);


clear x;
clear v1;
clear v2;
clear v3;
clear v4;

printf("Taylor dev. - Monomes : all unit tests passed.\n");


// End of script.
