//
// Tests on Taylor development : element-wise circular trigonometrics.
//
// Author : Benoit Hamelin
// 2004 Universit de Sherbrooke
//


x = ad_ivar(4);

// Tests on sines and cosines.
assertTaylor(sin(x), 4, %pi/3, [sqrt(3)/2 ; 1/2 ; -sqrt(3)/4 ; -1/12 ; sqrt(3)/48], 1e-15);
assertTaylor(cos(x), 4, %pi/3, [1/2 ; -sqrt(3)/2 ; -1/4 ; sqrt(3)/12 ; 1/48], 1e-15);

// Tests on complex sines and cosines.
assertTaylor(sin([x^2 - 2*x ; log(x) ; x - 2 ; %pi/6]), 3, 3, ...
	[ ...
		sin(3)		, 4*cos(3)			, cos(3) - 8*sin(3)					, -4*sin(3) - 32*cos(3)/3			; ...
		sin(log(3))	,   cos(log(3))/3	, -cos(log(3))/18 - sin(log(3))/18	, sin(log(3))/54 + cos(log(3))/162	; ...
		sin(1)		,   cos(1)			, -sin(1)/2							, -cos(1)/6							; ...
		1/2			, 0					, 0									, 0									  ...
	], 1e-15);
assertTaylor(sin([x^2 - 2*x , log(x) , x - 2 , %pi/6]), 3, 3, ...
	[ ...
		sin(3)		, 4*cos(3)			, cos(3) - 8*sin(3)					, -4*sin(3) - 32*cos(3)/3			; ...
		sin(log(3))	,   cos(log(3))/3	, -cos(log(3))/18 - sin(log(3))/18	, sin(log(3))/54 + cos(log(3))/162	; ...
		sin(1)		,   cos(1)			, -sin(1)/2							, -cos(1)/6							; ...
		1/2			, 0					, 0									, 0									  ...
	]', 1e-15);
assertTaylor(sin([x^2 - 2*x , log(x) ; x - 2 , %pi/6]), 3, 3, ...
	hypermat([2,2,4], ...
		[ ...
			sin(3)					;  sin(1)	; sin(log(3)) 						; 1/2	; ...
			4*cos(3)				;  cos(1)	; cos(log(3))/3						; 0		; ...
			cos(3) - 8*sin(3)		; -sin(1)/2	; -cos(log(3))/18 - sin(log(3))/18	; 0		; ...
			-4*sin(3) - 32*cos(3)/3	; -cos(1)/6	; sin(log(3))/54 + cos(log(3))/162	; 0		  ...
		]), 1e-15);
assertTaylor(cos([exp(x) ; -%pi/3 ; -x ; x^3]), 3, 3, ...
	[ ...
		 cos(exp(3))	,	-   exp(3)*sin(exp(3))	, -  exp(3)*sin(exp(3))/2 - exp(6)*cos(exp(3))/2, -exp(3)*sin(exp(3))/6 - exp(6)*cos(exp(3))/2 + exp(9)*sin(exp(3))/6	; ...
		 1/2			,	  0						,  0											,  0 																	; ...
		 cos(3)			,	-   sin(3)				, -  cos(3)/2									,  sin(3)/6																; ...
		 cos(27)		,	-27*sin(27)				, -9*sin(27) - 364.5*cos(27)					, -sin(27) - 243*cos(27) + 3280.5*sin(27)								  ...
	], 1e-12);
assertTaylor(cos([exp(x) , -%pi/3 , -x , x^3]), 3, 3, ...
	[ ...
		 cos(exp(3))	,	-   exp(3)*sin(exp(3))	, -  exp(3)*sin(exp(3))/2 - exp(6)*cos(exp(3))/2, -exp(3)*sin(exp(3))/6 - exp(6)*cos(exp(3))/2 + exp(9)*sin(exp(3))/6	; ...
		 1/2			,	  0						,  0											,  0 																	; ...
		 cos(3)			,	-   sin(3)				, -  cos(3)/2									,  sin(3)/6																; ...
		 cos(27)		,	-27*sin(27)				, -9*sin(27) - 364.5*cos(27)					, -sin(27) - 243*cos(27) + 3280.5*sin(27)								  ...
	]', 1e-12);
assertTaylor(cos([exp(x) , -%pi/3 ; -x , x^3]), 3, 3, ...
	hypermat([2,2,4], ...
		[ ...
			 cos(exp(3))									;  cos(3)	; 1/2	;     cos(27)									; ...
			-exp(3)*sin(exp(3))								; -sin(3)	; 0		; -27*sin(27)									; ...
			-exp(3)*sin(exp(3))/2 - exp(6)*cos(exp(3))/2	; -cos(3)/2	; 0		; - 9*sin(27) - 364.5*cos(27)					; ...
			-exp(3)*sin(exp(3))/6 - exp(6)*cos(exp(3))/2 + exp(9)*sin(exp(3))/6													  ...
															;  sin(3)/6	; 0		; -   sin(27) - 243*cos(27) + 3280.5 * sin(27)	  ...
		]), 1e-12);

assertTaylor(atan(x), 3, 1, [%pi/4 ; 1/2 ; -1/4 ; 1/12], 1e-15);
assertTaylor(atan(sin(x^2) / cos(x^2)), 4, 1, [1 ; 2 ; 1 ; 0 ; 0], 1e-12);

assertTaylor(atan([x^2 ; log(x) ; -x ; sqrt(3)]), 3, 2, ...
	[ ...
		atan(4)		,  4/17				, -47/289								,  1376/14739	; ...
		atan(log(2)),  1/(2+2*log(2)^2)	, -((1+log(2))^2)/(2*(2+2*log(2)^2)^2)	,  0.0733439	; ...
		atan(-2)	, -1/5				,   2/25								, -  11/375		; ...
		%pi/3		,  0				,   0									,     0			  ...
	], 1e-7);
assertTaylor(atan([x^2 , log(x) , -x , sqrt(3)]), 3, 2, ...
	[ ...
		atan(4)		,  4/17				, -47/289								,  1376/14739	; ...
		atan(log(2)),  1/(2+2*log(2)^2)	, -((1+log(2))^2)/(2*(2+2*log(2)^2)^2)	,  0.0733439	; ...
		atan(-2)	, -1/5				,   2/25								, -  11/375		; ...
		%pi/3		,  0				,   0									,     0			  ...
	]', 1e-7);
assertTaylor(atan([x^2 , log(x) ; -x , sqrt(3)]), 3, 2, ...
	hypermat([2,2,4], ...
		[ ...
			      atan(4)	;     atan(-2)	;                 atan(log(2))			; %pi/3	; ...
			    4/17		; - 1/5			;               1/(2+2*log(2)^2)		;   0	; ...
			-  47/289		;   2/25		; -((1+log(2))^2)/(2*(2+2*log(2)^2)^2)	;   0	; ...
			 1376/14739		; -11/375		; 0.0733439								;   0	  ...
		]), 1e-7);


clear x;

printf("Taylor dev. - Trigonometrics : all unit tests passed.\n");


// End of script.
