function [fvec,J]=kowosb(n,m,x,opt)
fvec=[];J=[];
 
// ***********************************************************
// Function [fvec, J]=kowosb(n,m,x,opt)
// Kowalik and Osborne function    [15]
// Dimensions:     n=4   m=11
// Function Definition:
//       f(x)= y(i) - [x1(u^2 +u*x2) / (u^2 + u*x3 + x4)]
// Standard starting point: (0.25,0.39,0.415,0.39)
// Minima of f= 3.07505...10^-4 and
//           f= 1.02734...10^-3  at (inf,-14.07...,-inf,-inf)
// 
// Coded in Matlab   October 1994        PLK
// **********************************************************
 
y = [0.1957,0.1947,0.1735,0.16,0.0844,0.0627;0.0456,0.0342,0.0323,0.0235,0.0246,0]';
 
u = [4,2,1,0.5,0.25,0.167;0.125,0.1,0.0833,0.0714,0.0625,0]';
 
 
 
for i = 1:m
  c1 = u(i)^2+u(i)*x(2);
  c2 = u(i)^2+u(i)*x(3)+x(4);
   
  if opt==1|opt==3 then
    fvec(1,i) = y(i)-x(1)*c1/c2;
  end
   
  if opt==2|opt==3 then
    J(i,1) = -c1/c2;
    J(i,2) = (-x(1)*u(i))/c2;
    J(i,3) = x(1)*c1*(c2^(-2))*u(i);
    J(i,4) = x(1)*c1*(c2^(-2));
  end
end
 
fvec = fvec';
 
if opt<1|opt>3 then
  disp('Error: Option value for KOWOSB.M is either <1 or >3');
end
 
